/* Accessibility.java

	Purpose:
		
	Description:
		
	History:
		11:24 AM 2022/9/19, Created by jumperchen

Copyright (C) 2022 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart;

import java.util.Map;

import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.json.JavaScriptValue;

/**
 * An options for configuring accessibility for the chart. Requires the accessibility module
 * to be loaded. For example,
 * <pre><code>
 * Library.setProperty("org.zkoss.chart.modules.accessibility", "true");
 * </code></pre>
 * @author jumperchen
 * @since 10.2.1.0
 */
public class Accessibility extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		announceNewData,
		customComponents,
		description,
		enabled,
		highContrastTheme,
		keyboardNavigation,
		landmarkVerbosity,
		linkedDescription,
		point,
		screenReaderSection,
		series,
		typeDescription
	}

	/**
	 * Returns an options for announcing new data to screen reader user, or create
	 * a new one.
	 * <p>Default: an instance of {@link AnnounceNewData}
	 */
	public AnnounceNewData getAnnounceNewData() {
		return (AnnounceNewData) getAttr(Attrs.announceNewData,
				AnnounceNewData.class).asValue();
	}

	/**
	 * Sets an options for announcing new data to screen reader user.
	 */
	public void setAnnounceNewData(AnnounceNewData announceNewData) {
		setAttr(Attrs.announceNewData, announceNewData);
	}

	/**
	 * Returns a hook for adding custom components to the accessibility module.
	 * Should be an object mapping component names to instances of classes inheriting
	 * from the Highcharts.AccessibilityComponent base class. Remember to add the component
	 * to the keyboardNavigation.order for the keyboard navigation to be usable.
	 * <p>Default: {@code null}
	 */
	public JavaScriptValue getCustomComponents() {
		return (JavaScriptValue) getAttr(Attrs.customComponents, null).asValue();
	}

	/**
	 * Sets a hook for adding custom components to the accessibility module.
	 * Should be an object mapping component names to instances of classes inheriting
	 * from the Highcharts.AccessibilityComponent base class. Remember to add the component
	 * to the keyboardNavigation.order for the keyboard navigation to be usable.
	 * <p>Default: {@code null}
	 */
	public void setCustomComponents(JavaScriptValue customComponents) {
		setAttr(Attrs.customComponents, customComponents);
	}

	/**
	 * Returns a text description of the chart.
	 * <p>
	 * Note: Prefer using {@code linkedDescription} or {@code caption} instead.
	 * <p>
	 * If the Accessibility module is loaded, this option is included by default
	 * as a long description of the chart in the hidden screen reader information region.
	 *
	 * <p>
	 * Note: Since Highcharts now supports captions and linked descriptions,
	 * it is preferred to define the description using those methods, as a visible
	 * {@code caption/description} benefits all users. If the accessibility.description
	 * option is defined, the linked description is ignored, and the caption is hidden
	 * from screen reader users.
	 * <p>Default: {@code null}
	 */
	public String getDescription() {
		return getAttr(Attrs.description, null).asString();
	}

	/**
	 * Sets a text description of the chart.
	 * <p>
	 * Note: Prefer using {@code linkedDescription} or {@code caption} instead.
	 * <p>
	 * If the Accessibility module is loaded, this option is included by default
	 * as a long description of the chart in the hidden screen reader information region.
	 *
	 * <p>
	 * Note: Since Highcharts now supports captions and linked descriptions,
	 * it is preferred to define the description using those methods, as a visible
	 * {@code caption/description} benefits all users. If the accessibility.description
	 * option is defined, the linked description is ignored, and the caption is hidden
	 * from screen reader users.
	 * <p>Default: {@code null}
	 */
	public void setDescription(String description) {
		setAttr(Attrs.description, description);
	}

	/**
	 * Returns whether to enable accessibility functionality for the chart.
	 * For more information on how to include these features, and why this is
	 * recommended, see Highcharts Accessibility.
	 *
	 * <p>
	 * Highcharts will by default emit a warning to the console if the accessibility
	 * module is not loaded. Setting this option to false will override and silence
	 * the warning.
	 *
	 * <p>
	 * Once the module is loaded, setting this option to false will disable the module
	 * for this chart.
	 *
	 * <p> Default: {@code true}
	 */
	public boolean isEnabled() {
		return getAttr(Attrs.enabled, true).asBoolean();
	}

	/**
	 * Sets whether to enable accessibility functionality for the chart.
	 * For more information on how to include these features, and why this is
	 * recommended, see Highcharts Accessibility.
	 *
	 * <p>
	 * Highcharts will by default emit a warning to the console if the accessibility
	 * module is not loaded. Setting this option to false will override and silence
	 * the warning.
	 *
	 * <p>
	 * Once the module is loaded, setting this option to false will disable the module
	 * for this chart.
	 */
	public void setEnabled(boolean enabled) {
		setAttr(Attrs.enabled, enabled);
	}

	/**
	 * Returns the theme to apply to the chart when Windows High Contrast Mode is detected.
	 * By default, a high contrast theme matching the high contrast system system colors is used.
	 * <p>Default: {@code null}
	 */
	public Map getHighContrastTheme() {
		return (Map) getAttr(Attrs.highContrastTheme, null).asValue();
	}

	/**
	 * Sets the theme to apply to the chart when Windows High Contrast Mode is detected.
	 * By default, a high contrast theme matching the high contrast system system colors is used.
	 * <p>Default: {@code null}
	 */
	public void setHighContrastTheme(Map highContrastTheme) {
		setAttr(Attrs.highContrastTheme, highContrastTheme);
	}

	/**
	 * Returns an options for keyboard navigation, or create a new one.
	 * <p>Default: an instance of {@link KeyboardNavigation}
	 */
	public KeyboardNavigation getKeyboardNavigation() {
		return (KeyboardNavigation) getAttr(Attrs.keyboardNavigation, KeyboardNavigation.class).asValue();
	}

	/**
	 * Sets an options for keyboard navigation, or create a new one.
	 */
	public void setKeyboardNavigation(KeyboardNavigation keyboardNavigation) {
		setAttr(Attrs.keyboardNavigation, keyboardNavigation);
	}

	/**
	 * Returns the amount of landmarks/regions to create for screen reader users.
	 * More landmarks can make navigation with screen readers easier, but can be
	 * distracting if there are lots of charts on the page.
	 * <p>Three modes are available:
	 * <ul><li>
	 * {@code "all"}: Adds regions for all series, legend, information region.
	 * </li>
	 * <li>
	 * {@code "one"}: Adds a single landmark per chart.
	 * </li>
	 * <li>
	 * {@code "disabled"}: No landmarks are added.
	 * </li>
	 * </ul>
	 * <p>Default: {@code "all").
	 */
	public String getLandmarkVerbosity() {
		return getAttr(Attrs.landmarkVerbosity, "all").asString();
	}
	/**
	 * Sets the amount of landmarks/regions to create for screen reader users.
	 * More landmarks can make navigation with screen readers easier, but can be
	 * distracting if there are lots of charts on the page.
	 * <p>Three modes are available:
	 * <ul><li>
	 * {@code "all"}: Adds regions for all series, legend, information region.
	 * </li>
	 * <li>
	 * {@code "one"}: Adds a single landmark per chart.
	 * </li>
	 * <li>
	 * {@code "disabled"}: No landmarks are added.
	 * </li>
	 * </ul>
	 * <p>Default: {@code "all").
	 */
	public void setLandmarkVerbosity(String landmarkVerbosity) {
		if (!("all".equals(landmarkVerbosity) || "one".equals(landmarkVerbosity)
			|| "disabled".equals(landmarkVerbosity))) {
			throw new IllegalArgumentException("Unknown value [" + landmarkVerbosity + "]");
		}
		setAttr(Attrs.landmarkVerbosity, landmarkVerbosity);
	}

	/**
	 * Returns the linking of the chart to an HTML element describing the contents of the chart.
	 */
	public String getLinkedDescription() {
		return getAttr(Attrs.linkedDescription,
				"*[data-highcharts-chart=\"{index}\"] + .highcharts-description").asString();
	}

	/**
	 * Link the chart to an HTML element describing the contents of the chart.
	 *
	 * <p>
	 * It is always recommended to describe charts using visible text, to improve
	 * SEO as well as accessibility for users with disabilities. This option lets
	 * an HTML element with a description be linked to the chart, so that screen
	 * reader users can connect the two.
	 *
	 * <p>
	 * By setting this option to a string, Highcharts runs the string as an HTML
	 * selector query on the entire document. If there is only a single match,
	 * this element is linked to the chart. The content of the linked element
	 * will be included in the chart description for screen reader users.
	 *
	 * <p>
	 * By default, the chart looks for an adjacent sibling element with the
	 * highcharts-description class.
	 *
	 * <p>
	 * The feature can be disabled by setting the option to an empty string,
	 * or overridden by providing the {@code accessibility.description} option.
	 * Alternatively, the HTML element to link can be passed in directly as an HTML node.
	 *
	 * <p>
	 * If you need the description to be part of the exported image, consider using
	 * the caption feature.
	 *
	 * <p>
	 * If you need the description to be hidden visually, use the {@code accessibility.description}
	 * option.
	 *
	 * <p>
	 * Defaults to {@code *[data-highcharts-chart="{index}"] + .highcharts-description.}
	 */
	public void setLinkedDescription(String linkedDescription) {
		setAttr(Attrs.linkedDescription, linkedDescription);
	}

	/**
	 * Returns options for descriptions of individual data points, or create a
	 * new one.
	 * <p>Default: an instance  of {@link AccessibilityPoint}
	 */
	public AccessibilityPoint getPoint() {
		return (AccessibilityPoint) getAttr(Attrs.point,
				AccessibilityPoint.class).asValue();
	}

	/**
	 * Sets options for descriptions of individual data points.
	 */
	public void setPoint(AccessibilityPoint point) {
		setAttr(Attrs.point, point);
	}

	/**
	 * Returns accessibility options for the screen reader information sections
	 * added before and after the chart, or create a new one.
	 * <p>Default: an instance of {@link ScreenReaderSection}</p>
	 */
	public ScreenReaderSection getScreenReaderSection() {
		return (ScreenReaderSection) getAttr(Attrs.screenReaderSection,
				ScreenReaderSection.class).asValue();
	}

	/**
	 * Sets accessibility options for the screen reader information sections
	 * added before and after the chart.
	 */
	public void setScreenReaderSection(
			ScreenReaderSection screenReaderSection) {
		setAttr(Attrs.screenReaderSection, screenReaderSection);
	}

	/**
	 * Returns accessibility options global to all data series. Individual series can also
	 * have specific {@link org.zkoss.chart.plotOptions.SeriesPlotOptions#getAccessibility()} set.
	 * If null, create a new one.
	 *
	 * <p>Default: an instance of {@link AccessibilitySeries}
	 */
	public AccessibilitySeries getSeries() {
		return (AccessibilitySeries) getAttr(Attrs.series,
				AccessibilitySeries.class).asValue();
	}

	/**
	 * Sets accessibility options global to all data series. Individual series can also
	 * have specific {@link org.zkoss.chart.plotOptions.SeriesPlotOptions#getAccessibility()} set.
	 */
	public void setSeries(AccessibilitySeries series) {
		setAttr(Attrs.series, series);
	}

	/**
	 * Returns a text description of the chart type.
	 * <p>Default: {@code null}
	 */
	public String getTypeDescription() {
		return getAttr(Attrs.typeDescription, null).asString();
	}

	/**
	 * Sets a text description of the chart type.
	 *
	 * <p>
	 * If the Accessibility module is loaded, this will be included in the description
	 * of the chart in the screen reader information region.
	 *
	 * <p>
	 * Highcharts will by default attempt to guess the chart type, but for more complex
	 * charts it is recommended to specify this property for clarity.
	 */
	public void setTypeDescription(String typeDescription) {
		setAttr(Attrs.typeDescription, typeDescription);
	}
}