/* AccessibilitySeries.java

	Purpose:
		
	Description:
		
	History:
		4:07 PM 2022/9/19, Created by jumperchen

Copyright (C) 2022 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart;

import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.json.JavaScriptValue;

/**
 * Accessibility options global to all data series. Individual series can also
 * have specific accessibility options set.
 * @author jumperchen
 * @since 10.2.1.0
 */
public class AccessibilitySeries extends Optionable {

	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		describeSingleSeries,
		descriptionFormat,
		descriptionFormatter,
		pointDescriptionEnabledThreshold
	}

	/**
	 * Returns whether or not to add series descriptions to charts with a single series.
	 * <p>Default: {@code false}
	 */
	public boolean isDescribeSingleSeries() {
		return getAttr(Attrs.describeSingleSeries, false).asBoolean();
	}

	/**
	 * Sets whether or not to add series descriptions to charts with a single series.
	 */
	public void setDescribeSingleSeries(boolean describeSingleSeries) {
		setAttr(Attrs.describeSingleSeries, describeSingleSeries);
	}

	/**
	 * Returns the format to use for describing the data series group to assistive
	 * technology - including screen readers.
	 * <p>Default: {@code "{seriesDescription}{authorDescription}{axisDescription}"}
	 */
	public String getDescriptionFormat() {
		return getAttr(Attrs.descriptionFormat,
				"{seriesDescription}{authorDescription}{axisDescription}").asString();
	}

	/**
	 * Sets the format to use for describing the data series group to assistive
	 * technology - including screen readers.
	 *
	 * <p>
	 * The series context and its subproperties are available under the variable
	 * {@code {series}}, for example {@code {series.name}} for the series name,
	 * and {@code {series.points.length}} for the number of data points.
	 *
	 * <p>
	 * The chart context and its subproperties are available under the variable
	 * {@code {chart}}, for example {@code {chart.series.length}} for the number
	 * of series in the chart.
	 *
	 * <p>
	 * {@code {seriesDescription}} refers to the automatic description of the series
	 * type and number of points added by Highcharts by default.
	 * {@code {authorDescription}} refers to the description added in {@code series.description}
	 * if one is present. {@code {axisDescription}} refers to the description added
	 * if the chart has multiple X or Y axes.
	 *
	 * <p>
	 * Note that if {@code series.descriptionFormatter} is declared it will take
	 * precedence, and this option will be overridden.
	 *
	 * <p>Default: {@code "{seriesDescription}{authorDescription}{axisDescription}"}
	 */
	public void setDescriptionFormat(String descriptionFormat) {
		setAttr(Attrs.descriptionFormat, descriptionFormat);
	}

	/**
	 * Returns the formatter function to use instead of the default for series
	 * descriptions.
	 * <p>Default: {@code null}
	 */
	public JavaScriptValue getDescriptionFormatter() {
		return (JavaScriptValue) getAttr(Attrs.descriptionFormatter,
				null).asValue();
	}

	/**
	 * Sets the formatter function to use instead of the default for series descriptions.
	 * Receives one argument, {@code series}, referring to the series to describe.
	 * Should return a string with the description of the series for a screen reader user.
	 * If false is returned, the default formatter will be used for that series.
	 */
	public void setDescriptionFormatter(JavaScriptValue descriptionFormatter) {
		setAttr(Attrs.descriptionFormatter, descriptionFormatter);
	}

	/**
	 * Returns whether to expose information about individual points to screen readers.
	 * <p>Sets to false to disable.
	 * <p>Default: {@code true}
	 */
	public boolean isPointDescriptionEnabledThreshold() {
		return getAttr(Attrs.pointDescriptionEnabledThreshold, true).asBoolean();
	}

	/**
	 * When a series contains more points than this, we no longer expose information
	 * about individual points to screen readers.
	 * <p>Sets to false to disable.
	 * <p>Default: {@code true}
	 */
	public void setPointDescriptionEnabledThreshold(
			boolean pointDescriptionEnabledThreshold) {
		setAttr(Attrs.pointDescriptionEnabledThreshold,
				pointDescriptionEnabledThreshold);
	}

	/**
	 * When a series contains more points than this, we no longer expose information
	 * about individual points to screen readers.
	 * <p>Default: {@code 200}
	 */
	public Number getPointDescriptionEnabledThreshold() {
		return getAttr(Attrs.pointDescriptionEnabledThreshold, 200).asNumber();
	}

	/**
	 * When a series contains more points than this, we no longer expose information
	 * about individual points to screen readers.
	 * <p>Default: {@code 200}
	 */
	public void setPointDescriptionEnabledThreshold(
			Number pointDescriptionEnabledThreshold) {
		setAttr(Attrs.pointDescriptionEnabledThreshold,
				pointDescriptionEnabledThreshold);
	}
}
