/* Annotation.java

	Purpose:
		
	Description:
		
	History:
		Wed Nov 06 18:26:30 CST 2019, Created by rudyhuang

Copyright (C) 2019 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.json.JSONValue;
import org.zkoss.lang.Generics;

/**
 * A basic type of an annotation.
 * It allows to add custom labels or shapes.
 * The items can be tied to points, axis coordinates or chart pixel coordinates.
 *
 * @author rudyhuang
 * @since 7.2.1.0
 */
public class Annotation extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		controlPointOptions,
		crop,
		draggable,
		events,
		id,
		labelOptions,
		labels,
		shapeOptions,
		shapes,
		visible,
		zIndex
	}

	/**
	 * Returns whether to hide the part of the annotation that is outside the plot area.
	 * <p>Default: {@code true}
	 * @since 10.2.1.0
	 */
	public boolean isCrop() {
		return getAttr(Attrs.crop, true).asBoolean();
	}

	/**
	 * Sets whether to hide the part of the annotation that is outside the plot area.
	 * <p>Default: {@code true}
	 * @since 10.2.1.0
	 */
	public void setCrop(boolean crop) {
		setAttr(Attrs.crop, crop, true);
	}

	/**
	 * Allow an annotation to be draggable by a user.
	 * Possible values are "x", "xy", "y" and "" (disabled).
	 * <p>
	 * Default: xy
	 */
	public String getDraggable() {
		return getAttr(Attrs.draggable, "xy").asString();
	}

	/**
	 * Allow an annotation to be draggable by a user.
	 * Possible values are "x", "xy", "y" and "" (disabled).
	 */
	public void setDraggable(String draggable) {
		setAttr(Attrs.draggable, draggable, "xy");
	}

	/**
	 * Returns an ID for an annotation.
	 * Can be user later when removing an annotation in removeAnnotation(id) method.
	 * <p>
	 * Default: null
	 */
	public Object getId() {
		return getAttr(Attrs.id, null).asValue();
	}

	/**
	 * Sets an ID for an annotation.
	 */
	public void setId(String id) {
		setAttr(Attrs.id, id);
	}

	/**
	 * Sets an ID for an annotation.
	 */
	public void setId(Number id) {
		setAttr(Attrs.id, id);
	}

	/**
	 * Returns options for annotation's labels.
	 * Each label inherits options from the labelOptions object.
	 * An option from the labelOptions can be overwritten by config for a specific label.
	 */
	public AnnotationLabelOptions getLabelOptions() {
		AnnotationLabelOptions labelOptions = (AnnotationLabelOptions) getAttr(Attrs.labelOptions);
		if (labelOptions == null) {
			labelOptions = new AnnotationLabelOptions();
			setLabelOptions(labelOptions);
		}
		return labelOptions;
	}

	/**
	 * Sets options for annotation's labels.
	 * Each label inherits options from the labelOptions object.
	 * An option from the labelOptions can be overwritten by config for a specific label.
	 */
	public void setLabelOptions(AnnotationLabelOptions labelOptions) {
		setAttr(Attrs.labelOptions, labelOptions);
	}

	/**
	 * Returns the list of labels for the annotation.
	 */
	public List<AnnotationLabel> getLabels() {
		return Generics.cast(getAttr(Attrs.labels));
	}

	/**
	 * Sets a list of labels for the annotation.
	 * For options that apply to multiple labels, they can be added to the {@link #setLabelOptions}.
	 */
	public void setLabels(List<AnnotationLabel> labels) {
		setAttr(Attrs.labels, labels);
	}

	/**
	 * Sets an array of labels for the annotation.
	 */
	public void setLabels(AnnotationLabel... labels) {
		// use ArrayList instead of fixed ArrayList
		setLabels(new ArrayList<AnnotationLabel>(Arrays.asList(labels)));
	}

	/**
	 * Returns options for annotation's shapes.
	 * Each shape inherits options from the shapeOptions object.
	 * An option from the shapeOptions can be overwritten by config for a specific shape.
	 */
	public AnnotationShapeOptions getShapeOptions() {
		AnnotationShapeOptions shapeOptions = (AnnotationShapeOptions) getAttr(Attrs.shapeOptions);
		if (shapeOptions == null) {
			shapeOptions = new AnnotationShapeOptions();
			setShapeOptions(shapeOptions);
		}
		return shapeOptions;
	}

	/**
	 * Sets options for annotation's shapes.
	 * Each shape inherits options from the shapeOptions object.
	 * An option from the shapeOptions can be overwritten by config for a specific shape.
	 */
	public void setShapeOptions(AnnotationShapeOptions shapeOptions) {
		setAttr(Attrs.shapeOptions, shapeOptions);
	}

	/**
	 * Returns the list of shapes for the annotation.
	 */
	public List<AnnotationShape> getShapes() {
		return Generics.cast(getAttr(Attrs.shapes, null).asValue());
	}

	/**
	 * Sets a list of shapes for the annotation.
	 * For options that apply to multiple shapes, they can be added to the {@link #setShapeOptions}.
	 */
	public void setShapes(List<AnnotationShape> shapes) {
		setAttr(Attrs.shapes, shapes);
	}

	/**
	 * Sets an array of shapes for the annotation.
	 */
	public void setShapes(AnnotationShape... shapes) {
		// use ArrayList instead of fixed ArrayList
		setShapes(new ArrayList<AnnotationShape>(Arrays.asList(shapes)));
	}

	/**
	 * Returns whether the annotation is visible.
	 * <p>
	 * Default: true
	 */
	public boolean isVisible() {
		return getAttr(Attrs.visible, true).asBoolean();
	}

	/**
	 * Sets whether the annotation is visible.
	 */
	public void setVisible(boolean visible) {
		setAttr(Attrs.visible, visible, true);
	}

	/**
	 * Returns the Z index of the annotation.
	 * <p>
	 * Default: 6
	 */
	public Number getZIndex() {
		return getAttr(Attrs.zIndex, 6).asNumber();
	}

	/**
	 * Sets the Z index of the annotation.
	 */
	public void setZIndex(Number zIndex) {
		setAttr(Attrs.zIndex, zIndex, 6);
	}

	/**
	 * This object defines the point to which the label or shape will be connected.
	 * It can be either the point which exists in the series - it is
	 * referenced by the point's id - or a new point with defined x, y
	 * properties and optionally axes.
	 *
	 * @author rudyhuang
	 * @since 7.2.1.0
	 */
	public static class Point extends Optionable {
		private enum Attrs implements PlotAttribute, DynamicalAttribute {
			x, xAxis, y, yAxis,
			pointId // internal
		}

		/**
		 * Default constructor.
		 */
		public Point() {
		}

		/**
		 * Constructs with a point id.
		 */
		public Point(String pointId) {
			setAttr(Attrs.pointId, pointId);
		}

		/**
		 * Constructs with x and y position.
		 */
		public Point(Number x, Number y) {
			setX(x);
			setY(y);
		}

		/**
		 * Constructs with x and y position and indexes of xAxis and yAxis.
		 */
		public Point(Number x, Number y, Number xAxis, Number yAxis) {
			setX(x);
			setY(y);
			setXAxis(xAxis);
			setYAxis(yAxis);
		}

		/**
		 * Returns the x position of the point.
		 * Units can be either in axis or chart pixel coordinates.
		 */
		public Number getX() {
			return getAttr(Attrs.x, null).asNumber();
		}

		/**
		 * Sets the x position of the point.
		 * Units can be either in axis or chart pixel coordinates.
		 */
		public void setX(Number x) {
			setAttr(Attrs.x, x);
		}

		/**
		 * Returns which xAxis the point is connected to.
		 * It refers to either the axis id or the index of the axis in the xAxis array.
		 * If the option is not configured or the axis is not found the point's x coordinate refers to the chart pixels.
		 * <p>
		 * Default: null
		 */
		public Object getXAxis() {
			return getAttr(Attrs.xAxis);
		}

		/**
		 * Sets the index of which axis in the xAxis array the point is connected to.
		 * If the option is not configured or the axis is not found the point's x coordinate refers to the chart pixels.
		 */
		public void setXAxis(Number xAxis) {
			setAttr(Attrs.xAxis, xAxis);
		}

		/**
		 * Sets the id of which xAxis the point is connected to.
		 * If the option is not configured or the axis is not found the point's x coordinate refers to the chart pixels.
		 */
		public void setXAxis(String xAxis) {
			setAttr(Attrs.xAxis, xAxis);
		}

		/**
		 * Returns the y position of the point.
		 * Units can be either in axis or chart pixel coordinates.
		 */
		public Number getY() {
			return getAttr(Attrs.y, null).asNumber();
		}

		/**
		 * Sets the y position of the point.
		 * Units can be either in axis or chart pixel coordinates.
		 */
		public void setY(Number y) {
			setAttr(Attrs.y, y);
		}

		/**
		 * Returns which yAxis the point is connected to.
		 * It refers to either the axis id or the index of the axis in the yAxis array.
		 * If the option is not configured or the axis is not found the point's y coordinate refers to the chart pixels.
		 * <p>
		 * Default: null
		 */
		public Object getYAxis() {
			return getAttr(Attrs.yAxis);
		}

		/**
		 * Sets the index of which axis in the yAxis array the point is connected to.
		 * If the option is not configured or the axis is not found the point's y coordinate refers to the chart pixels.
		 */
		public void setYAxis(Number yAxis) {
			setAttr(Attrs.yAxis, yAxis);
		}

		/**
		 * Sets the id of which yAxis the point is connected to.
		 * If the option is not configured or the axis is not found the point's y coordinate refers to the chart pixels.
		 */
		public void setYAxis(String yAxis) {
			setAttr(Attrs.yAxis, yAxis);
		}

		public String toJSONString() {
			String pointId = getAttr(Attrs.pointId, null).asString();
			if (pointId != null) {
				return JSONValue.toJSONString(pointId);
			} else {
				return super.toJSONString();
			}
		}
	}
}
