/* Data.java

        Purpose:
                
        Description:
                
        History:
                Mon Oct 16 16:26:26 CST 2023, Created by jamson

Copyright (C) 2023 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart;

import java.util.List;

import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.json.JavaScriptValue;
import org.zkoss.lang.Generics;

/**
 * The Data module provides a simplified interface for adding data to a chart from sources like CVS, HTML tables or grid views. See also the tutorial article on the Data module.
 * <p>It requires the modules/data.js file to be loaded.
 * <p>Please note that the default way of adding data in Highcharts, without the need of a module, is through the series.type.data option.
 * @author Jamson Chan
 * @since 11.2.0.0
 */
public class Data extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		beforeParse,
		columns,
		columnsURL,
		complete,
		csv,
		csvURL,
		dataRefreshRate,
		dateFormat,
		decimalPoint,
		enablePolling,
		endColumn,
		endRow,
		firstRowAsNames,
		googleAPIKey,
		googleSpreadsheetKey,
		googleSpreadsheetRange,
		googleSpreadsheetWorksheet,
		itemDelimiter,
		lineDelimiter,
		parsed,
		parseDate,
		rows,
		rowsURL,
		seriesMapping, // TODO
		startColumn,
		startRow,
		switchRowsAndColumns,
		table
	}

	/**
	 * Returns a callback function to modify the CSV
	 * before parsing it. Return the modified string.
	 * <p>Default: {@code null}
	 */
	public JavaScriptValue getBeforeParse() {
		return (JavaScriptValue) getAttr(Attrs.beforeParse, null).asValue();
	}

	/**
	 * Sets a callback function to modify the CSV
	 * before parsing it. Return the modified string.
	 * <p>Default: {@code null}
	 */
	public void setBeforeParse(JavaScriptValue beforeParse) {
		setAttr(Attrs.beforeParse, beforeParse, null);
	}

	/**
	 * Returns a two-dimensional array representing the
	 * input data on tabular form. This input can be used
	 * when the data is already parsed, for example from
	 * a grid view component. Each cell can be a string or
	 * number. If not switchRowsAndColumns is set, the
	 * columns are interpreted as series.
	 * <p>Default: {@code null}
	 */
	public List<List<Object>> getColumns() {
		return Generics.cast(getAttr(Attrs.columns, null).asValue());
	}

	/**
	 * Sets a two-dimensional array representing the
	 * input data on tabular form. This input can be used
	 * when the data is already parsed, for example from
	 * a grid view component. Each cell can be a string or
	 * number. If not switchRowsAndColumns is set, the
	 * columns are interpreted as series.
	 * <p>Default: {@code null}
	 */
	public void setColumns(List<List<Object>> columns) {
		setAttr(Attrs.columns, columns, null);
	}

	/**
	 * Returns a URL to a remote JSON dataset, structured as a
	 * column array. Will be fetched when the chart is created using Ajax.
	 * <p>Default: {@code null}
	 */
	public String getColumnsURL() {
		return getAttr(Attrs.columnsURL, null).asString();
	}

	/**
	 * Sets a URL to a remote JSON dataset, structured as a
	 * column array. Will be fetched when the chart is created using Ajax.
	 * <p>Default: {@code null}
	 */
	public void setColumnsURL(String columnsURL) {
		setAttr(Attrs.columnsURL, columnsURL, null);
	}

	/**
	 * Returns the callback that is evaluated when the data is finished
	 * loading, optionally from an external source, and parsed. The first
	 * argument passed is a finished chart options object, containing the
	 * series. These options can be extended with additional options and
	 * passed directly to the chart constructor.
	 * <p>Default: {@code null}
	 */
	public JavaScriptValue getComplete() {
		return (JavaScriptValue) getAttr(Attrs.complete, null).asValue();
	}

	/**
	 * Sets the callback that is evaluated when the data is finished
	 * loading, optionally from an external source, and parsed. The first
	 * argument passed is a finished chart options object, containing the
	 * series. These options can be extended with additional options and
	 * passed directly to the chart constructor.
	 * <p>Default: {@code null}
	 */
	public void setComplete(JavaScriptValue complete) {
		setAttr(Attrs.complete, complete, null);
	}

	/**
	 * Returns a comma delimited string to be parsed. Related options
	 * are startRow, endRow, startColumn and endColumn to delimit what
	 * part of the table is used. The lineDelimiter and itemDelimiter
	 * options define the CSV delimiter formats.
	 * <p>The built-in CSV parser doesn't support all flavours of CSV,
	 * so in some cases it may be necessary to use an external CSV parser.
	 * See this example of parsing CSV through the MIT licensed Papa Parse library.
	 * <p>Default: {@code null}
	 */
	public String getCsv() {
		return getAttr(Attrs.csv, null).asString();
	}

	/**
	 * Sets a comma delimited string to be parsed. Related options
	 * are startRow, endRow, startColumn and endColumn to delimit what
	 * part of the table is used. The lineDelimiter and itemDelimiter
	 * options define the CSV delimiter formats.
	 * <p>The built-in CSV parser doesn't support all flavours of CSV,
	 * so in some cases it may be necessary to use an external CSV parser.
	 * See this example of parsing CSV through the MIT licensed Papa Parse library.
	 * <p>Default: {@code null}
	 */
	public void setCsv(String csv) {
		setAttr(Attrs.csv, csv, null);
	}

	/**
	 * Returns an URL to a remote CSV dataset.
	 * Will be fetched when the chart is created using Ajax.
	 * <p>Default: {@code null}
	 */
	public String getCsvURL() {
		return getAttr(Attrs.csvURL, null).asString();
	}

	/**
	 * Sets an URL to a remote CSV dataset.
	 * Will be fetched when the chart is created using Ajax.
	 * <p>Default: {@code null}
	 */
	public void setCsvURL(String csvURL) {
		setAttr(Attrs.csvURL, csvURL, null);
	}

	/**
	 * Returns the refresh rate for data polling when importing
	 * remote dataset by setting <code>data.csvURL</code>, <code>data.rowsURL</code>,
	 * <code>data.columnsURL</code>, or <code>data.googleSpreadsheetKey</code>.
	 * <p>Note that polling must be enabled by setting data.enablePolling to true.
	 * <p>The value is the number of seconds between pollings. It cannot be set to less than 1 second.
	 * <p>Default: {@code 1}
	 */
	public Number getDataRefreshRate() {
		return getAttr(Attrs.dataRefreshRate, 1).asNumber();
	}

	/**
	 * Sets the refresh rate for data polling when importing
	 * remote dataset by setting <code>data.csvURL</code>, <code>data.rowsURL</code>,
	 * <code>data.columnsURL</code>, or <code>data.googleSpreadsheetKey</code>.
	 * <p>Note that polling must be enabled by setting data.enablePolling to true.
	 * <p>The value is the number of seconds between pollings. It cannot be set to less than 1 second.
	 * <p>Default: {@code 1}
	 */
	public void setDataRefreshRate(Number dataRefreshRate) {
		setAttr(Attrs.dataRefreshRate, dataRefreshRate, 1);
	}

	/**
	 * Returns which of the predefined date formats in
	 * Date.prototype.dateFormats to use to parse date values.
	 * <p>Defaults to a best guess based on what format
	 * gives valid and ordered dates. Valid options include:
	 * YYYY/mm/dd, dd/mm/YYYY, mm/dd/YYYY, dd/mm/YY, mm/dd/YY.
	 * <p>Default: {@code null}
	 */
	public String getDateFormat() {
		return getAttr(Attrs.dateFormat, null).asString();
	}

	/**
	 * Sets which of the predefined date formats in
	 * Date.prototype.dateFormats to use to parse date values.
	 * <p>Defaults to a best guess based on what format
	 * gives valid and ordered dates. Valid options include:
	 * YYYY/mm/dd, dd/mm/YYYY, mm/dd/YYYY, dd/mm/YY, mm/dd/YY.
	 * <p>Default: {@code null}
	 */
	public void setDateFormat(String dateFormat) {
		setAttr(Attrs.dateFormat, dateFormat, null);
	}

	/**
	 * Returns the decimal point used for parsing numbers in the CSV.
	 * <p>If both this and data.delimiter is set to undefined, the
	 * parser will attempt to deduce the decimal point automatically.
	 * <p>Default: {@code "."}
	 */
	public String getDecimalPoint() {
		return getAttr(Attrs.decimalPoint, ".").asString();
	}

	/**
	 * Sets the decimal point used for parsing numbers in the CSV.
	 * <p>If both this and data.delimiter is set to undefined, the
	 * parser will attempt to deduce the decimal point automatically.
	 * <p>Default: {@code "."}
	 */
	public void setDecimalPoint(String decimalPoint) {
		setAttr(Attrs.decimalPoint, decimalPoint, ".");
	}

	/**
	 * Returns enables automatic refetching of remote datasets every n seconds
	 * (defined by setting <code>data.dataRefreshRate</code>).
	 * <p>Only works when either <code>data.csvURL</code>, <code>data.rowsURL</code>,
	 * <code>data.columnsURL</code>, or <code>data.googleSpreadsheetKey</code>.
	 * <p>Default: {@code false}
	 */
	public boolean isEnablePolling() {
		return getAttr(Attrs.enablePolling, false).asBoolean();
	}

	/**
	 * Sets enables automatic refetching of remote datasets every n seconds
	 * (defined by setting <code>data.dataRefreshRate</code>).
	 * <p>Only works when either <code>data.csvURL</code>, <code>data.rowsURL</code>,
	 * <code>data.columnsURL</code>, or <code>data.googleSpreadsheetKey</code>.
	 * <p>Default: {@code false}
	 */
	public void setEnablePolling(Boolean enablePolling) {
		setAttr(Attrs.enablePolling, enablePolling, false);
	}

	/**
	 * Returns in tabular input data, the last column (indexed by 0) to use.
	 * <p>Defaults to the last column containing data.
	 * <p>Default: {@code null}
	 */
	public Number getEndColumn() {
		return getAttr(Attrs.endColumn, null).asNumber();
	}

	/**
	 * Sets in tabular input data, the last column (indexed by 0) to use.
	 * <p>Defaults to the last column containing data.
	 * <p>Default: {@code null}
	 */
	public void setEndColumn(Number endColumn) {
		setAttr(Attrs.endColumn, endColumn, null);
	}

	/**
	 * Returns in tabular input data, the last row (indexed by 0) to use.
	 * <p>Defaults to the last row containing data.
	 * <p>Default: {@code null}
	 */
	public Number getEndRow() {
		return getAttr(Attrs.endRow, null).asNumber();
	}

	/**
	 * Sets in tabular input data, the last row (indexed by 0) to use.
	 * <p>Defaults to the last row containing data.
	 * <p>Default: {@code null}
	 */
	public void setEndRow(Number endRow) {
		setAttr(Attrs.endRow, endRow, null);
	}

	/**
	 * Returns whether to use the first row in the data set as series names.
	 * <p>Default: {@code true}
	 */
	public boolean isFirstRowAsNames() {
		return getAttr(Attrs.firstRowAsNames, true).asBoolean();
	}

	/**
	 * Sets whether to use the first row in the data set as series names.
	 * <p>Default: {@code true}
	 */
	public void setFirstRowAsNames(Boolean firstRowAsNames) {
		setAttr(Attrs.firstRowAsNames, firstRowAsNames, true);
	}

	/**
	 * Returns the Google Spreadsheet API key required for access generated at
	 * API Services / Credentials. See a comprehensive tutorial for setting up
	 * the key at the Hands-On Data Visualization book website.
	 * <p>Default: {@code null}
	 */
	public String getGoogleAPIKey() {
		return getAttr(Attrs.googleAPIKey, null).asString();
	}

	/**
	 * Sets the Google Spreadsheet API key required for access generated at
	 * API Services / Credentials. See a comprehensive tutorial for setting up
	 * the key at the Hands-On Data Visualization book website.
	 * <p>Default: {@code null}
	 */
	public void setGoogleAPIKey(String googleAPIKey) {
		setAttr(Attrs.googleAPIKey, googleAPIKey, null);
	}

	/**
	 * Returns the key or spreadsheetId value for a Google Spreadsheet to load.
	 * See developers.google.com for how to find the spreadsheetId.
	 * <p>In order for Google Sheets to load, a valid googleAPIKey must also be given.
	 * <p>Default: {@code null}
	 */
	public String getGoogleSpreadsheetKey() {
		return getAttr(Attrs.googleSpreadsheetKey, null).asString();
	}

	/**
	 * Sets the key or spreadsheetId value for a Google Spreadsheet to load.
	 * See developers.google.com for how to find the spreadsheetId.
	 * <p>In order for Google Sheets to load, a valid googleAPIKey must also be given.
	 * <p>Default: {@code null}
	 */
	public void setGoogleSpreadsheetKey(String googleSpreadsheetKey) {
		setAttr(Attrs.googleSpreadsheetKey, googleSpreadsheetKey, null);
	}

	/**
	 * Returns the Google Spreadsheet range to use in combination with <code>googleSpreadsheetKey</code>.
	 * <code>See developers.google.com</code> for details.
	 * <p>If given, it takes precedence over <code>startColumn</code>, <code>endColumn</code>,
	 * <code>startRow</code> and <code>endRow</code>.
	 * <p>Default: {@code null}
	 */
	public String getGoogleSpreadsheetRange() {
		return getAttr(Attrs.googleSpreadsheetRange, null).asString();
	}

	/**
	 * Sets the Google Spreadsheet range to use in combination with <code>googleSpreadsheetKey</code>.
	 * <code>See developers.google.com</code> for details.
	 * <p>If given, it takes precedence over <code>startColumn</code>, <code>endColumn</code>,
	 * <code>startRow</code> and <code>endRow</code>.
	 * <p>Default: {@code null}
	 */
	public void setGoogleSpreadsheetRange(String googleSpreadsheetRange) {
		setAttr(Attrs.googleSpreadsheetRange, googleSpreadsheetRange, null);
	}

	/**
	 * Returns item or cell delimiter for parsing CSV. Defaults to the
	 * tab character <code>\t</code> if a tab character is found in the
	 * CSV string, if not it defaults to <code>,</code>.
	 * <p>If this is set to false or undefined, the parser will attempt
	 * to deduce the delimiter automatically.
	 * <p>Default: {@code null}
	 */
	public String getItemDelimiter() {
		return getAttr(Attrs.itemDelimiter, null).asString();
	}

	/**
	 * Sets item or cell delimiter for parsing CSV. Defaults to the
	 * tab character <code>\t</code> if a tab character is found in the
	 * CSV string, if not it defaults to <code>,</code>.
	 * <p>If this is set to false or undefined, the parser will attempt
	 * to deduce the delimiter automatically.
	 * <p>Default: {@code null}
	 */
	public void setItemDelimiter(String itemDelimiter) {
		setAttr(Attrs.itemDelimiter, itemDelimiter, null);
	}

	/**
	 * Returns line delimiter for parsing CSV.
	 * <p>Default: {@code "\n"}
	 */
	public String getLineDelimiter() {
		return getAttr(Attrs.lineDelimiter, "\n").asString();
	}

	/**
	 * Sets line delimiter for parsing CSV.
	 * <p>Default: {@code "\n"}
	 */
	public void setLineDelimiter(String lineDelimiter) {
		setAttr(Attrs.lineDelimiter, lineDelimiter, "\n");
	}

	/**
	 * Returns a callback function to access the parsed columns, the two-dimentional
	 * input data array directly, before they are interpreted into series data and
	 * categories. Return <code>false</code> to stop completion, or call
	 * <code>this.complete()</code> to continue async.
	 * <p>Default: {@code null}
	 */
	public JavaScriptValue getParsed() {
		return (JavaScriptValue) getAttr(Attrs.parsed, null).asValue();
	}

	/**
	 * Sets a callback function to access the parsed columns, the two-dimentional
	 * input data array directly, before they are interpreted into series data and
	 * categories. Return <code>false</code> to stop completion, or call
	 * <code>this.complete()</code> to continue async.
	 * <p>Default: {@code null}
	 */
	public void setParsed(JavaScriptValue parsed) {
		setAttr(Attrs.parsed, parsed, null);
	}

	/**
	 * Returns a callback function to parse string representations of dates
	 * into JavaScript timestamps. Should return an integer timestamp on success.
	 * <p>Default: {@code null}
	 */
	public JavaScriptValue getParseDate() {
		return (JavaScriptValue) getAttr(Attrs.parseDate, null).asValue();
	}

	/**
	 * Sets a callback function to parse string representations of dates
	 * into JavaScript timestamps. Should return an integer timestamp on success.
	 * <p>Default: {@code null}
	 */
	public void setParseDate(JavaScriptValue parseDate) {
		setAttr(Attrs.parseDate, parseDate, null);
	}

	/**
	 * Returns the same as the columns input option,
	 * but defining rows intead of columns.
	 * <p>Default: {@code null}
	 */
	public List<List<Object>> getRows() {
		return Generics.cast(getAttr(Attrs.rows, null).asValue());
	}

	/**
	 * Sets the same as the columns input option,
	 * but defining rows intead of columns.
	 * <p>Default: {@code null}
	 */
	public void setRows(List<List<Object>> rows) {
		setAttr(Attrs.rows, rows, null);
	}

	/**
	 * Returns a URL to a remote JSON dataset, structured as a row array.
	 * Will be fetched when the chart is created using Ajax.
	 * <p>Default: {@code null}
	 */
	public String getRowsURL() {
		return getAttr(Attrs.rowsURL, null).asString();
	}

	/**
	 * Sets a URL to a remote JSON dataset, structured as a row array.
	 * Will be fetched when the chart is created using Ajax.
	 * <p>Default: {@code null}
	 */
	public void setRowsURL(String rowsURL) {
		setAttr(Attrs.rowsURL, rowsURL, null);
	}

	/**
	 * Returns in tabular input data, the first column (indexed by 0) to use.
	 * <p>Default: {@code 0}
	 */
	public Number getStartColumn() {
		return getAttr(Attrs.startColumn, 0).asNumber();
	}

	/**
	 * Sets in tabular input data, the first column (indexed by 0) to use.
	 * <p>Default: {@code 0}
	 */
	public void setStartColumn(Number startColumn) {
		setAttr(Attrs.startColumn, startColumn, 0);
	}

	/**
	 * Returns in tabular input data, the first row (indexed by 0) to use.
	 * <p>Default: {@code 0}
	 */
	public Number getStartRow() {
		return getAttr(Attrs.startRow, 0).asNumber();
	}

	/**
	 * Sets in tabular input data, the first row (indexed by 0) to use.
	 * <p>Default: {@code 0}
	 */
	public void setStartRow(Number startRow) {
		setAttr(Attrs.startRow, startRow, 0);
	}

	/**
	 * Returns switch rows and columns of the input data, so that
	 * <code>this.columns</code> effectively becomes the rows of the data set,
	 * and the rows are interpreted as series.
	 * <p>Default: {@code false}
	 */
	public boolean isSwitchRowsAndColumns() {
		return getAttr(Attrs.switchRowsAndColumns, false).asBoolean();
	}

	/**
	 * Sets switch rows and columns of the input data, so that
	 * <code>this.columns</code> effectively becomes the rows of the data set,
	 * and the rows are interpreted as series.
	 * <p>Default: {@code false}
	 */
	public void setSwitchRowsAndColumns(Boolean switchRowsAndColumns) {
		setAttr(Attrs.switchRowsAndColumns, switchRowsAndColumns, false);
	}

	/**
	 * Returns an HTML table or the id of such to be parsed as input data.
	 * Related options are <code>startRow</code>, <code>endRow</code>,
	 * <code>startColumn</code> and <code>endColumn</code> to delimit
	 * what part of the table is used.
	 * <p>Default: {@code null}
	 */
	public String getTable() {
		return getAttr(Attrs.table, null).asString();
	}

	/**
	 * Sets an HTML table or the id of such to be parsed as input data.
	 * Related options are <code>startRow</code>, <code>endRow</code>,
	 * <code>startColumn</code> and <code>endColumn</code> to delimit
	 * what part of the table is used.
	 * <p>Default: {@code null}
	 */
	public void setTable(String table) {
		setAttr(Attrs.table, table, null);
	}
}
