/* LangAccessibility.java

	Purpose:
		
	Description:
		
	History:
		10:18 AM 2022/9/20, Created by jumperchen

Copyright (C) 2022 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart;

import org.zkoss.chart.util.DynamicalAttribute;

/**
 * Configure the accessibility strings in the chart.
 * <p>
 * Requires the accessibility module
 * to be loaded. For example,
 * <pre><code>
 * Library.setProperty("org.zkoss.chart.modules.accessibility", "true");
 * </code></pre>
 *
 * <p>
 * For more dynamic control over the accessibility functionality, see {@link AccessibilityPoint#getDescriptionFormatter()},
 * {@link AccessibilitySeries#getDescriptionFormatter()}, and {@link ScreenReaderSection#getBeforeChartFormatter()}.
 * @author jumperchen
 */
public class LangAccessibility extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		announceNewData,
		axis,
		chartContainerLabel,
		chartTypes,
		credits,
		defaultChartTitle,
		drillUpButton,
		exporting,
		graphicContainerLabel,
		legend,
		rangeSelector,
		screenReaderSection,
		series,
		seriesTypeDescriptions,
		sonification,
		svgContainerLabel,
		svgContainerTitle,
		table,
		thousandsSep,
		zoom
	}

	/**
	 * Returns default announcement for new data in charts. If null,
	 * create a new one.
	 * <p>Default: an instance of {@link LangAnnounceNewData}
	 */
	public LangAnnounceNewData getAnnounceNewData() {
		return (LangAnnounceNewData) getAttr(Attrs.announceNewData,
				LangAnnounceNewData.class).asValue();
	}

	/**
	 * Sets default announcement for new data in charts.
	 */
	public void setAnnounceNewData(LangAnnounceNewData announceNewData) {
		setAttr(Attrs.announceNewData, announceNewData);
	}

	/**
	 * Returns axis description format strings. If null, create a new one.
	 */
	public LangAxis getAxis() {
		return (LangAxis) getAttr(Attrs.axis, LangAxis.class).asValue();
	}

	/**
	 * Sets axis description format strings.
	 * @param axis
	 */
	public void setAxis(LangAxis axis) {
		setAttr(Attrs.axis, axis);
	}

	/**
	 * <p>Default: {@code "{title}. Highcharts interactive chart."}
	 */
	public String getChartContainerLabel() {
		return getAttr(Attrs.chartContainerLabel,
				"{title}. Highcharts interactive chart.").asString();
	}

	public void setChartContainerLabel(String chartContainerLabel) {
		setAttr(Attrs.chartContainerLabel, chartContainerLabel);
	}

	/**
	 * Returns Chart type description strings. This is added to the chart information region.
	 * If null, create a new one.
	 */
	public LangChartTypes getChartTypes() {
		return (LangChartTypes) getAttr(Attrs.chartTypes,
				LangChartTypes.class).asValue();
	}

	/**
	 * Sets Chart type description strings. This is added to the chart information region.
	 * <p>If there is only a single series type used in the chart, we use the format string
	 * for the series type, or default if missing. There is one format string for cases
	 * where there is only a single series in the chart, and one for multiple series of the same type.
	 */
	public void setChartTypes(LangChartTypes chartTypes) {
		setAttr(Attrs.chartTypes, chartTypes);
	}

	/**
	 * <p>Default: {@code "Chart credits: {creditsStr}"}
	 */
	public String getCredits() {
		return getAttr(Attrs.credits, "Chart credits: {creditsStr}").asString();
	}

	public void setCredits(String credits) {
		setAttr(Attrs.credits, credits);
	}

	/**
	 * <p>Default: {@code "Chart"}
	 */
	public String getDefaultChartTitle() {
		return getAttr(Attrs.defaultChartTitle, "Chart").asString();
	}

	public void setDefaultChartTitle(String defaultChartTitle) {
		setAttr(Attrs.defaultChartTitle, defaultChartTitle);
	}

	/**
	 * <p>Default: {@code "{buttonText}"}
	 */
	public String getDrillUpButton() {
		return getAttr(Attrs.drillUpButton, "{buttonText}").asString();
	}

	public void setDrillUpButton(String drillUpButton) {
		setAttr(Attrs.drillUpButton, drillUpButton);
	}

	/**
	 * Returns exporting menu format strings for accessibility module. If null,
	 * create a new one.
	 */
	public LangExporting getExporting() {
		return (LangExporting) getAttr(Attrs.exporting,
				LangExporting.class).asValue();
	}

	/**
	 * Sets exporting menu format strings for accessibility module.
	 */
	public void setExporting(LangExporting exporting) {
		setAttr(Attrs.exporting, exporting);
	}

	/**
	 * Returns a label on the container wrapping the SVG.
	 * <p>Default: {@code ""}
	 */
	public String getGraphicContainerLabel() {
		return getAttr(Attrs.graphicContainerLabel, "").asString();
	}

	/**
	 * Sets a label on the container wrapping the SVG.
	 */
	public void setGraphicContainerLabel(String graphicContainerLabel) {
		setAttr(Attrs.graphicContainerLabel, graphicContainerLabel);
	}

	/**
	 * Returns language options for accessibility of the legend. If null, create
	 * a new one.
	 */
	public LangLegend getLegend() {
		return (LangLegend) getAttr(Attrs.legend, LangLegend.class).asValue();
	}

	/**
	 * Sets language options for accessibility of the legend.
	 */
	public void setLegend(LangLegend legend) {
		setAttr(Attrs.legend, legend);
	}

	/**
	 * Returns range selector language options for accessibility.
	 * If null, create a new one.
	 */
	public LangRangeSelector getRangeSelector() {
		return (LangRangeSelector) getAttr(Attrs.rangeSelector,
				LangRangeSelector.class).asValue();
	}

	/**
	 * Sets range selector language options for accessibility.
	 */
	public void setRangeSelector(LangRangeSelector rangeSelector) {
		setAttr(Attrs.rangeSelector, rangeSelector);
	}

	/**
	 * Returns language options for the screen reader information sections added before and after the charts.
	 * If null, create a new one.
	 */
	public LangScreenReaderSection getScreenReaderSection() {
		return (LangScreenReaderSection) getAttr(
				Attrs.screenReaderSection, LangScreenReaderSection.class).asValue();
	}

	/**
	 * Sets language options for the screen reader information sections added before and after the charts.
	 */
	public void setScreenReaderSection(
			LangScreenReaderSection screenReaderSection) {
		setAttr(Attrs.screenReaderSection, screenReaderSection);
	}

	/**
	 * Returns lang configuration for different series types.
	 * If null, create a new one.
	 */
	public LangSeries getSeries() {
		return (LangSeries) getAttr(Attrs.series, LangSeries.class).asValue();
	}

	/**
	 * Sets lang configuration for different series types.
	 */
	public void setSeries(LangSeries series) {
		setAttr(Attrs.series, series);
	}

	/**
	 * Returns the descriptions of lesser known series types. If null, create a
	 * new one.
	 */
	public LangSeriesTypeDescriptions getSeriesTypeDescriptions() {
		return (LangSeriesTypeDescriptions) getAttr(
				Attrs.seriesTypeDescriptions,
				LangSeriesTypeDescriptions.class).asValue();
	}

	/**
	 * Sets descriptions of lesser known series types.
	 */
	public void setSeriesTypeDescriptions(
			LangSeriesTypeDescriptions seriesTypeDescriptions) {
		setAttr(Attrs.seriesTypeDescriptions, seriesTypeDescriptions);
	}

	/**
	 * Returns language options for sonification. If null, create a new one.
	 */
	public LangSonification getSonification() {
		return (LangSonification) getAttr(Attrs.sonification,
				LangSonification.class).asValue();
	}

	/**
	 * Sets language options for sonification.
	 */
	public void setSonification(LangSonification sonification) {
		setAttr(Attrs.sonification, sonification);
	}

	/**
	 * <p>Default: {@code "Interactive chart"}
	 */
	public String getSvgContainerLabel() {
		return getAttr(Attrs.svgContainerLabel, "Interactive chart").asString();
	}

	public void setSvgContainerLabel(String svgContainerLabel) {
		setAttr(Attrs.svgContainerLabel, svgContainerLabel);
	}

	/**
	 * Returns the title element text for the chart SVG element.
	 * Leave this empty to disable adding the title element. Browsers will display
	 * this content when hovering over elements in the chart. Assistive technology may
	 * use this element to label the chart.
	 * <p>Default: {@code ""}
	 */
	public String getSvgContainerTitle() {
		return getAttr(Attrs.svgContainerTitle, "").asString();
	}


	/**
	 * Sets the title element text for the chart SVG element.
	 * Leave this empty to disable adding the title element. Browsers will display
	 * this content when hovering over elements in the chart. Assistive technology may
	 * use this element to label the chart.
	 */
	public void setSvgContainerTitle(String svgContainerTitle) {
		setAttr(Attrs.svgContainerTitle, svgContainerTitle);
	}

	/**
	 * Returns accessibility language options for the data table. If null, create
	 * a new one.
	 */
	public LangTable getTable() {
		return (LangTable) getAttr(Attrs.table, LangTable.class).asValue();
	}

	/**
	 * Sets accessibility language options for the data table.
	 */
	public void setTable(LangTable table) {
		setAttr(Attrs.table, table);
	}

	/**
	 * Returns thousands separator to use when formatting numbers for screen readers.
	 * <p>
	 * Note that many screen readers will not handle space as a thousands separator,
	 * and will consider "11 700" as two numbers.
	 * <p>Default: {@code ""}
	 */
	public String getThousandsSep() {
		return getAttr(Attrs.thousandsSep, ",").asString();
	}


	/**
	 * Sets thousands separator to use when formatting numbers for screen readers.
	 * <p>
	 * Note that many screen readers will not handle space as a thousands separator,
	 * and will consider "11 700" as two numbers.
	 * <p>Sets to null to use the separator defined in {@link Lang#getThousandsSep()}.
	 */
	public void setThousandsSep(String thousandsSep) {
		setAttr(Attrs.thousandsSep, thousandsSep);
	}

	/**
	 * Returns chart and map zoom accessibility language options.
	 * If null, create a new one.
	 */
	public LangZoom getZoom() {
		return (LangZoom) getAttr(Attrs.zoom, LangZoom.class).asValue();
	}

	/**
	 * Sets chart and map zoom accessibility language options.
	 */
	public void setZoom(LangZoom zoom) {
		setAttr(Attrs.zoom, zoom);
	}
}