/* Mapping.java

        Purpose:
                
        Description:
                
        History:
                Fri Sep 15 17:57:22 CST 2023, Created by jamson

Copyright (C) 2023 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart;

import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.json.JavaScriptValue;

/**
 * Mapping options for the audio parameters.
 * <p>All parameters can be either:
 * <ul>
 *     <li>A string, referencing a point property to map to.</li>
 *     <li>A number, setting the value of the audio parameter directly.</li>
 *     <li>A callback function, returning the value programmatically.</li>
 *     <li>An object defining detailed configuration of the mapping.</li>
 * </ul>
 * If a function is used, it should return the desired value for the audio parameter.
 * The function is called for each audio event to be played,
 * and receives a context object parameter with time,
 * and potentially point and value depending on the track.
 * point is available if the audio event is related to a data point,
 * and value is available if the track is used for a context track using valueInterval.
 * @author Jamson Chan
 * @since 11.2.0.0
 */
public class Mapping extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		frequency,
		gapBetweenNotes,
		highpass,
		lowpass,
		noteDuration,
		pan,
		pitch,
		playDelay,
		rate,
		text,
		time,
		tremolo,
		volume
	}

	/**
	 * Returns the Frequency or Number or String object of frequency, depend on what was set.
	 * <p>Map to filter frequency in Hertz from 1 to 20,000Hz.
	 * <p>Frequency in Hertz of notes. Overrides pitch mapping if set.
	 */
	public Object getFrequency() {
		Object frequency = this.getAttr(Attrs.frequency);
		if (frequency == null) {
			frequency = new Frequency();
			setFrequency((Frequency) frequency);
		}
		return frequency;
	}

	/**
	 * Sets the Frequency object of frequency.
	 * <p>Map to filter frequency in Hertz from 1 to 20,000Hz.
	 * <p>Frequency in Hertz of notes. Overrides pitch mapping if set.
	 */
	public void setFrequency(Frequency frequency) {
		setAttr(Attrs.frequency, frequency);
	}

	/**
	 * Sets the Number object of frequency.
	 * <p>Map to filter frequency in Hertz from 1 to 20,000Hz.
	 * <p>Frequency in Hertz of notes. Overrides pitch mapping if set.
	 */
	public void setFrequency(Number frequency) {
		setAttr(Attrs.frequency, frequency);
	}

	/**
	 * Sets the String object of frequency.
	 * <p>Map to filter frequency in Hertz from 1 to 20,000Hz.
	 * <p>Frequency in Hertz of notes. Overrides pitch mapping if set.
	 */
	public void setFrequency(String frequency) {
		setAttr(Attrs.frequency, frequency);
	}

	/**
	 * Returns the GapBetweenNotes or Number or String object of gapBetweenNotes, depend on what was set.
	 * <p>Gap in milliseconds between notes if pitch is mapped to an array of notes.
	 * <p>Can be set to a fixed value, a prop to map to, a function, or a mapping object.
	 */
	public Object getGapBetweenNotes() {
		Object gapBetweenNotes = this.getAttr(Attrs.gapBetweenNotes);
		if (gapBetweenNotes == null) {
			gapBetweenNotes = new GapBetweenNotes();
			setGapBetweenNotes((GapBetweenNotes) gapBetweenNotes);
		}
		return gapBetweenNotes;
	}

	/**
	 * Sets the GapBetweenNotes object of gapBetweenNotes.
	 * <p>Gap in milliseconds between notes if pitch is mapped to an array of notes.
	 * <p>Can be set to a fixed value, a prop to map to, a function, or a mapping object.
	 */
	public void setGapBetweenNotes(GapBetweenNotes gapBetweenNotes) {
		setAttr(Attrs.gapBetweenNotes, gapBetweenNotes);
	}

	/**
	 * Sets the Number object of gapBetweenNotes.
	 * <p>Gap in milliseconds between notes if pitch is mapped to an array of notes.
	 * <p>Can be set to a fixed value, a prop to map to, a function, or a mapping object.
	 */
	public void setGapBetweenNotes(Number gapBetweenNotes) {
		setAttr(Attrs.gapBetweenNotes, gapBetweenNotes);
	}

	/**
	 * Sets the String object of gapBetweenNotes.
	 * <p>Gap in milliseconds between notes if pitch is mapped to an array of notes.
	 * <p>Can be set to a fixed value, a prop to map to, a function, or a mapping object.
	 */
	public void setGapBetweenNotes(String gapBetweenNotes) {
		setAttr(Attrs.gapBetweenNotes, gapBetweenNotes);
	}

	/**
	 * Returns the Highpass object.
	 * <p>Mapping options for the highpass filter.
	 * <p>A highpass filter lets high frequencies through, but stops low frequencies, making the sound thinner.
	 */
	public Highpass getHighpass() {
		Highpass highpass = (Highpass) getAttr(Attrs.highpass);
		if (highpass == null) {
			highpass = new Highpass();
			setHighpass(highpass);
		}
		return highpass;
	}

	/**
	 * Sets the Highpass object.
	 * <p>Mapping options for the highpass filter.
	 * <p>A highpass filter lets high frequencies through, but stops low frequencies, making the sound thinner.
	 */
	public void setHighpass(Highpass highpass) {
		setAttr(Attrs.highpass, highpass);
	}

	/**
	 * Returns the Lowpass object.
	 * <p>Mapping options for the lowpass filter.
	 * <p>lowpass filter lets low frequencies through, but stops high frequencies, making the sound more dull.
	 */
	public Lowpass getLowpass() {
		Lowpass lowpass = (Lowpass) getAttr(Attrs.lowpass);
		if (lowpass == null) {
			lowpass = new Lowpass();
			setLowpass(lowpass);
		}
		return lowpass;
	}

	/**
	 * Sets the Lowpass object.
	 * <p>Mapping options for the lowpass filter.
	 * <p>lowpass filter lets low frequencies through, but stops high frequencies, making the sound more dull.
	 */
	public void setLowpass(Lowpass lowpass) {
		setAttr(Attrs.lowpass, lowpass);
	}

	/**
	 * Returns the NoteDuration or Number or String object of note duration, depend on what was set.
	 * <p>Note duration determines for how long a note plays, in milliseconds.
	 * <p>It only affects instruments that are able to play continuous sustained notes.
	 * Examples of these instruments from the presets include
	 * <code>flute</code>, <code>saxophone</code>, <code>trumpet</code>, <code>sawsynth</code>, <code>wobble</code>,
	 * <code>basic1</code>, <code>basic2</code>, <code>sine</code>, <code>sineGlide</code>, <code>triangle</code>,
	 * <code>square</code>, <code>sawtooth</code>, <code>noise</code>, <code>filteredNoise, and <code>wind</code>.
	 * <p>Can be set to a fixed value, a prop to map to, a function, or a mapping object.
	 */
	public Object getNoteDuration() {
		Object noteDuration = this.getAttr(Attrs.noteDuration);
		if (noteDuration == null) {
			noteDuration = new NoteDuration();
			setNoteDuration((NoteDuration) noteDuration);
		}
		return noteDuration;
	}

	/**
	 * Sets the NoteDuration object of note duration.
	 * <p>Note duration determines for how long a note plays, in milliseconds.
	 * <p>It only affects instruments that are able to play continuous sustained notes.
	 * Examples of these instruments from the presets include
	 * <code>flute</code>, <code>saxophone</code>, <code>trumpet</code>, <code>sawsynth</code>, <code>wobble</code>,
	 * <code>basic1</code>, <code>basic2</code>, <code>sine</code>, <code>sineGlide</code>, <code>triangle</code>,
	 * <code>square</code>, <code>sawtooth</code>, <code>noise</code>, <code>filteredNoise, and <code>wind</code>.
	 * <p>Can be set to a fixed value, a prop to map to, a function, or a mapping object.
	 */
	public void setNoteDuration(NoteDuration noteDuration) {
		setAttr(Attrs.noteDuration, noteDuration);
	}

	/**
	 * Sets the Number object of note duration.
	 * <p>Note duration determines for how long a note plays, in milliseconds.
	 * <p>It only affects instruments that are able to play continuous sustained notes.
	 * Examples of these instruments from the presets include
	 * <code>flute</code>, <code>saxophone</code>, <code>trumpet</code>, <code>sawsynth</code>, <code>wobble</code>,
	 * <code>basic1</code>, <code>basic2</code>, <code>sine</code>, <code>sineGlide</code>, <code>triangle</code>,
	 * <code>square</code>, <code>sawtooth</code>, <code>noise</code>, <code>filteredNoise, and <code>wind</code>.
	 * <p>Can be set to a fixed value, a prop to map to, a function, or a mapping object.
	 */
	public void setNoteDuration(Number noteDuration) {
		setAttr(Attrs.noteDuration, noteDuration);
	}

	/**
	 * Sets the String object of note duration.
	 * <p>Note duration determines for how long a note plays, in milliseconds.
	 * <p>It only affects instruments that are able to play continuous sustained notes.
	 * Examples of these instruments from the presets include
	 * <code>flute</code>, <code>saxophone</code>, <code>trumpet</code>, <code>sawsynth</code>, <code>wobble</code>,
	 * <code>basic1</code>, <code>basic2</code>, <code>sine</code>, <code>sineGlide</code>, <code>triangle</code>,
	 * <code>square</code>, <code>sawtooth</code>, <code>noise</code>, <code>filteredNoise, and <code>wind</code>.
	 * <p>Can be set to a fixed value, a prop to map to, a function, or a mapping object.
	 */
	public void setNoteDuration(String noteDuration) {
		setAttr(Attrs.noteDuration, noteDuration);
	}

	/**
	 * Returns the Pan or Number or String object of pan, depend on what was set.
	 * <p>Pan refers to the stereo panning position of the sound. It is defined from -1 (left) to 1 (right).
	 * <p>By default it is mapped to x, making the sound move from left to right as the chart plays.
	 * <p>Can be set to a fixed value, a prop to map to, a function, or a mapping object.
	 */
	public Object getPan() {
		Object pan = this.getAttr(Attrs.pan);
		if (pan == null) {
			pan = new Pan();
			setPan((Pan) pan);
		}
		return pan;
	}

	/**
	 * Sets the Pan object of pan.
	 * <p>Pan refers to the stereo panning position of the sound. It is defined from -1 (left) to 1 (right).
	 * <p>By default it is mapped to x, making the sound move from left to right as the chart plays.
	 * <p>Can be set to a fixed value, a prop to map to, a function, or a mapping object.
	 */
	public void setPan(Pan pan) {
		setAttr(Attrs.pan, pan);
	}

	/**
	 * Sets the Number object of pan.
	 * <p>Pan refers to the stereo panning position of the sound. It is defined from -1 (left) to 1 (right).
	 * <p>By default it is mapped to x, making the sound move from left to right as the chart plays.
	 * <p>Can be set to a fixed value, a prop to map to, a function, or a mapping object.
	 */
	public void setPan(Number pan) {
		setAttr(Attrs.pan, pan);
	}

	/**
	 * Sets the String object of pan.
	 * <p>Pan refers to the stereo panning position of the sound. It is defined from -1 (left) to 1 (right).
	 * <p>By default it is mapped to x, making the sound move from left to right as the chart plays.
	 * <p>Can be set to a fixed value, a prop to map to, a function, or a mapping object.
	 */
	public void setPan(String pan) {
		setAttr(Attrs.pan, pan);
	}

	/**
	 * Returns the Pitch or Number or String object of pitch, depend on what was set.
	 * <p>Musical pitch refers to how high or low notes are played.
	 * <p>By default it is mapped to y so low y values are played with a lower pitch,
	 * and high values are played with a higher pitch.
	 * <p>Pitch mapping has a few extra features compared to other audio parameters.
	 * <p>Firstly, it accepts not only number values, but also string values denoting note names.
	 * These are given in the form <code><note><octave></code>, for example <code>"c6"</code> or <code>"F#2"</code>.
	 * <p>Secondly, it is possible to map pitch to an array of notes. In this case,
	 * the <code>[gapBetweenNotes](#sonification.defaultInstrumentOptions.mapping.gapBetweenNotes)</code>
	 * mapping determines the delay between these notes.
	 * <p>Thirdly, it is possible to define a musical scale to follow when mapping.
	 * <p>Can be set to a fixed value, an array, a prop to map to, a function, or a mapping object.
	 */
	public Object getPitch() {
		Object pitch = this.getAttr(Attrs.pitch);
		if (pitch == null) {
			pitch = new Pitch();
			setPitch((Pitch) pitch);
		}
		return pitch;
	}

	/**
	 * Sets the Pitch object of pitch.
	 * <p>Musical pitch refers to how high or low notes are played.
	 * <p>By default it is mapped to y so low y values are played with a lower pitch,
	 * and high values are played with a higher pitch.
	 * <p>Pitch mapping has a few extra features compared to other audio parameters.
	 * <p>Firstly, it accepts not only number values, but also string values denoting note names.
	 * These are given in the form <code><note><octave></code>, for example <code>"c6"</code> or <code>"F#2"</code>.
	 * <p>Secondly, it is possible to map pitch to an array of notes. In this case,
	 * the <code>[gapBetweenNotes](#sonification.defaultInstrumentOptions.mapping.gapBetweenNotes)</code>
	 * mapping determines the delay between these notes.
	 * <p>Thirdly, it is possible to define a musical scale to follow when mapping.
	 * <p>Can be set to a fixed value, an array, a prop to map to, a function, or a mapping object.
	 */
	public void setPitch(Pitch pitch) {
		setAttr(Attrs.pitch, pitch);
	}

	/**
	 * Sets the Number object of pitch.
	 * <p>Musical pitch refers to how high or low notes are played.
	 * <p>By default it is mapped to y so low y values are played with a lower pitch,
	 * and high values are played with a higher pitch.
	 * <p>Pitch mapping has a few extra features compared to other audio parameters.
	 * <p>Firstly, it accepts not only number values, but also string values denoting note names.
	 * These are given in the form <code><note><octave></code>, for example <code>"c6"</code> or <code>"F#2"</code>.
	 * <p>Secondly, it is possible to map pitch to an array of notes. In this case,
	 * the <code>[gapBetweenNotes](#sonification.defaultInstrumentOptions.mapping.gapBetweenNotes)</code>
	 * mapping determines the delay between these notes.
	 * <p>Thirdly, it is possible to define a musical scale to follow when mapping.
	 * <p>Can be set to a fixed value, an array, a prop to map to, a function, or a mapping object.
	 */
	public void setPitch(Number pitch) {
		setAttr(Attrs.pitch, pitch);
	}

	/**
	 * Sets the String object of pitch.
	 * <p>Musical pitch refers to how high or low notes are played.
	 * <p>By default it is mapped to y so low y values are played with a lower pitch,
	 * and high values are played with a higher pitch.
	 * <p>Pitch mapping has a few extra features compared to other audio parameters.
	 * <p>Firstly, it accepts not only number values, but also string values denoting note names.
	 * These are given in the form <code><note><octave></code>, for example <code>"c6"</code> or <code>"F#2"</code>.
	 * <p>Secondly, it is possible to map pitch to an array of notes. In this case,
	 * the <code>[gapBetweenNotes](#sonification.defaultInstrumentOptions.mapping.gapBetweenNotes)</code>
	 * mapping determines the delay between these notes.
	 * <p>Thirdly, it is possible to define a musical scale to follow when mapping.
	 * <p>Can be set to a fixed value, an array, a prop to map to, a function, or a mapping object.
	 */
	public void setPitch(String pitch) {
		setAttr(Attrs.pitch, pitch);
	}

	/**
	 * Sets the JavaScriptValue object of pitch.
	 * <p>Musical pitch refers to how high or low notes are played.
	 * <p>By default it is mapped to y so low y values are played with a lower pitch,
	 * and high values are played with a higher pitch.
	 * <p>Pitch mapping has a few extra features compared to other audio parameters.
	 * <p>Firstly, it accepts not only number values, but also string values denoting note names.
	 * These are given in the form <code><note><octave></code>, for example <code>"c6"</code> or <code>"F#2"</code>.
	 * <p>Secondly, it is possible to map pitch to an array of notes. In this case,
	 * the <code>[gapBetweenNotes](#sonification.defaultInstrumentOptions.mapping.gapBetweenNotes)</code>
	 * mapping determines the delay between these notes.
	 * <p>Thirdly, it is possible to define a musical scale to follow when mapping.
	 * <p>Can be set to a fixed value, an array, a prop to map to, a function, or a mapping object.
	 */
	public void setPitch(JavaScriptValue pitch) {
		setAttr(Attrs.pitch, pitch);
	}

	/**
	 * Returns the PlayDelay or Number or String object of play delay, depend on what was set.
	 * <p>Milliseconds to wait before playing, comes in addition to the time determined by the time mapping.
	 * <p>Can also be negative to play before the mapped time.
	 */
	public Object getPlayDelay() {
		Object playDelay = this.getAttr(Attrs.playDelay);
		if (playDelay == null) {
			playDelay = new PlayDelay();
			setPlayDelay((PlayDelay) playDelay);
		}
		return playDelay;
	}

	/**
	 * Sets the PlayDelay object of play delay.
	 * <p>Milliseconds to wait before playing, comes in addition to the time determined by the time mapping.
	 * <p>Can also be negative to play before the mapped time.
	 */
	public void setPlayDelay(PlayDelay playDelay) {
		setAttr(Attrs.playDelay, playDelay);
	}

	/**
	 * Sets the Number object of play delay.
	 * <p>Milliseconds to wait before playing, comes in addition to the time determined by the time mapping.
	 * <p>Can also be negative to play before the mapped time.
	 */
	public void setPlayDelay(Number playDelay) {
		setAttr(Attrs.playDelay, playDelay);
	}

	/**
	 * Sets the String object of play delay.
	 * <p>Milliseconds to wait before playing, comes in addition to the time determined by the time mapping.
	 * <p>Can also be negative to play before the mapped time.
	 */
	public void setPlayDelay(String playDelay) {
		setAttr(Attrs.playDelay, playDelay);
	}

	/**
	 * Returns the Rate or Number or String object of rate, depend on what was set.
	 * <p>Rate mapping for speech tracks.
	 */
	public Object getRate() {
		Object rate = this.getAttr(Attrs.rate);
		if (rate == null) {
			rate = new Rate();
			setRate((Rate) rate);
		}
		return rate;
	}

	/**
	 * Sets the Rate object of rate.
	 * <p>Rate mapping for speech tracks.
	 */
	public void setRate(Rate rate) {
		setAttr(Attrs.rate, rate);
	}

	/**
	 * Sets the Number object of rate.
	 * <p>Rate mapping for speech tracks.
	 */
	public void setRate(Number rate) {
		setAttr(Attrs.rate, rate);
	}

	/**
	 * Sets the String object of rate.
	 * <p>Rate mapping for speech tracks.
	 */
	public void setRate(String rate) {
		setAttr(Attrs.rate, rate);
	}

	/**
	 * Returns the text mapping for speech tracks.
	 * <p>Default: {@code null}
	 */
	public String getText() {
		return getAttr(Attrs.text, null).asString();
	}

	/**
	 * Sets the text mapping for speech tracks.
	 */
	public void setText(String text) {
		setAttr(Attrs.text, text, null);
	}

	/**
	 * Returns the Time or Number or String object of time, depend on what was set.
	 * <p>Time mapping determines what time each point plays.
	 * It is defined as an offset in milliseconds,
	 * where 0 means it plays immediately when the chart is sonified.
	 * <p>By default time is mapped to x, meaning points with the lowest x value plays first,
	 * and points with the highest x value plays last.
	 * <p>Can be set to a fixed value, a prop to map to, a function, or a mapping object.
	 */
	public Object getTime() {
		Object time = this.getAttr(Attrs.time);
		if (time == null) {
			time = new Time();
			setTime((Time) time);
		}
		return time;
	}

	/**
	 * Sets the Time object of time.
	 * <p>Time mapping determines what time each point plays.
	 * It is defined as an offset in milliseconds,
	 * where 0 means it plays immediately when the chart is sonified.
	 * <p>By default time is mapped to x, meaning points with the lowest x value plays first,
	 * and points with the highest x value plays last.
	 * <p>Can be set to a fixed value, a prop to map to, a function, or a mapping object.
	 */
	public void setTime(Time time) {
		setAttr(Attrs.time, time);
	}

	/**
	 * Sets the Number object of time.
	 * <p>Time mapping determines what time each point plays.
	 * It is defined as an offset in milliseconds,
	 * where 0 means it plays immediately when the chart is sonified.
	 * <p>By default time is mapped to x, meaning points with the lowest x value plays first,
	 * and points with the highest x value plays last.
	 * <p>Can be set to a fixed value, a prop to map to, a function, or a mapping object.
	 */
	public void setTime(Number time) {
		setAttr(Attrs.time, time);
	}

	/**
	 * Sets the String object of time.
	 * <p>Time mapping determines what time each point plays.
	 * It is defined as an offset in milliseconds,
	 * where 0 means it plays immediately when the chart is sonified.
	 * <p>By default time is mapped to x, meaning points with the lowest x value plays first,
	 * and points with the highest x value plays last.
	 * <p>Can be set to a fixed value, a prop to map to, a function, or a mapping object.
	 */
	public void setTime(String time) {
		setAttr(Attrs.time, time);
	}

	/**
	 * Returns the Tremolo object of tremolo.
	 * <p>Mapping options for tremolo effects.
	 * <p>Tremolo is repeated changes of volume over time.
	 */
	public Tremolo getTremolo() {
		Tremolo tremolo = (Tremolo) getAttr(Attrs.tremolo);
		if (tremolo == null) {
			tremolo = new Tremolo();
			setTremolo(tremolo);
		}
		return tremolo;
	}

	/**
	 * Sets the Tremolo object of tremolo.
	 * <p>Mapping options for tremolo effects.
	 * <p>Tremolo is repeated changes of volume over time.
	 */
	public void setTremolo(Tremolo tremolo) {
		setAttr(Attrs.tremolo, tremolo);
	}

	/**
	 * Returns the Volume or Number or String object of volume, depend on what was set.
	 * <p>The volume of notes, from 0 to 1.
	 */
	public Object getVolume() {
		Object volume = this.getAttr(Attrs.volume);
		if (volume == null) {
			volume = new Volume();
			setVolume((Volume) volume);
		}
		return volume;
	}

	/**
	 * Sets the Volume object of volume.
	 * <p>The volume of notes, from 0 to 1.
	 */
	public void setVolume(Volume volume) {
		setAttr(Attrs.volume, volume);
	}

	/**
	 * Sets the Number object of volume.
	 * <p>The volume of notes, from 0 to 1.
	 */
	public void setVolume(Number volume) {
		setAttr(Attrs.volume, volume);
	}

	/**
	 * Sets the String object of volume.
	 * <p>The volume of notes, from 0 to 1.
	 */
	public void setVolume(String volume) {
		setAttr(Attrs.volume, volume);
	}
}
