/* ScreenReaderSection.java

	Purpose:
		
	Description:
		
	History:
		2:57 PM 2022/9/19, Created by jumperchen

Copyright (C) 2022 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart;

import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.json.JavaScriptValue;

/**
 * Accessibility options for the screen reader information sections added before and after the chart.
 * @author jumperchen
 * @since 10.2.1.0
 */
public class ScreenReaderSection extends Optionable {

	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		afterChartFormat,
		afterChartFormatter,
		axisRangeDateFormat,
		beforeChartFormat,
		beforeChartFormatter,
		onPlayAsSoundClick,
		onViewDataTableClick
	}

	/**
	 * Returns the format for the screen reader information region after the chart.
	 * Analogous to {@code beforeChartFormat}.
	 * <p>Default: {@code "{endOfChartMarker}"}
	 */
	public String getAfterChartFormat() {
		return getAttr(Attrs.afterChartFormat, "{endOfChartMarker}").asString();
	}

	/**
	 * Sets the format for the screen reader information region after the chart.
	 * Analogous to {@code beforeChartFormat}.
	 * <p>Default: {@code "{endOfChartMarker}"}
	 */
	public void setAfterChartFormat(String afterChartFormat) {
		setAttr(Attrs.afterChartFormat, afterChartFormat);
	}

	/**
	 * Returns a formatter function to create the HTML contents of the hidden screen
	 * reader information region after the chart. Analogous to {@code beforeChartFormatter}.
	 *
	 * <p>Default: {@code null}.
	 */
	public JavaScriptValue getAfterChartFormatter() {
		return (JavaScriptValue) getAttr(Attrs.afterChartFormatter,
				null).asValue();
	}

	/**
	 * Sets a formatter function to create the HTML contents of the hidden screen
	 * reader information region after the chart. Analogous to {@code beforeChartFormatter}.
	 *
	 * <p>Default: {@code null}.
	 */
	public void setAfterChartFormatter(JavaScriptValue afterChartFormatter) {
		setAttr(Attrs.afterChartFormatter, afterChartFormatter);
	}

	/**
	 * Returns the date format to use to describe range of datetime axes.
	 *
	 * <p>Default: {@code "%Y-%m-%d %H:%M:%S"}.
	 */
	public String getAxisRangeDateFormat() {
		return getAttr(Attrs.axisRangeDateFormat,
				"%Y-%m-%d %H:%M:%S").asString();
	}

	/**
	 * Sets the date format to use to describe range of datetime axes.
	 * <p>Default: {@code "%Y-%m-%d %H:%M:%S"}.
	 * @see AccessibilityPoint#setDateFormat(String) 
	 */
	public void setAxisRangeDateFormat(String axisRangeDateFormat) {
		setAttr(Attrs.axisRangeDateFormat, axisRangeDateFormat);
	}

	/**
	 * Returns the format for the screen reader information region before the chart.
	 * <p>Default: {@code "<{headingTagName}>{chartTitle}</{headingTagName}><div>{typeDescription}</div><div>{chartSubtitle}</div><div>{chartLongdesc}</div><div>{playAsSoundButton}</div><div>{viewTableButton}</div><div>{xAxisDescription}</div><div>{yAxisDescription}</div><div>{annotationsTitle}{annotationsList}</div>"}
	 */
	public String getBeforeChartFormat() {
		return getAttr(Attrs.beforeChartFormat,
				"<{headingTagName}>{chartTitle}</{headingTagName}><div>{typeDescription}</div><div>{chartSubtitle}</div><div>{chartLongdesc}</div><div>{playAsSoundButton}</div><div>{viewTableButton}</div><div>{xAxisDescription}</div><div>{yAxisDescription}</div><div>{annotationsTitle}{annotationsList}</div>").asString();
	}

	/**
	 * Sets the format for the screen reader information region before the chart.
	 * Supported HTML tags are {@code <h1-6>}, {@code <p>}, {@code <div>}, {@code <a>},
	 * {@code <ul>}, {@code <ol>}, {@code <li>}, and {@code <button>}. Attributes
	 * are not supported, except for id on {@code <div>}, {@code <a>}, and {@code <button>}.
	 * Id is required on {@code <a>} and {@code <button>} in the format {@code <tag id="abcd">}.
	 * Numbers, lower- and uppercase letters, "-" and "#" are valid characters in IDs.
	 *
	 * <p>
	 * The headingTagName is an auto-detected heading (h1-h6) that corresponds to
	 * the heading level below the previous heading in the DOM.
	 *
	 * <p>
	 * Set to empty string to remove the region altogether.
	 */
	public void setBeforeChartFormat(String beforeChartFormat) {
		setAttr(Attrs.beforeChartFormat, beforeChartFormat);
	}

	/**
	 * Sets a formatter function to create the HTML contents of the hidden screen
	 * reader information region before the chart.
	 * <p> Default: {@code null}
	 */
	public JavaScriptValue getBeforeChartFormatter() {
		return (JavaScriptValue) getAttr(Attrs.beforeChartFormatter,
				null).asValue();
	}

	/**
	 * Sets a formatter function to create the HTML contents of the hidden screen
	 * reader information region before the chart. Receives one argument, {@code chart},
	 * referring to the chart object. Should return a string with the HTML content
	 * of the region. By default this returns an automatic description of the chart
	 * based on {@link #setBeforeChartFormat(String)}.
	 */
	public void setBeforeChartFormatter(JavaScriptValue beforeChartFormatter) {
		setAttr(Attrs.beforeChartFormatter, beforeChartFormatter);
	}

	/**
	 * Returns the function to run upon clicking the "Play as sound" button in the
	 * screen reader region.
	 * <p> Default: {@code null}
	 */
	public JavaScriptValue getOnPlayAsSoundClick() {
		return (JavaScriptValue) getAttr(Attrs.onPlayAsSoundClick,
				null).asValue();
	}

	/**
	 * Sets the function to run upon clicking the "Play as sound" button in the
	 * screen reader region.
	 * <p>
	 * By default Highcharts will call the {@code chart.sonify} function.
	 */
	public void setOnPlayAsSoundClick(JavaScriptValue onPlayAsSoundClick) {
		setAttr(Attrs.onPlayAsSoundClick, onPlayAsSoundClick);
	}

	/**
	 * Returns the function to run upon clicking the "View as Data Table" link in
	 * the screen reader region.
	 *
	 * <p>Default: {@code null}
	 */
	public JavaScriptValue getOnViewDataTableClick() {
		return (JavaScriptValue) getAttr(Attrs.onViewDataTableClick,
				null).asValue();
	}

	/**
	 * Sets the function to run upon clicking the "View as Data Table" link in the
	 * screen reader region.
	 *
	 * <p>
	 * By default Highcharts will insert and set focus to a data table representation of the chart.
	 *
	 */
	public void setOnViewDataTableClick(JavaScriptValue onViewDataTableClick) {
		setAttr(Attrs.onViewDataTableClick, onViewDataTableClick);
	}
}