/** YAxis.java.

	Purpose:
		
	Description:
		
	History:
		5:05:49 PM Jan 9, 2014, Created by jumperchen

Copyright (C) 2014 Potix Corporation. All Rights Reserved.
 */
package org.zkoss.chart;

import java.util.Arrays;
import java.util.List;

import org.zkoss.chart.util.DeferredCall;
import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.chart.util.JSFunction;
import org.zkoss.json.JSONObject;
import org.zkoss.lang.Generics;

/**
 * The Y axis or value axis. Normally this is the vertical axis, though if the
 * chart is inverted this is the horizontal axis. In case of multiple axes, the
 * yAxis node is an array of configuration objects.
 * 
 * @author jumperchen
 */
public class YAxis extends XAxis {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		endOnTick,
		gridLineInterpolation,
		gridLineWidth,
		labels,
		lineWidth,
		maxColor,
		maxPadding,
		minColor,
		minPadding,
		reversed,
		reversedStacks,
		stackLabels,
		stackShadow,
		startOnTick,
		stops,
		tickPixelInterval,
		tickWidth,
		title,
		tooltipValueFormat,
	}

	/**
	 * Returns the stack labels show the total value for each bar in a stacked
	 * column or bar chart. The label will be placed on top of positive columns
	 * and below negative columns. In case of an inverted column chart or a bar
	 * chart the label is placed to the right of positive bars and to the left
	 * of negative bars.
	 */
	public StackLabels getStackLabels() {
		StackLabels stackLabels = (StackLabels) getAttr(Attrs.stackLabels);
		if (stackLabels == null) {
			stackLabels = new StackLabels();
			setStackLabels(stackLabels);
		}
		return stackLabels;
	}

	/**
	 * Sets the stack labels show the total value for each bar in a stacked
	 * column or bar chart. The label will be placed on top of positive columns
	 * and below negative columns. In case of an inverted column chart or a bar
	 * chart the label is placed to the right of positive bars and to the left
	 * of negative bars.
	 */
	public void setStackLabels(StackLabels stackLabels) {
		setAttr(Attrs.stackLabels, stackLabels);
	}


	/**
	 * Returns the stack labels show the total value for each bar in a stacked
	 * column or bar chart. The label will be placed on top of positive columns
	 * and below negative columns. In case of an inverted column chart or a bar
	 * chart the label is placed to the right of positive bars and to the left
	 * of negative bars.
	 * @since 11.2.0.0
	 */
	public StackShadow getStackShadow() {
		StackShadow stackShadow = (StackShadow) getAttr(Attrs.stackShadow);
		if (stackShadow == null) {
			stackShadow = new StackShadow();
			setStackShadow(stackShadow);
		}
		return stackShadow;
	}

	/**
	 * Sets the stack labels show the total value for each bar in a stacked
	 * column or bar chart. The label will be placed on top of positive columns
	 * and below negative columns. In case of an inverted column chart or a bar
	 * chart the label is placed to the right of positive bars and to the left
	 * of negative bars.
	 * @since 11.2.0.0
	 */
	public void setStackShadow(StackShadow stackShadow) {
		setAttr(Attrs.stackShadow, stackShadow);
	}

	/**
	 * Returns whether to force the axis to end on a tick. Use this option with
	 * the {@link #setMaxPadding(Number)} option to control the axis end.
	 * <p>
	 * Default: true.
	 */
	public boolean isEndOnTick() {
		return getAttr(Attrs.endOnTick, true).asBoolean();
	}

	/**
	 * Sets whether to force the axis to end on a tick. Use this option with the
	 * {@link #setMaxPadding(Number)} option to control the axis end.
	 * <p>
	 * Default: true.
	 */
	public void setEndOnTick(boolean endOnTick) {
		setAttr(Attrs.endOnTick, endOnTick);
	}

	/**
	 * Returns whether to force the axis to start on a tick. Use this option
	 * with the <code>maxPadding</code> option to control the axis start.
	 * <p>
	 * Default: true.
	 */
	public boolean isStartOnTick() {
		return getAttr(Attrs.startOnTick, true).asBoolean();
	}

	/**
	 * Returns the pixel width of the major tick marks.
	 * <p>
	 * Default: 0.
	 */
	public Number getTickWidth() {
		return getAttr(Attrs.tickWidth, 0).asNumber();
	}

	/**
	 * Sets the pixel width of the major tick marks.
	 */
	public void setTickWidth(Number tickWidth) {
		setAttr(Attrs.tickWidth, tickWidth, 0);
	}

	/**
	 * Parallel coordinates only.
	 * Returns a format that will be used for point.y and available in tooltip.pointFormat as {point.formattedValue}.
	 * If not set, {point.formattedValue} will use other options, in this order:
	 * <ol>
	 *   <li>yAxis.labels.format will be used if set</li>
	 *   <li>If yAxis is a category, then category name will be displayed</li>
	 *   <li>If yAxis is a datetime, then value will use the same format as yAxis labels</li>
	 *   <li>If yAxis is linear/logarithmic type, then simple value will be used</li>
	 * </ol>
	 * <p>
	 * Default: null
	 * @since 7.2.1.0
	 */
	public String getTooltipValueFormat() {
		return getAttr(Attrs.tooltipValueFormat, null).asString();
	}

	/**
	 * Parallel coordinates only.
	 * Sets a format that will be used for point.y and available in tooltip.pointFormat as {point.formattedValue}.
	 * If not set, {point.formattedValue} will use other options, in this order:
	 * <ol>
	 *   <li>yAxis.labels.format will be used if set</li>
	 *   <li>If yAxis is a category, then category name will be displayed</li>
	 *   <li>If yAxis is a datetime, then value will use the same format as yAxis labels</li>
	 *   <li>If yAxis is linear/logarithmic type, then simple value will be used</li>
	 * </ol>
	 * @since 7.2.1.0
	 */
	public void setTooltipValueFormat(String tooltipValueFormat) {
		setAttr(Attrs.tooltipValueFormat, tooltipValueFormat);
	}
	
	/**
	 * Returns the max color. Solid gauge only. Unless stops are set,
	 * the color to represent the maximum value of the Y axis.
	 * <p>
	 * Default: #102D4C.
	 */
	public Color getMaxColor() {
		if (!containsKey(Attrs.maxColor)) {
			setMaxColor(new Color("#102D4C"));
		}
		return (Color) getAttr(Attrs.maxColor);
	}

	/**
	 * Sets the the max color. Solid gauge only. Unless stops are set,
	 * the color to represent the maximum value of the Y axis.
	 */
	public void setMaxColor(Color color) {
		setAttr(Attrs.maxColor, color, NOT_NULL_VALUE);
	}

	/**
	 * Sets the the max color. Solid gauge only. Unless stops are set,
	 * the color to represent the maximum value of the Y axis.
	 */
	public void setMaxColor(String color) {
		setMaxColor(new Color(color));
	}

	/**
	 * Sets the the max color. Solid gauge only. Unless stops are set,
	 * the color to represent the maximum value of the Y axis.
	 */
	public void setMaxColor(LinearGradient color) {
		setMaxColor(new Color(color));
	}

	/**
	 * Sets the the max color. Solid gauge only. Unless stops are set,
	 * the color to represent the maximum value of the Y axis.
	 */
	public void setMaxColor(RadialGradient color) {
		setMaxColor(new Color(color));
	}
	

	/**
	 * Returns padding of the max value relative to the length of the axis. A
	 * padding of 0.05 will make a 100px axis 5px longer. This is useful when
	 * you don't want the highest data value to appear on the edge of the plot
	 * area. When the {@link #setMax(Number)} option is set or a max extreme is
	 * set using {@link #setExtremes(Number, Number)}, the maxPadding will be
	 * ignored.
	 * <p>
	 * Default: 0.05.
	 */
	public Number getMaxPadding() {
		return getAttr(Attrs.maxPadding, 0.05).asNumber();
	}

	/**
	 * Sets padding of the max value relative to the length of the axis. A
	 * padding of 0.05 will make a 100px axis 5px longer. This is useful when
	 * you don't want the highest data value to appear on the edge of the plot
	 * area. When the {@link #setMax(Number)} option is set or a max extreme is
	 * set using {@link #setExtremes(Number, Number)}, the maxPadding will be
	 * ignored.
	 */
	public void setMaxPadding(Number maxPadding) {
		setAttr(Attrs.maxPadding, maxPadding, 0.05);
	}
	
	/**
	 * Returns the min color. Solid gauge only. Unless stops are set,
	 * the color to represent the minimum value of the Y axis.
	 * <p>
	 * Default: #EFEFFF.
	 */
	public Color getMinColor() {
		if (!containsKey(Attrs.minColor)) {
			setMinColor(new Color("#EFEFFF"));
		}
		return (Color) getAttr(Attrs.minColor);
	}

	/**
	 * Sets the min color. Solid gauge only. Unless stops are set,
	 * the color to represent the minimum value of the Y axis.
	 */
	public void setMinColor(Color color) {
		setAttr(Attrs.minColor, color, NOT_NULL_VALUE);
	}

	/**
	 * Sets the min color. Solid gauge only. Unless stops are set,
	 * the color to represent the minimum value of the Y axis.
	 */
	public void setMinColor(String color) {
		setMinColor(new Color(color));
	}

	/**
	 * Sets the min color. Solid gauge only. Unless stops are set,
	 * the color to represent the minimum value of the Y axis.
	 */
	public void setMinColor(LinearGradient color) {
		setMinColor(new Color(color));
	}

	/**
	 * Sets the min color. Solid gauge only. Unless stops are set,
	 * the color to represent the minimum value of the Y axis.
	 */
	public void setMinColor(RadialGradient color) {
		setMinColor(new Color(color));
	}

	/**
	 * Returns padding of the min value relative to the length of the axis. A
	 * padding of 0.05 will make a 100px axis 5px longer. This is useful when
	 * you don't want the lowest data value to appear on the edge of the plot
	 * area. When the {@link #setMin(Number)} option is set or a min extreme is
	 * set using {@link #setExtremes(Number, Number)}, the minPadding will be
	 * ignored.
	 * <p>
	 * Default: 0.05.
	 */
	public Number getMinPadding() {
		return getAttr(Attrs.minPadding, 0.05).asNumber();
	}

	/**
	 * Sets padding of the min value relative to the length of the axis. A
	 * padding of 0.05 will make a 100px axis 5px longer. This is useful when
	 * you don't want the lowest data value to appear on the edge of the plot
	 * area. When the {@link #setMin(Number)} option is set or a min extreme is
	 * set using {@link #setExtremes(Number, Number)}, the minPadding will be
	 * ignored.
	 */
	public void setMinPadding(Number minPadding) {
		setAttr(Attrs.minPadding, minPadding, 0.05);
	}

	/**
	 * Returns whether to reverse the axis so that the highest number is closest to the origin.
	 * <p>
	 * Default: false.
	 * @since 11.2.0.0
	 */
	public boolean isReversed() {
		return getAttr(Attrs.reversed, false).asBoolean();
	}

	/**
	 * Sets whether to reverse the axis so that the highest number is closest to the origin.
	 * <p>
	 * Default: false.
	 * @since 11.2.0.0
	 */
	public void setReversed(boolean reversed) {
		setAttr(Attrs.reversed, reversed, false);
	}

	/**
	 * Returns the reversedStacks. If true, the first series in a stack will be drawn on top in a
	 * positive, non-reversed Y axis. If false, the first series is in the base of the stack.
	 * <p>
	 * Default: true.
	 */
	public boolean isReversedStacks() {
		return getAttr(Attrs.reversedStacks, true).asBoolean();
	}

	/**
	 * Sets the reversedStacks. If true, the first series in a stack will be drawn on top in a
	 * positive, non-reversed Y axis. If false, the first series is in the base of the stack.
	 * <p>
	 * Default: true.
	 */
	public void setReversedStacks(boolean reversedStacks) {
		setAttr(Attrs.reversedStacks, reversedStacks, true);
	}
	
	/**
	 * Get the list of stop colors.
	 * <p>
	 * Default: null
	 * @return list of stop colors
	 * @see Stop
	 */
	public List<Stop> getStops() {
		return Generics.cast(getAttr(Attrs.stops, null).asValue());
	}

	/**
	 * Solid gauge series only. Color stops for the solid gauge.
	 * @param stops a list of stop colors
	 * @see Stop
	 */
	public void setStops(List<Stop> stops) {
		setAttr(Attrs.stops, stops);
	}
	
	/**
	 * Solid gauge series only. Color stops for the solid gauge.
	 * @param stops an array of stop colors
	 * @see Stop
	 */
	public void setStops(Stop... stops) {
		setStops(Arrays.asList(stops));
	}
	
	/**
	 * Returns the width of the grid lines extending the ticks across the plot
	 * area.
	 * <p>
	 * Default: 1.
	 */
	public Number getGridLineWidth() {
		return getAttr(Attrs.gridLineWidth, 1).asNumber();
	}

	/**
	 * Sets the width of the grid lines extending the ticks across the plot
	 * area.
	 */
	public void setGridLineWidth(Number gridLineWidth) {
		setAttr(Attrs.gridLineWidth, gridLineWidth, 1);
	}

	/**
	 * Returns if tickInterval is <code>null</code> this option sets the
	 * approximate pixel interval of the tick marks. Not applicable to
	 * categorized axis. Defaults to <code>72</code> for the Y axis and
	 * <code>100</code> for the X axis.
	 * <p>
	 * Default: 72.
	 */
	public Number getTickPixelInterval() {
		return getAttr(Attrs.tickPixelInterval, 72).asNumber();
	}

	/**
	 * Sets if tickInterval is <code>null</code> this option sets the
	 * approximate pixel interval of the tick marks. Not applicable to
	 * categorized axis. Defaults to <code>72</code> for the Y axis and
	 * <code>100</code> for the X axis.
	 * <p>
	 * Default: 72.
	 */
	public void setTickPixelInterval(Number tickPixelInterval) {
		setAttr(Attrs.tickPixelInterval, tickPixelInterval, 72);
	}

	/**
	 * Returns the axis labels show the number or category for each tick.
	 * <p>
	 * Default:
	 * 
	 * <pre>
	 * labels: {
	 *	  x: -8,
	 *	  y: 3
	 * }
	 * </pre>
	 */
	public AxisLabels getLabels() {
		AxisLabels labels = (AxisLabels) getAttr(Attrs.labels);
		if (labels == null) {
			labels = new YAxisLabels();
			setLabels(labels);
		}
		return labels;
	}

	/**
	 * Returns the width of the line marking the axis itself.
	 * <p>
	 * Default: 0.
	 */
	public Number getLineWidth() {
		return getAttr(Attrs.lineWidth, 0).asNumber();
	}

	/**
	 * Sets the width of the line marking the axis itself.
	 */
	public void setLineWidth(Number lineWidth) {
		setAttr(Attrs.lineWidth, lineWidth, NOT_NULL_VALUE);
	}

	/**
	 * Returns the axis title, showing next to the axis line.
	 * <p>
	 * Default: an instance of {@link AxisTitle}.
	 */
	public AxisTitle getTitle() {
		AxisTitle title = (AxisTitle) getAttr(Attrs.title);
		if (title == null) {
			title = new YAxisTitle();
			setAttr(Attrs.title, title);
		}
		return title;
	}

	// Override
	public void onChange(OptionDataEvent event) {
		final Optionable target = event.getTarget();

		if (target instanceof StackLabels) {
			event.addJSFunctionCall(new DeferredCall() {
				public void execute(JSFunction func) {
					JSONObject json = new JSONObject();
					json.put("stackLabels", target);
					func.callFunction("update", json);
				}
			});

			event.setCurrentTarget(this);

			fireEvent(event);
		} else {
			super.onChange(event);
		}
	}
	
	/**
	 * Get whether the grid lines should draw as a polygon with straight lines 
	 * between categories, or as circles. Can be either circle or polygon.
	 * <p>
	 * Defaults to null.
	 * @since 2.1.0
	 */
	public String getGridLineInterpolation() {
		return getAttr(Attrs.gridLineInterpolation, null).asString();
	}

	/**
	 * Set whether the grid lines should draw as a polygon with straight lines 
	 * between categories, or as circles. Can be either circle or polygon.
	 * @param gridLineInterpolation
	 * @since 2.1.0
	 */
	public void setGridLineInterpolation(String gridLineInterpolation) {
		if (!("circle".equalsIgnoreCase(gridLineInterpolation) || 
			"polygon".equalsIgnoreCase(gridLineInterpolation))) {
			throw new IllegalArgumentException("Unsupported grid line interpolation:[" + gridLineInterpolation + "]");
		}
		setAttr(Attrs.gridLineInterpolation, gridLineInterpolation);
	}
}
