/* ArcDiagramPlotOptions.java

	Purpose:
		
	Description:
		
	History:
		11:28 AM 2022/9/7, Created by jumperchen

Copyright (C) 2022 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart.plotOptions;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

import org.zkoss.chart.Color;
import org.zkoss.chart.Level;
import org.zkoss.chart.PlotAttribute;
import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.lang.Generics;

/**
 * An options for arc diagram plot.
 * <p>
 * All the options in this class support {@link DynamicalAttribute}.
 * @author jumperchen
 * @since 10.2.1.0
 */
public class ArcDiagramPlotOptions extends SeriesPlotOptions {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		borderColor, borderWidth, centeredLinks, colorByPoint, colors, equalNodes,
		levels, linkOpacity, linkWeight, minLinkWidth, nodeWidth, offset, reversed
	}

	/**
	 * Returns the color of the border surrounding each column or bar.
	 * <p>Default: {@code #ffffff}
	 */
	public String getBorderColor() {
		return getAttr(Attrs.borderColor, "#ffffff").asString();
	}

	/**
	 * Sets the color of the border surrounding each column or bar.
	 */
	public void setBorderColor(String borderColor) {
		setAttr(Attrs.borderColor, borderColor, "#ffffff");
	}

	/**
	 * Returns the width of the border surrounding each column or bar.
	 * <p>
	 * Default: {@code 0} when the columns are so dense that a border would cover the next column.
	 */
	public Number getBorderWidth() {
		return getAttr(Attrs.borderWidth, 0).asNumber();
	}

	/**
	 * Sets the width of the border surrounding each column or bar.
	 */
	public void setBorderWidth(Number borderWidth) {
		setAttr(Attrs.borderWidth, borderWidth);
	}

	/**
	 * Returns the option to center links rather than position them one after another
	 *
	 * <p>
	 * Default: {@code false}.
	 */
	public boolean isCenteredLinks() {
		return getAttr(Attrs.centeredLinks, false).asBoolean();
	}

	/**
	 * Sets the option to center links rather than position them one after another
	 */
	public void setCenteredLinks(boolean centeredLinks) {
		setAttr(Attrs.centeredLinks, centeredLinks);
	}

	/**
	 * Returns whether the chart should receive one color
	 * per series or one color per point.
	 * <p>
	 * Default: {@code true}.
	 */
	public boolean isColorByPoint() {
		return getAttr(Attrs.colorByPoint, true).asBoolean();
	}

	/**
	 * Sets whether the chart should receive one color per series or one color per point.
	 */
	public void setColorByPoint(boolean colorByPoint) {
		setAttr(Attrs.colorByPoint, colorByPoint);
	}
	/**
	 * Sets the series specific or series type specific color set to apply
	 * instead of the global colors when {@link #isColorByPoint()} is true.
	 */
	public void setColors(List<Color> colors) {
		setAttr(Attrs.colors, colors);
	}

	/**
	 * Sets the series specific or series type specific color set to apply
	 * instead of the global colors when {@link #isColorByPoint()} is true.
	 */
	public void setColors(String... colors) {
		setColors(Arrays.stream(colors).map(Color::new).collect(Collectors.toList()));
	}

	/**
	 * Sets the series specific or series type specific color set to apply
	 * instead of the global colors when {@link #isColorByPoint()} is true.
	 */
	public void setColors(Color... colors) {
		setColors(Arrays.asList(colors));
	}

	/**
	 * Returns the series specific or series type specific color set to apply
	 * instead of the global colors when {@link #isColorByPoint()} is true.
	 */
	public List<Color> getColors() {
		return Generics.cast(getAttr(Attrs.colors, null).asValue());
	}

	/**
	 * Returns whether nodes with different values should have the same size.
	 * If set to true, all nodes are calculated based on the nodePadding and current
	 * plotArea.
	 *
	 * <p>
	 * Default: {@code false}.
	 */
	public boolean isEqualNodes() {
		return getAttr(Attrs.equalNodes, false).asBoolean();
	}

	/**
	 * Sets whether nodes with different values should have the same size.
	 * If set to true, all nodes are calculated based on the nodePadding and current
	 * plotArea.
	 */
	public void setEqualNodes(boolean equalNodes) {
		setAttr(Attrs.equalNodes, equalNodes);
	}

	/**
	 * Returns options on specific levels. Takes precedence over series options,
	 * but not node and link options.
	 * <p>
	 * Default: null
	 */
	public List<Level> getLevels() {
		return Generics.cast(getAttr(Attrs.levels, null).asValue());
	}

	/**
	 * Set options on specific levels. Takes precedence over series options,
	 * but not node and link options.
	 */
	public void setLevels(List<Level> levels) {
		setAttr(Attrs.levels, levels);
	}

	/**
	 * Set options on specific levels. Takes precedence over series options,
	 * but not node and link options.
	 */
	public void setLevels(Level... levels) {
		setLevels(Arrays.asList(levels));
	}

	/**
	 * Returns linkOpacity on all points which lay on the same level.
	 * <p>
	 * Default: {@code 0.5}
	 */
	public Number getLinkOpacity() {
		return getAttr(Attrs.linkOpacity, 0.5).asNumber();
	}

	/**
	 * Sets linkOpacity on all points which lay on the same level.
	 */
	public void setLinkOpacity(Number linkOpacity) {
		setAttr(Attrs.linkOpacity, linkOpacity);
	}

	/**
	 * The global link weight. If not set, width is calculated per link,
	 * depending on the weight value. (Requires arcdiagram)
	 *
	 * <p>Defaults to {@code null}.
	 */
	public Number getLinkWeight() {
		return getAttr(Attrs.linkWeight, null).asNumber();
	}

	/**
	 * The global link weight. If not set, width is calculated per link,
	 * depending on the weight value. (Requires arcdiagram)
	 *
	 * <p>Defaults to {@code null}.
	 */
	public void setLinkWeight(Number linkWeight) {
		setAttr(Attrs.linkWeight, linkWeight);
	}

	/**
	 * Returns the minimal width for a line of a sankey.
	 * <p>
	 * Default: {@code 0}
	 */
	public Number getMinLinkWidth() {
		return getAttr(Attrs.minLinkWidth, 0).asNumber();
	}

	/**
	 * Sets the minimal width for a line of a sankey.
	 */
	public void setMinLinkWidth(Number minLinkWidth) {
		setAttr(Attrs.minLinkWidth, minLinkWidth);
	}

	/**
	 * Returns the pixel width of each node in a sankey diagram or dependency wheel,
	 * or the height in case the chart is inverted.
	 * <p>
	 * Default: {@code 20}
	 */
	public Number getNodeWidth() {
		return getAttr(Attrs.nodeWidth, 20).asNumber();
	}

	/**
	 * Sets the pixel width of each node in a sankey diagram or dependency wheel,
	 * or the height in case the chart is inverted.
	 */
	public void setNodeWidth(Number nodeWidth) {
		setAttr(Attrs.nodeWidth, nodeWidth, 20);
	}

	/**
	 * Returns the offset of an arc diagram nodes column in relation to the {@code plotArea}.
	 * The offset equal to 50% places nodes in the center of a chart. By default
	 * the series is placed so that the biggest node is touching the bottom border
	 * of the {@code plotArea}.
	 * <p>Default: {@code "100%"}
	 */
	public String getOffset() {
		return getAttr(Attrs.offset, "100%").asString();
	}

	/**
	 * Sets the offset of an arc diagram nodes column in relation to the {@code plotArea}.
	 * The offset equal to 50% places nodes in the center of a chart. By default
	 * the series is placed so that the biggest node is touching the bottom border
	 * of the {@code plotArea}.
	 * <p>Default: {@code "100%"}
	 */
	public void setOffset(String offset) {
		setAttr(Attrs.offset, offset);
	}

	/**
	 * Returns whether the series should be placed on the other side of the plotArea.
	 *
	 * <p>
	 * Default: {@code false}.
	 */
	public boolean isReversed() {
		return getAttr(Attrs.reversed, false).asBoolean();
	}

	/**
	 * Sets whether the series should be placed on the other side of the plotArea.
	 */
	public void setReversed(boolean reversed) {
		setAttr(Attrs.reversed, reversed);
	}
}