package org.zkoss.chart.plotOptions;

import java.util.Arrays;
import java.util.List;

import org.zkoss.chart.Charts;
import org.zkoss.chart.Color;
import org.zkoss.chart.LinearGradient;
import org.zkoss.chart.PlotAttribute;
import org.zkoss.chart.RadialGradient;
import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.lang.Generics;


/**
 * An option for heatmap plot.
 * <p>
 * All the options in this class support {@link DynamicalAttribute}.
 * 
 * @author hanhsu
 */
public class HeatmapPlotOptions extends SeriesPlotOptions {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		borderColor, borderRadius, borderWidth, colors, colsize,
		interpolation, nullColor, rowsize, pointPadding
	}

	/**
	 * Returns whether make the heatmap render its data points as an interpolated image.
	 * <p>Default: {@code false}
	 * @since 11.2.0.0
	 */
	public boolean isInterpolation() {
		return getAttr(Attrs.interpolation, false).asBoolean();
	}

	/**
	 * Sets whether make the heatmap render its data points as an interpolated image.
	 * @since 11.2.0.0
	 */
	public void setInterpolation(Boolean interpolation) {
		setAttr(Attrs.interpolation, interpolation, false);
	}

	/**
	 * Returns the color of the border surrounding each column or bar.
	 * <p>
	 * Default: "#FFFFFF".
	 */
	public Color getBorderColor() {
		if (!containsKey(Attrs.borderColor)) {
			setBorderColor("#FFFFFF");
		}
		return (Color) getAttr(Attrs.borderColor);
	}

	/**
	 * Sets the color of the border surrounding each column or bar.
	 */
	public void setBorderColor(Color color) {
		setAttr(Attrs.borderColor, color, NOT_NULL_VALUE);
	}

	/**
	 * Sets the color of the border surrounding each column or bar.
	 */
	public void setBorderColor(String color) {
		setBorderColor(new Color(color));
	}

	/**
	 * Sets the color of the border surrounding each column or bar.
	 */
	public void setBorderColor(LinearGradient color) {
		setBorderColor(new Color(color));
	}

	/**
	 * Sets the color of the border surrounding each column or bar.
	 */
	public void setBorderColor(RadialGradient color) {
		setBorderColor(new Color(color));
	}
	
	/**
	 * Returns the corner radius of the border surrounding each column or bar.
	 * <p>
	 * Default: 0.
	 * 
	 * @return corner radius of the border surrounding each column or bar.
	 */
	public Object getBorderRadius() {
		return getAttr(Attrs.borderRadius, 0).asValue();
	}

	/**
	 * Returns the width of the border surrounding each column or bar.
	 * <p>
	 * Default: 1.
	 * 
	 * @return width of the border surrounding each column or bar.
	 */
	public Number getBorderWidth() {
		return getAttr(Attrs.borderWidth, 1).asNumber();
	}

	/**
	 * Sets the width of the border surrounding each column or bar.
	 */
	public void setBorderWidth(Number borderWidth) {
		setAttr(Attrs.borderWidth, borderWidth, 1);
	}
	
	/**
	 * Returns a series specific or series type specific color set to apply
	 * instead of the global {@link Charts#getColors()} when
	 * {@link #setColorByPoint(boolean)} is true.
	 */
	public List<Color> getColors() {
		return Generics.cast(getAttr(Attrs.colors, null).asValue());
	}

	/**
	 * Sets a series specific or series type specific color set to apply
	 * instead of the global {@link Charts#getColors()} when
	 * {@link #setColorByPoint(boolean)} is true.
	 */
	public void setColors(List<Color> colors) {
		setAttr(Attrs.colors, colors);
	}

	/**
	 * Sets a series specific or series type specific color set to apply
	 * instead of the global {@link Charts#getColors()} when
	 * {@link #setColorByPoint(boolean)} is true.
	 */
	public void setColors(String... source) {
		Color[] colors = new Color[source.length];
		int i = 0;
		for (String s : source)
			colors[i++] = new Color(s);
		setColors(Arrays.asList(colors));
	}
	
	/**
	 * Returns the column size - how many X axis units each column in the heatmap should span.
	 * <p>
	 * Default: 1.
	 * 
	 * @return column size - how many X axis units each column in the heatmap should span.
	 */
	public Number getColsize() {
		return getAttr(Attrs.colsize, 0).asNumber();
	}

	/**
	 * Sets the column size - how many X axis units each column in the heatmap should span.
	 */
	public void setColsize(Number colsize) {
		setAttr(Attrs.colsize, colsize, 0);
	}

	/**
	 * Returns the color applied to null points.
	 * <p>
	 * Default: "#f7f7f7".
	 * @since 7.2.1.0
	 */
	public Color getNullColor() {
		if (!containsKey(Attrs.nullColor)) {
			setBorderColor("#f7f7f7");
		}
		return (Color) getAttr(Attrs.nullColor);
	}

	/**
	 * Sets the color applied to null points.
	 * @since 7.2.1.0
	 */
	public void setNullColor(Color color) {
		setAttr(Attrs.nullColor, color, NOT_NULL_VALUE);
	}

	/**
	 * Sets the color applied to null points.
	 * @since 7.2.1.0
	 */
	public void setNullColor(String color) {
		setNullColor(new Color(color));
	}

	/**
	 * Sets the color applied to null points.
	 * @since 7.2.1.0
	 */
	public void setNullColor(LinearGradient color) {
		setNullColor(new Color(color));
	}

	/**
	 * Sets the color of the border surrounding each column or bar.
	 * @since 7.2.1.0
	 */
	public void setNullColor(RadialGradient color) {
		setNullColor(new Color(color));
	}
	
	/**
	 * Returns the row size - how many Y axis units each heatmap row should span.
	 * <p>
	 * Default: 1.
	 * 
	 * @return row size - how many Y axis units each heatmap row should span.
	 */
	public Number getRowsize() {
		return getAttr(Attrs.rowsize, 1).asNumber();
	}

	/**
	 * Sets the row size - how many Y axis units each heatmap row should span.
	 */
	public void setRowsize(Number rowsize) {
		setAttr(Attrs.rowsize, rowsize);
	}

	/**
	 * Returns padding between each column or bar, in x axis units.
	 * <p> Default: 0
	 * @since 10.2.1.0
	 */
	public Number getPointPadding() {
		return getAttr(Attrs.pointPadding, 0).asNumber();
	}

	/**
	 * Sets padding between each column or bar, in x axis units. 'pointPadding
	 * padding between each column or bar, in x axis units.
	 * @since 10.2.1.0
	 */
	public void setPointPadding(Number pointPadding) {
		setAttr(Attrs.pointPadding, pointPadding);
	}
}
