/* OrganizationPlotOptions.java

	Purpose:

	Description:

	History:
		Mon Nov 25 16:24:35 CST 2019, Created by rudyhuang

Copyright (C) 2019 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart.plotOptions;

import java.util.Arrays;
import java.util.List;

import org.zkoss.chart.Color;
import org.zkoss.chart.Level;
import org.zkoss.chart.PlotAttribute;
import org.zkoss.chart.Series;
import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.lang.Generics;

/**
 * An option for organization plot.
 * <p>
 * All the options in this class support {@link DynamicalAttribute}.
 *
 * @author rudyhuang
 * @since 7.2.1.0
 */
public class OrganizationPlotOptions extends SankeyPlotOptions {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		borderColor,
		borderWidth,
		hangingIndent,
		hangingIndentTranslation,
		levels,
		linkColor,
		linkLineWidth,
		linkRadius,
		minNodeLength,
		nodeWidth
	}

	/**
	 * Returns the color of the border surrounding each slice.
	 * <p>
	 * Default: #666666
	 * @since 10.2.1.0
	 */
	public Color getBorderColor() {
		return (Color) getAttr(Attrs.borderColor, new Color("#666666")).asValue();
	}

	/**
	 * Sets the color of the border surrounding each slice.
	 * Default: #666666
	 * @since 10.2.1.0
	 */
	public void setBorderColor(Color borderColor) {
		setAttr(Attrs.borderColor, borderColor);
	}

	/**
	 * Sets the color of the border surrounding each slice.
	 * Default: #666666
	 * @since 10.2.1.0
	 */
	public void setBorderColor(String borderColor) {
		setBorderColor(new Color(borderColor));
	}
	/**
	 * Returns the width of the border surrounding each column or bar.
	 * <p>
	 * Default: {@code 1} when the columns are so dense that a border would cover the next column.
	 * @since 10.2.1.0
	 */
	public Number getBorderWidth() {
		return getAttr(Attrs.borderWidth, 1).asNumber();
	}

	/**
	 * Sets the width of the border surrounding each column or bar.
	 * <p>
	 * Default: {@code 1} when the columns are so dense that a border would cover the next column.
	 * @since 10.2.1.0
	 */
	public void setBorderWidth(Number borderWidth) {
		setAttr(Attrs.borderWidth, borderWidth);
	}
	@Override
	protected DataLabels newDataLabels() {
		return new OrganizationDataLabels();
	}

	@Override
	public OrganizationDataLabels getDataLabels() {
		return (OrganizationDataLabels) super.getDataLabels();
	}

	/**
	 * Returns the indentation in pixels of hanging nodes, nodes which parent has layout set to hanging.
	 * <p>
	 * Default: 20
	 */
	public Number getHangingIndent() {
		return getAttr(Attrs.hangingIndent, 20).asNumber();
	}

	/**
	 * Sets the indentation in pixels of hanging nodes, nodes which parent has layout set to hanging.
	 */
	public void setHangingIndent(Number hangingIndent) {
		setAttr(Attrs.hangingIndent, hangingIndent);
	}

	/**
	 * Defines the indentation of a hanging layout parent's children. (Requires organization)
	 * <p>
	 * Possible options:
	 * <ul><li>
	 * {@code "inherit"} (default): Only the first child adds the indentation,
	 * children of a child with indentation inherit the indentation.
	 * </li><li>
	 * {@code "cumulative"}: All children of a child with indentation add its own indent.
	 * The option may cause overlapping of nodes. Then use shrink option:
	 * </li><li>
	 * {@code "shrink"}: Nodes shrink by the hangingIndent value until they reach the minNodeLength.
	 * </li></ul>
	 * <p>
	 * Defaults to {@code "inherit"}.
	 * @since 10.2.1.0
	 */
	public String getHangingIndentTranslation() {
		return getAttr(Attrs.hangingIndentTranslation, "inherit").asString();
	}

	/**
	 * Defines the indentation of a hanging layout parent's children. (Requires organization)
	 * <p>
	 * Possible options:
	 * <ul><li>
	 * {@code "inherit"} (default): Only the first child adds the indentation,
	 * children of a child with indentation inherit the indentation.
	 * </li><li>
	 * {@code "cumulative"}: All children of a child with indentation add its own indent.
	 * The option may cause overlapping of nodes. Then use shrink option:
	 * </li><li>
	 * {@code "shrink"}: Nodes shrink by the hangingIndent value until they reach the minNodeLength.
	 * </li></ul>
	 * <p>
	 * Defaults to {@code "inherit"}.
	 * @since 10.2.1.0
	 */
	public void setHangingIndentTranslation(String hangingIndentTranslation) {
		if (!("inherit".equals(hangingIndentTranslation) || "cumulative".equals(hangingIndentTranslation)
		|| "shrink".equals(hangingIndentTranslation))) {
			throw new IllegalArgumentException("Not supported value. [" + hangingIndentTranslation + "]");
		}
		setAttr(Attrs.hangingIndentTranslation, hangingIndentTranslation);
	}

	/**
	 * Returns options on specific levels. Takes precedence over series options,
	 * but not point options.
	 * <p>
	 * Default: null
	 */
	public List<Level> getLevels() {
		return Generics.cast(getAttr(Attrs.levels, null).asValue());
	}

	/**
	 * Sets options on specific levels. Takes precedence over series options,
	 * but not point options.
	 */
	public void setLevels(List<Level> levels) {
		setAttr(Attrs.levels, levels);
	}

	/**
	 * Set options on specific levels. Takes precedence over series options,
	 * but not node and link options.
	 * @since 10.2.1.0
	 */
	public void setLevels(Level... levels) {
		setLevels(Arrays.asList(levels));
	}

	/**
	 * Returns the color of the links between nodes.
	 * <p>
	 * Default: #666666
	 */
	public Color getLinkColor() {
		return (Color) getAttr(Attrs.linkColor, new Color("#666666")).asValue();
	}

	/**
	 * Sets the color of the links between nodes.
	 */
	public void setLinkColor(Color linkColor) {
		setAttr(Attrs.linkColor, linkColor);
	}

	/**
	 * Sets the color of the links between nodes.
	 */
	public void setLinkColor(String linkColor) {
		setLinkColor(new Color(linkColor));
	}

	/**
	 * Returns the line width of the links connecting nodes, in pixels.
	 * <p>
	 * Default: 1
	 */
	public Number getLinkLineWidth() {
		return getAttr(Attrs.linkLineWidth, 1).asNumber();
	}

	/**
	 * Sets the line width of the links connecting nodes, in pixels.
	 */
	public void setLinkLineWidth(Number linkLineWidth) {
		setAttr(Attrs.linkLineWidth, linkLineWidth);
	}

	/**
	 * Returns radius for the rounded corners of the links between nodes.
	 * <p>
	 * Default: 10
	 */
	public Number getLinkRadius() {
		return getAttr(Attrs.linkRadius, 10).asNumber();
	}

	/**
	 * Sets radius for the rounded corners of the links between nodes.
	 */
	public void setLinkRadius(Number linkRadius) {
		setAttr(Attrs.linkRadius, linkRadius);
	}

	/**
	 * In a horizontal chart, returns the width of the nodes in pixels.
	 * Node that most organization charts are vertical, so the name of this option is counterintuitive.
	 * <p>
	 * Default: 50
	 */
	@Override
	public Number getNodeWidth() {
		return getAttr(Attrs.nodeWidth, 50).asNumber();
	}

	/**
	 * In a horizontal chart, the minimum width of the hanging nodes only, in pixels.
	 * In a vertical chart, the minimum height of the hanging nodes only, in pixels too.
	 *
	 * <p>
	 * Note: Used only when {@link #getHangingIndentTranslation()} is set to {@code "shrink").
	 * <p>
	 * Defaults to {@code 10}.
	 * @since 10.2.1.0
	 */
	public Number getMinNodeLength() {
		return getAttr(Attrs.minNodeLength, 10).asNumber();
	}

	/**
	 * In a horizontal chart, the minimum width of the hanging nodes only, in pixels.
	 * In a vertical chart, the minimum height of the hanging nodes only, in pixels too.
	 *
	 * <p>
	 * Note: Used only when {@link #getHangingIndentTranslation()} is set to {@code "shrink").
	 * <p>
	 * Defaults to {@code 10}.
	 * @since 10.2.1.0
	 */
	public void setMinNodeLength(Number minNodeLength) {
		setAttr(Attrs.minNodeLength, minNodeLength);
	}
}
