/* PictorialPlotOptions.java

        Purpose:
                
        Description:
                
        History:
                Thu Sep 14 12:45:04 CST 2023, Created by jamson

Copyright (C) 2023 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart.plotOptions;

import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;
import java.util.stream.Collectors;

import org.zkoss.chart.Color;
import org.zkoss.chart.Path;
import org.zkoss.chart.PlotAttribute;
import org.zkoss.chart.Tooltip;
import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.lang.Generics;

/**
 * An options for pictorial plot.
 * <p>
 * All the options in this class support {@link DynamicalAttribute}.
 * @author Jamson Chan
 * @since 11.2.0.0
 */
public class PictorialPlotOptions extends SeriesPlotOptions {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		borderColor, borderWidth, colors, cropThreshold, edgeColor, edgeWidth, grouping, groupZPadding, includeInDataExport,
		maxPointWidth, minPointLength, paths, pointPadding, pointRange, pointWidth,
		showInLegend, stickyTracking

	}

	/**
	 * Returns the color of the border surrounding each column or bar.
	 * <p>Default: {@code #ffffff}
	 */
	public String getBorderColor() {
		return getAttr(PictorialPlotOptions.Attrs.borderColor, "#ffffff").asString();
	}

	/**
	 * Sets the color of the border surrounding each column or bar.
	 */
	public void setBorderColor(String borderColor) {
		setAttr(PictorialPlotOptions.Attrs.borderColor, borderColor, "#ffffff");
	}

	/**
	 * Returns the width of the border surrounding each column or bar.
	 * <p>
	 * Default: {@code 0} when the columns are so dense that a border would cover the next column.
	 */
	public Number getBorderWidth() {
		return getAttr(PictorialPlotOptions.Attrs.borderWidth, 0).asNumber();
	}

	/**
	 * Sets the width of the border surrounding each column or bar.
	 */
	public void setBorderWidth(Number borderWidth) {
		setAttr(PictorialPlotOptions.Attrs.borderWidth, borderWidth);
	}

	/**
	 * Sets the series specific or series type specific color set to apply
	 * instead of the global colors when {@link #isColorByPoint()} is true.
	 */
	public void setColors(List<Color> colors) {
		setAttr(PictorialPlotOptions.Attrs.colors, colors);
	}

	/**
	 * Sets the series specific or series type specific color set to apply
	 * instead of the global colors when {@link #isColorByPoint()} is true.
	 */
	public void setColors(String... colors) {
		setColors(Arrays.stream(colors).map(Color::new).collect(Collectors.toList()));
	}

	/**
	 * Sets the series specific or series type specific color set to apply
	 * instead of the global colors when {@link #isColorByPoint()} is true.
	 */
	public void setColors(Color... colors) {
		setColors(Arrays.asList(colors));
	}

	/**
	 * Returns the series specific or series type specific color set to apply
	 * instead of the global colors when {@link #isColorByPoint()} is true.
	 */
	public List<Color> getColors() {
		return Generics.cast(getAttr(PictorialPlotOptions.Attrs.colors, null).asValue());
	}

	/**
	 * Returns when the series contains less points than the crop threshold, all
	 * points are drawn, even if the points fall outside the visible plot area
	 * at the current zoom. The advantage of drawing all points (including
	 * markers and columns), is that animation is performed on updates. On the
	 * other hand, when the series contains more points than the crop threshold,
	 * the series data is cropped to only contain points that fall within the
	 * plot area. The advantage of cropping away invisible points is to increase
	 * performance on large series.
	 * <p>
	 * Default: 50
	 */
	public Number getCropThreshold() {
		return getAttr(PictorialPlotOptions.Attrs.cropThreshold, 50).asNumber();
	}

	/**
	 * Sets when the series contains less points than the crop threshold, all
	 * points are drawn, even if the points fall outside the visible plot area
	 * at the current zoom. The advantage of drawing all points (including
	 * markers and columns), is that animation is performed on updates. On the
	 * other hand, when the series contains more points than the crop threshold,
	 * the series data is cropped to only contain points that fall within the
	 * plot area. The advantage of cropping away invisible points is to increase
	 * performance on large series.
	 */
	public void setCropThreshold(Number cropThreshold) {
		setAttr(PictorialPlotOptions.Attrs.cropThreshold, cropThreshold, 50);
	}

	/**
	 * 3D columns only. The color of the edges. Similar to {@code borderColor},
	 * except it defaults to the same color as the column.
	 * <p>Default: {@code null}
	 */
	public Color getEdgeColor() {
		return (Color) getAttr(PictorialPlotOptions.Attrs.edgeColor, null).asValue();
	}

	/**
	 * 3D columns only. The color of the edges. Similar to {@code borderColor},
	 * except it defaults to the same color as the column.
	 * <p>Default: {@code null}
	 */
	public void setEdgeColor(Color color) {
		setAttr(PictorialPlotOptions.Attrs.edgeColor, color);
	}

	/**
	 * 3D columns only. The color of the edges. Similar to {@code borderColor},
	 * except it defaults to the same color as the column.
	 * <p>Default: {@code null}
	 */
	public void setEdgeColor(String color) {
		setColor(new Color(color));
	}

	/**
	 * 3D columns only. Returns the width of the colored edges.
	 * <p>
	 * Default: 1
	 */
	public Number getEdgeWidth() {
		return getAttr(PictorialPlotOptions.Attrs.edgeWidth, 1).asNumber();
	}

	/**
	 * 3D columns only. Sets the width of the colored edges.
	 */
	public void setEdgeWidth(Number edgeWidth) {
		setAttr(PictorialPlotOptions.Attrs.edgeWidth, edgeWidth);
	}

	/**
	 * Returns whether to group non-stacked columns or to let them render independent
	 * of each other. Non-grouped columns will be laid out individually and overlap each other.
	 * <p>Default: {@code true}
	 */
	public boolean isGrouping() {
		return getAttr(PictorialPlotOptions.Attrs.grouping, true).asBoolean();
	}

	/**
	 * Sets whether to group non-stacked columns or to let them render independent
	 * of each other. Non-grouped columns will be laid out individually and overlap each other.
	 * <p>Default: {@code true}
	 */
	public void setGrouping(boolean grouping) {
		setAttr(PictorialPlotOptions.Attrs.grouping, grouping);
	}

	/**
	 * Returns the spacing between columns on the Z Axis in a 3D chart.
	 * <p>Default: 1
	 */
	public Number getGroupZPadding() {
		return getAttr(PictorialPlotOptions.Attrs.groupZPadding, 1).asNumber();
	}

	/**
	 * Sets the spacing between columns on the Z Axis in a 3D chart.
	 */
	public void setGroupZPadding(Number groupZPadding) {
		setAttr(PictorialPlotOptions.Attrs.groupZPadding, groupZPadding, 1);
	}

	/**
	 * Returns the maximum allowed pixel width for a column,
	 * translated to the height of a bar in a bar chart.
	 * This prevents the columns from becoming too wide when there is a small
	 * number of points in the chart.
	 * <p>Default: null
	 */
	public Number getMaxPointWidth() {
		return getAttr(PictorialPlotOptions.Attrs.maxPointWidth, null).asNumber();
	}

	/**
	 * Sets the maximum allowed pixel width for a column,
	 * translated to the height of a bar in a bar chart.
	 * This prevents the columns from becoming too wide when there is a small
	 * number of points in the chart.
	 */
	public void setMaxPointWidth(Number maxPointWidth) {
		setAttr(PictorialPlotOptions.Attrs.maxPointWidth, maxPointWidth);
	}

	/**
	 * Returns the minimal height for a column or width for a bar. By default, 0
	 * values are not shown. To visualize a 0 (or close to zero) point, set the
	 * minimal point length to a pixel value like 3. In stacked column charts,
	 * minPointLength might not be respected for tightly packed values.
	 * <p>Default: 0
	 */
	public Number getMinPointLength() {
		return getAttr(PictorialPlotOptions.Attrs.minPointLength, 0).asNumber();
	}

	/**
	 * Sets the minimal height for a column or width for a bar. By default, 0
	 * values are not shown. To visualize a 0 (or close to zero) point, set the
	 * minimal point length to a pixel value like 3. In stacked column charts,
	 * minPointLength might not be respected for tightly packed values.
	 */
	public void setMinPointLength(Number minPointLength) {
		setAttr(PictorialPlotOptions.Attrs.minPointLength, minPointLength, 0);
	}

	/**
	 * Returns the size of paths.
	 */
	public int getPathsSize() {
		List<String> list = Generics.cast(getAttr(Attrs.paths));
		return list == null ? 0 : list.size();
	}

	/**
	 * Returns the Path options, if any, Otherwise, create a new one.
	 * <p>
	 * Returns the first node (index 0) by default.
	 */
	public Path getPaths() {
		return getPaths(0);
	}

	/**
	 * Returns the Path options from given index, if any, Otherwise,
	 * create a new one.
	 * <p>
	 * The paths include options describing the point image.
	 */
	public Path getPaths(int index) {
		List<Path> list = Generics.cast(getAttr(Attrs.paths));
		if (list == null) {
			list = new LinkedList<Path>();
			setAttr(Attrs.paths, list);
		}

		int size = list.size();
		if (size <= index) {
			for (int i = index - size + 1; i > 0; i--) {
				list.add(new Path());
			}
		}
		return list.get(index);
	}

	/**
	 * Returns padding between each column or bar, in x axis units.
	 * <p> Default: 0
	 */
	public Number getPointPadding() {
		return getAttr(PictorialPlotOptions.Attrs.pointPadding, 0.1).asNumber();
	}

	/**
	 * Sets padding between each column or bar, in x axis units. 'pointPadding
	 * padding between each column or bar, in x axis units.
	 */
	public void setPointPadding(Number pointPadding) {
		setAttr(PictorialPlotOptions.Attrs.pointPadding, pointPadding);
	}

	/**
	 * Returns the X axis range that each point is valid for. This determines
	 * the width of the column. On a categorized axis, the range will be 1 by
	 * default (one category unit). On linear and datetime axes, the range will
	 * be computed as the distance between the two closest data points.
	 * <p> Default: null
	 */
	public Number getPointRange() {
		return getAttr(PictorialPlotOptions.Attrs.pointRange, null).asNumber();
	}

	/**
	 * Sets the X axis range that each point is valid for. This determines the
	 * width of the column. On a categorized axis, the range will be 1 by
	 * default (one category unit). On linear and datetime axes, the range will
	 * be computed as the distance between the two closest data points.
	 */
	public void setPointRange(Number pointRange) {
		setAttr(PictorialPlotOptions.Attrs.pointRange, pointRange);
	}

	/**
	 * Returns a pixel value specifying a fixed width for each column or bar.
	 * When <code>null</code>, the width is calculated from the
	 * <code>pointPadding</code> and <code>groupPadding</code>.
	 * <p>Default: null
	 */
	public Number getPointWidth() {
		return getAttr(PictorialPlotOptions.Attrs.pointWidth, null).asNumber();
	}

	/**
	 * Sets a pixel value specifying a fixed width for each column or bar. When
	 * <code>null</code>, the width is calculated from the
	 * <code>pointPadding</code> and <code>groupPadding</code>.
	 */
	public void setPointWidth(int pointWidth) {
		setAttr(PictorialPlotOptions.Attrs.pointWidth, pointWidth);
	}

	/**
	 * Returns whether to display this particular series or series type in the
	 * legend. Pies are not shown in the legend by default.
	 */
	public boolean isShowInLegend() {
		return getAttr(PictorialPlotOptions.Attrs.showInLegend, null).asBoolean();
	}

	/**
	 * Sets whether to display this particular series or series type in the
	 * legend. Pies are not shown in the legend by default.
	 */
	public void setShowInLegend(boolean showInLegend) {
		setAttr(PictorialPlotOptions.Attrs.showInLegend, showInLegend);
	}

	/**
	 * Returns sticky tracking of mouse events. When true, the
	 * <code>mouseOut</code> event on a series isn't triggered until the mouse
	 * moves over another series, or out of the plot area. When false, the
	 * <code>mouseOut</code> event on a series is triggered when the mouse
	 * leaves the area around the series' graph or markers. This also implies
	 * the tooltip. When <code>stickyTracking</code> is false and
	 * {@link Tooltip#isShared()} is false, the tooltip will be hidden when
	 * moving the mouse between series. Defaults to true for line and area type
	 * series, but to false for columns, pies etc.
	 * <p>Note: The boost module will force this option because of technical limitations.
	 * <p>Default: {@code false}
	 */
	public boolean isStickyTracking() {
		return getAttr(PictorialPlotOptions.Attrs.stickyTracking, false).asBoolean();
	}

	/**
	 * Sets sticky tracking of mouse events. When true, the
	 * <code>mouseOut</code> event on a series isn't triggered until the mouse
	 * moves over another series, or out of the plot area. When false, the
	 * <code>mouseOut</code> event on a series is triggered when the mouse
	 * leaves the area around the series' graph or markers. This also implies
	 * the tooltip. When <code>stickyTracking</code> is false and
	 * {@link Tooltip#isShared()} is false, the tooltip will be hidden when
	 * moving the mouse between series. Defaults to true for line and area type
	 * series, but to false for columns, pies etc.
	 */
	public void setStickyTracking(boolean stickyTracking) {
		setAttr(PictorialPlotOptions.Attrs.stickyTracking, stickyTracking);
	}


}
