/* SankeyPlotOptions.java

	Purpose:
		
	Description:
		
	History:
		Wed Nov 27 14:55:53 CST 2019, Created by rudyhuang

Copyright (C) 2019 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart.plotOptions;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

import org.zkoss.chart.Color;
import org.zkoss.chart.Level;
import org.zkoss.chart.PlotAttribute;
import org.zkoss.chart.Series;
import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.lang.Generics;

/**
 * An option for sankey plot
 *
 * @author rudyhuang
 * @since 7.2.1.0
 */
public class SankeyPlotOptions extends SeriesPlotOptions {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		borderColor,
		centerInCategory,
		colors,
		curveFactor,
		levels,
		linkColorMode,
		linkOpacity,
		minLinkWidth,
		nodeAlignment,
		nodePadding,
		nodeWidth
	}

	/**
	 * Sets the series specific or series type specific color set to apply
	 * instead of the global colors when {@link #isColorByPoint()} is true.
	 * @since 10.2.1.0
	 */
	public void setColors(List<Color> colors) {
		setAttr(Attrs.colors, colors);
	}

	/**
	 * Sets the series specific or series type specific color set to apply
	 * instead of the global colors when {@link #isColorByPoint()} is true.
	 * @since 10.2.1.0
	 */
	public void setColors(String... colors) {
		setColors(Arrays.stream(colors).map(Color::new).collect(Collectors.toList()));
	}

	/**
	 * Sets the series specific or series type specific color set to apply
	 * instead of the global colors when {@link #isColorByPoint()} is true.
	 * @since 10.2.1.0
	 */
	public void setColors(Color... colors) {
		setColors(Arrays.asList(colors));
	}

	/**
	 * Returns the series specific or series type specific color set to apply
	 * instead of the global colors when {@link #isColorByPoint()} is true.
	 * @since 10.2.1.0
	 */
	public List<Color> getColors() {
		return Generics.cast(getAttr(Attrs.colors, null).asValue());
	}

	/**
	 * When true, the columns will center in the category, ignoring null or missing points.
	 * When false, space will be reserved for null or missing points.
	 *
	 * <p>
	 * Default: {@code false}.
	 * @since 10.2.1.0
	 */
	public boolean isCenterInCategory() {
		return getAttr(Attrs.centerInCategory, false).asBoolean();
	}

	/**
	 * When true, the columns will center in the category, ignoring null or missing points.
	 * When false, space will be reserved for null or missing points.
	 * @since 10.2.1.0
	 */
	public void setCenterInCategory(boolean centerInCategory) {
		setAttr(Attrs.centerInCategory, centerInCategory);
	}

	/**
	 * Returns the color of the border surrounding each slice.
	 * <p>
	 * Default: #FFFFFF
	 * @since 10.2.1.0
	 */
	public Color getBorderColor() {
		return (Color) getAttr(Attrs.borderColor, new Color("#FFFFFF")).asValue();
	}

	/**
	 * Sets the color of the border surrounding each slice.
	 * Default: #FFFFFF
	 * @since 10.2.1.0
	 */
	public void setBorderColor(Color borderColor) {
		setAttr(Attrs.borderColor, borderColor);
	}

	/**
	 * Sets the color of the border surrounding each slice.
	 * Default: #FFFFFF
	 * @since 10.2.1.0
	 */
	public void setBorderColor(String borderColor) {
		setBorderColor(new Color(borderColor));
	}

	/**
	 * Higher numbers makes the links in a sankey diagram or dependency wheelrender more curved.
	 * A curveFactor of 0 makes the lines straight.
	 * <p>
	 * Default: 0.33
	 */
	public Number getCurveFactor() {
		return getAttr(Attrs.curveFactor, 0.33).asNumber();
	}

	/**
	 * Higher numbers makes the links in a sankey diagram or dependency wheelrender more curved.
	 * A curveFactor of 0 makes the lines straight.
	 */
	public void setCurveFactor(Number curveFactor) {
		setAttr(Attrs.curveFactor, curveFactor);
	}

	@Override
	protected DataLabels newDataLabels() {
		return new SankeyDataLabels();
	}

	@Override
	public SankeyDataLabels getDataLabels() {
		return (SankeyDataLabels) super.getDataLabels();
	}

	/**
	 * Set options on specific levels. Takes precedence over series options,
	 * but not point options.
	 * <p>
	 * Default: null
	 */
	public List<Level> getLevels() {
		return Generics.cast(getAttr(Attrs.levels, null).asValue());
	}

	/**
	 * @param levels
	 * @see #getLevels()
	 */
	public void setLevels(List<Level> levels) {
		setAttr(Attrs.levels, levels);
	}

	/**
	 * Set options on specific levels. Takes precedence over series options,
	 * but not node and link options.
	 * @since 10.2.1.0
	 */
	public void setLevels(Level... levels) {
		setLevels(Arrays.asList(levels));
	}

	/**
	 * Returns color mode for sankey links.
	 * (Can be <code>"from"</code>, <code>"gradient"</code>, <code>"to"</code>)
	 * <p>Available options:
	 * <ul>
	 *     <li><code>from</code> color of the sankey link will be the same as the 'from node'</li>
	 *     <li><code>gradient</code> color of the sankey link will be set to gradient between
	 *     colors of 'from node' and 'to node'</li>
	 *     <li><code>to</code> color of the sankey link will be same as the 'to node'.</li>
	 * </ul>
	 * Default: "from"
	 * @since 11.2.0.0
	 */
	public String getLinkColorMode() {
		return getAttr(Attrs.linkColorMode, "from").asString();
	}

	/**
	 * Sets color mode for sankey links.
	 * (Can be <code>"from"</code>, <code>"gradient"</code>, <code>"to"</code>)
	 * <p>Available options:
	 * <ul>
	 *     <li><code>from</code> color of the sankey link will be the same as the 'from node'</li>
	 *     <li><code>gradient</code> color of the sankey link will be set to gradient between
	 *     colors of 'from node' and 'to node'</li>
	 *     <li><code>to</code> color of the sankey link will be same as the 'to node'.</li>
	 * </ul>
	 * @since 11.2.0.0
	 */
	public void setLinkColorMode(String linkColorMode) {
		setAttr(Attrs.linkColorMode, linkColorMode);
	}

	/**
	 * Returns opacity for the links between nodes in the sankey diagram.
	 * <p>
	 * Default: 0.5
	 */
	public Number getLinkOpacity() {
		return getAttr(Attrs.linkOpacity, 0.5).asNumber();
	}

	/**
	 * Sets opacity for the links between nodes in the sankey diagram.
	 */
	public void setLinkOpacity(Number linkOpacity) {
		setAttr(Attrs.linkOpacity, linkOpacity);
	}

	/**
	 * Returns the minimal width for a line of a sankey. By default, 0 values are not shown.
	 * <p>
	 * Default: 0
	 */
	public Number getMinLinkWidth() {
		return getAttr(Attrs.minLinkWidth, 0).asNumber();
	}

	/**
	 * Sets the minimal width for a line of a sankey. By default, 0 values are not shown.
	 */
	public void setMinLinkWidth(Number minLinkWidth) {
		setAttr(Attrs.minLinkWidth, minLinkWidth);
	}

	/**
	 * Returns which side of the chart the nodes are to be aligned to.
	 * (Can be <code>"top"</code>, <code>"center"</code>, <code>"bottom"</code>)
	 * <p>When the chart is inverted, <code>top</code> aligns to the left and
	 * <code>bottom</code> to the right.
	 * <p>Default: null
	 * @since 11.2.0.0
	 */
	public String getNodeAlignment() {
		return getAttr(Attrs.nodeAlignment, null).asString();
	}

	/**
	 * Sets which side of the chart the nodes are to be aligned to.
	 * (Can be <code>"top"</code>, <code>"center"</code>, <code>"bottom"</code>)
	 * <p>When the chart is inverted, <code>top</code> aligns to the left and
	 * <code>bottom</code> to the right.
	 * @since 11.2.0.0
	 */
	public void setNodeAlignment(String nodeAlignment) {
		setAttr(Attrs.nodeAlignment, nodeAlignment);
	}

	/**
	 * Returns the padding between nodes in a sankey diagram or dependency wheel, in pixels.
	 * <p>
	 * Default: 10
	 */
	public Number getNodePadding() {
		return getAttr(Attrs.nodePadding, 10).asNumber();
	}

	/**
	 * Sets the padding between nodes in a sankey diagram or dependency wheel, in pixels.
	 */
	public void setNodePadding(Number nodePadding) {
		setAttr(Attrs.nodePadding, nodePadding);
	}

	/**
	 * Returns the pixel width of each node in a sankey diagram or dependency wheel,
	 * or the height in case the chart is inverted.
	 * <p>
	 * Default: 20
	 * @since 10.2.1.0
	 */
	public Object getNodeWidth() {
		return getAttr(Attrs.nodeWidth, 20).asValue();
	}

	/**
	 * Sets the pixel width of each node in a sankey diagram or dependency wheel,
	 * or the height in case the chart is inverted. (Requires arcdiagram or sankey)
	 * @since 10.2.1.0
	 */
	public void setNodeWidth(String nodeWidth) {
		setAttr(Attrs.nodeWidth, nodeWidth);
	}

	/**
	 * Sets the pixel width of each node in a sankey diagram or dependency wheel,
	 * or the height in case the chart is inverted.
	 * @since 10.2.1.0
	 */
	public void setNodeWidth(Number nodeWidth) {
		setAttr(Attrs.nodeWidth, nodeWidth);
	}
}
