/* SeriesLabel.java

	Purpose:
		
	Description:
		
	History:
		Wed Nov 27 12:54:24 CST 2019, Created by rudyhuang

Copyright (C) 2019 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart.plotOptions;

import java.util.LinkedHashMap;
import java.util.Map;

import org.zkoss.chart.Optionable;
import org.zkoss.chart.PlotAttribute;
import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.chart.util.MapsHelper;
import org.zkoss.lang.Generics;

/**
 * Series labels are placed as close to the series as possible in a natural way,
 * seeking to avoid other series.
 * The goal of this feature is to make the chart more easily readable,
 * like if a human designer placed the labels in the optimal position.
 *
 * The series labels currently work with series types having a graph or an area.
 *
 * @author rudyhuang
 * @since 7.2.1.0
 */
public class SeriesLabel extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		boxesToAvoid,
		connectorAllowed,
		connectorNeighbourDistance,
		enabled,
		maxFontSize,
		minFontSize,
		onArea,
		style
	}

	/**
	 * Returns whether to allow labels to be placed distant to the graph if necessary,
	 * and draw a connector line to the graph.
	 * Setting this option to true may decrease the performance significantly,
	 * since the algorithm with systematically search for open spaces in the whole plot area.
	 * Visually, it may also result in a more cluttered chart, though more of the series will be labeled.
	 * <p>
	 * Default: false
	 */
	public boolean isConnectorAllowed() {
		return getAttr(Attrs.connectorAllowed, false).asBoolean();
	}

	/**
	 * Sets whether to allow labels to be placed distant to the graph if necessary,
	 * and draw a connector line to the graph.
	 * Setting this option to true may decrease the performance significantly,
	 * since the algorithm with systematically search for open spaces in the whole plot area.
	 * Visually, it may also result in a more cluttered chart, though more of the series will be labeled.
	 */
	public void setConnectorAllowed(boolean connectorAllowed) {
		setAttr(Attrs.connectorAllowed, connectorAllowed);
	}

	/**
	 * If the label is closer than this to a neighbour graph, draw a connector.
	 * <p>
	 * Default: 24
	 */
	public Number getConnectorNeighbourDistance() {
		return getAttr(Attrs.connectorNeighbourDistance, 24).asNumber();
	}

	/**
	 * If the label is closer than this to a neighbour graph, draw a connector.
	 */
	public void setConnectorNeighbourDistance(Number connectorNeighbourDistance) {
		setAttr(Attrs.connectorNeighbourDistance, connectorNeighbourDistance);
	}

	/**
	 * Returns if enable the series label per series.
	 * <p>
	 * Default: true
	 */
	public boolean isEnabled() {
		return getAttr(Attrs.enabled, true).asBoolean();
	}

	/**
	 * Sets if enable the series label per series.
	 */
	public void setEnabled(boolean enabled) {
		setAttr(Attrs.enabled, enabled);
	}

	/**
	 * For area-like series, allow the font size to vary so that small areas get a smaller font size.
	 * The default applies this effect to area-like series but not line-like series.
	 * <p>
	 * Default: null
	 */
	public Number getMaxFontSize() {
		return getAttr(Attrs.maxFontSize, null).asNumber();
	}

	/**
	 * For area-like series, allow the font size to vary so that small areas get a smaller font size.
	 * The default applies this effect to area-like series but not line-like series.
	 */
	public void setMaxFontSize(Number maxFontSize) {
		setAttr(Attrs.maxFontSize, maxFontSize);
	}

	/**
	 * For area-like series, allow the font size to vary so that small areas get a smaller font size.
	 * The default applies this effect to area-like series but not line-like series.
	 * <p>
	 * Default: null
	 */
	public Number getMinFontSize() {
		return getAttr(Attrs.minFontSize, null).asNumber();
	}

	/**
	 * For area-like series, allow the font size to vary so that small areas get a smaller font size.
	 * The default applies this effect to area-like series but not line-like series.
	 */
	public void setMinFontSize(Number minFontSize) {
		setAttr(Attrs.minFontSize, minFontSize);
	}

	/**
	 * Returns whether to draw the label on the area of an area series.
	 * By default it is drawn on the area.
	 * Set it to false to draw it next to the graph instead.
	 * <p>
	 * Default: true
	 */
	public boolean isOnArea() {
		return getAttr(Attrs.onArea, true).asBoolean();
	}

	/**
	 * Sets whether to draw the label on the area of an area series.
	 * By default it is drawn on the area.
	 * Set it to false to draw it next to the graph instead.
	 */
	public void setOnArea(boolean onArea) {
		setAttr(Attrs.onArea, onArea);
	}

	/**
	 * Returns styles for the series label.
	 * The color defaults to the series color, or a contrast color if onArea.
	 * Default:
	 * <pre>
	 * font-weight: 'bold'
	 * </pre>
	 * @param <K> type for the key
	 * <p>
	 * if style was set using {@link #setStyle(String)}, type will be String
	 * <p>
	 * if style was set using {@link #setStyle(Map)}, type will be the same as the type from the key of the map
	 * <p>
	 * if style was not set, default style will be applied, and type will be String
	 * @param <V> type for the value
	 * <p>
	 * if style was set using {@link #setStyle(String)}, type will be String
	 * <p>
	 * if style was set using {@link #setStyle(Map)}, type will be the same as the type from the key of the map
	 * <p>
	 * if style was not set, default style will be applied, and type will be String
	 */
	public <K, V> Map<K, V> getStyle() {
		if (!containsKey(Attrs.style)) {
			setStyle("font-weight: 'bold'");
		}
		return Generics.cast(getAttr(Attrs.style));
	}

	/**
	 * Set styles for the series label.
	 * This method allows <b>simple</b> settings like
	 * <pre>{@code
	 * label.setStyle("fontWeight: 'bold'; color: 'white'; textShadow: '0px 1px 2px black'");
	 * }</pre>
	 * <p>
	 * In the format of CSS key:value, and ";" separating multiple key:value pairs,
	 * if you have more a complex CSS styling, we recommend using {@link #setStyle(Map)}
	 * @param style one or many CSS key:value pairs in a single line string format
	 */
	public void setStyle(String style) {
		setStyle(MapsHelper.parse(new LinkedHashMap<String, String>(), style, ':', ';', '\''));
	}

	/**
	 * Set styles for the series label.
	 * This method allows <b>complex</b> styling like:
	 * <pre>
	 * {@code
	 * Map<String, Object> styles = new HashMap<String, Object>();
	 * styles.put("fill", "#E8E8EA");
	 * styles.put("stroke", "#6E6E78");
	 * styles.put("stroke-width", "1");
	 * styles.put("r", "0");
	 * Map<String, Object> states = new HashMap<String, Object>();
	 * Map<String, String> hover = new HashMap<String, String>();
	 * hover.put("fill", "#F4F4F6");
	 * hover.put("stroke", "#6E6E78");
	 * hover.put("stroke-width", "1");
	 * hover.put("r", "0");
	 * states.put("hover", hover);
	 * styles.put("states", states);
	 * Map<String, String> style = new HashMap<String, String>();
	 * style.put("color", "#4D5663");
	 * style.put("fontSize", "12px");
	 * style.put("fontFamily", "'Arial'");
	 * style.put("fontWeight", "normal");
	 * style.put("padding", "10px");
	 * styles.put("style", style);
	 * }</pre>
	 * which will generate the final results like:
	 * <pre>{@code
	 * {
	 *     "fill": "#E8E8EA",
	 *     "stroke": "#6E6E78",
	 *     "stroke-width": "1",
	 *     "r": "0",
	 *     "states":
	 *     {
	 *         "hover":
	 *         {
	 *             "fill": "#F4F4F6",
	 *             "stroke": "#6E6E78",
	 *             "stroke-width": "1",
	 *             "r": "0"
	 *         }
	 *     },
	 *     "style":
	 *     {
	 *         "color": "#4D5663",
	 *         "fontSize": "12px",
	 *         "fontFamily": "'Arial'",
	 *         "fontWeight": "normal",
	 *         "padding": "10px"
	 *     }
	 * }
	 * }</pre>
	 * @param <K> Type for the key, usually String
	 * @param <V> Type for the value, usually String. For nested maps, use Object or any other type.
	 * @param style the style object
	 */
	public <K, V> void setStyle(Map<K, V> style) {
		setAttr(Attrs.style, style);
	}
}
