/** Series.java.

	Purpose:
		
	Description:
		
	History:
		2:06:58 PM Jan 9, 2014, Created by jumperchen

Copyright (C) 2014 Potix Corporation. All Rights Reserved.
 */
package org.zkoss.chart;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.zkoss.chart.OptionDataEvent.EventType;
import org.zkoss.chart.plotOptions.CollapseButton;
import org.zkoss.chart.plotOptions.DataLabels;
import org.zkoss.chart.plotOptions.DragDrop;
import org.zkoss.chart.plotOptions.GaugeDialPlotOptions;
import org.zkoss.chart.plotOptions.GaugePivotPlotOptions;
import org.zkoss.chart.plotOptions.Jitter;
import org.zkoss.chart.plotOptions.NetworkGraphLayoutAlgorithm;
import org.zkoss.chart.plotOptions.NetworkGraphLink;
import org.zkoss.chart.plotOptions.PackedBubbleLayoutAlgorithm;
import org.zkoss.chart.plotOptions.PackedBubbleParentNode;
import org.zkoss.chart.plotOptions.SeriesLabel;
import org.zkoss.chart.plotOptions.SeriesPlotOptions;
import org.zkoss.chart.plotOptions.TargetOptions;
import org.zkoss.chart.plotOptions.TreeGraphLink;
import org.zkoss.chart.plotOptions.WindBarbDataGrouping;
import org.zkoss.chart.plotOptions.WordCloudRotation;
import org.zkoss.chart.plotOptions.XRangePartialFill;
import org.zkoss.chart.util.AnyVal;
import org.zkoss.chart.util.DeferredCall;
import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.chart.util.JSFunction;
import org.zkoss.chart.util.MapsHelper;
import org.zkoss.json.JSONObject;
import org.zkoss.json.JavaScriptValue;
import org.zkoss.lang.Generics;
import org.zkoss.lang.Objects;

/**
 * The Series object is the JavaScript representation of each line, area series,
 * pie etc. The object can be accessed in a number of ways. All series and point
 * event handlers give a reference to the series object. The chart object has a
 * series property that is a collection of all the chart's series. The point
 * objects also have the same reference. Another way to reference the series
 * programmatically is by id.
 * <p>
 * All the options in this class support {@link DynamicalAttribute}.
 * 
 * @author jumperchen
 */
public class Series extends Optionable implements OptionDataListener {
	private enum DAttrs implements PlotAttribute, DynamicalAttribute {
		accessibility, // all chart types
		allowPointSelect, // all chart types
		allowTraversingTree, // sunburst and treemap
		alternateStartingDirection, // treemap
		animation, // except network-graph
		animationLimit, // except network-graph, timeline, solid-gauge, gauge, and sankey
		baseSeries, // pareto, histogram, and bellcurve
		binsNumber, // histogram
		binWidth, // histogram
		boostBlending, // area, arearange, bar, bubble, column, columnrange, dependencywheel, heatmap, line, scatter, and treemap
		boostThreshold, // area, arearange, bar, bubble, column, columnrange, dependencywheel, heatmap, line, scatter, and treemap

		// arcdiagram, bar, bullet, column, columnpyramid, columnrange, cylinder,
		// dependencywheel, funnel, funnel3d, histogram, organization, pie, pyramid, pyramid3d,
		// sankey, sunburst, variablepie, variwide, waterfall, windbarb, wordcloud, and xrange
		borderColor,
		borderDashStyle, // venn

		// bar, bullet, column, columnrange, cylinder, funnel3d, heatmap, histogram,
		// organization, pyramid3d, tilemap, treemap, variwide, waterfall, windbarb,
		// wordcloud, and xrange
		borderRadius,

		// arcdiagram, dependencywheel, funnel, funnel3d, organization, pie, pyramid,
		// pyramid3d, sunburst, variablepie, and wordcloud
		borderWidth,
		boxDashStyle, // boxplot and errorbar
		breadcrumbs, // sunburst and treemap
		brighten, // venn
		center, // dependencywheel, funnel, funnel3d, item, pie, pyramid, pyramid3d, sunburst, and variablepie
		centeredLinks, // arcdiagram

		// bar, boxplot, bullet, column, columnpyramid, columnrange, cylinder, dependencywheel,
		// errorbar, funnel3d, histogram, organization, pyramid3d, sankey, variwide,
		// waterfall, windbarb, wordcloud, and xrange
		centerInCategory,
		centerPos, // arcdiagram
		className, // all types
		clip, // all types
		cluster, // scatter, vector, and venn
		collapseButton, // treegraph
		color, // all types
		colorAxis, // except arcdiagram, cylinder, funnel3d, and pyramid3d

		// arcdiagram, bar, boxplot, bullet, column, columnpyramid, columnrange, cylinder
		// dependencywheel, errorbar, funnel3d, histogram, organization, pyramid3d,
		// sankey, solidgauge, sunburst, timeline, treemap, variwide, venn, waterfall,
		// windbarb, wordcloud, and xrange
		colorByPoint,
		colorIndex, // all types
		colorKey, // except arcdiagram

		// arcdiagram, bar, boxplot, bullet, column, columnpyramid, columnrange, cylinder
		// dependencywheel, errorbar, funnel, funnel3d, histogram, item, organization, pie
		// pyramid, pyramid3d, sankey, sunburst, treemap, variablepie, variwide,
		// waterfall, windbarb, wordcloud, and xrange
		colors,
		colsize, // heatmap and tilemap

		// except arcdiagram, bar, column columnpyramid, gauge, networkgraph,
		// solidgauge, timeline, venn
		connectEnds,

		// except arcdiagram, bar, column columnpyramid, gauge, networkgraph,
		// solidgauge, timeline, venn
		connectNulls,
		connectorColor, // dumbbell and lollipop
		connectorWidth, // dumbbell and lollipop
		crisp, // except sankey and xrange

		// arcdiagram, area, arearange, areaspline, areasplinerange, bar, bellcurve,
		// boxplot, bubble, bullet, column, columnpyramid, columnrange, cylinder, dependencywheel,
		// dumbbell, errorbar, funnel, funnel3d, histogram, item, line, lollipop,
		// orgranization, pareto, polygon, pyramid, pyramid3d, scatter3d, spline,
		// streamgraph, sunburst, tilemap, treemap, variablepie, variwide, vector,
		// waterfall, windbarb, and wordcloud
		cropThreshold,
		cursor, // all types
		curveFactor, // sankey and dependencywheel
		custom, // all types
		dashStyle, // except bar, solidgauge, columnpyramid, gauge, timeline
		dataGrouping, // windbarb
		dataAsColumns, // all types
		dataLabels, // all types

		// except arcdiagram, bullet, dependencywheel, funnel, funnel3d, gauge, item,
		// networkgraph, organization, packedbubble, pie, pyramid, pyramid3d, sankey,
		// solidgauge, sunburst, tilemap, timeline, treemap, variablepie, venn,
		// wordcloud, and xrange
		dataSorting,

		// bar, boxplot, bullet, column, columnrange, cylinder, errorbar, funnel,
		// funnel3d, histogram, pie, pyramid, pyramid3d, variablepie, waterfall,
		// and windbarb
		depth,
		description, // all types
		dial, // gauge
		displayNegative, // bubble and packedbubble
		dragDrop, // except arcdiagram, networkgraph, packedbubble, and sankey
		draggable, // networkgraph and packedbubble

		// bar, boxplot, bullet, column, columnrange, cylinder, errorbar, funnel3d,
		// histogram, pyramid3d, waterfall, and windbarb
		edgeColor,
		// bar, boxplot, bullet, column, columnrange, cylinder, errorbar, funnel3d,
		// histogram, pyramid3d, waterfall, windbarb, and wordcloud
		edgeWidth,
		enableMouseTracking, // all types
		endAngle, // funnel, funnel3d, item, pie, pyramid, pyramid3d, variablepie
		equalNodes, // arcdiagram
		events, // TODO: all types

		// area, arearange, areaspline,  areasplinerange, bellcurve, boxplot, errorbar,
		// funnel, funnel3d, item, pie, pyramid, pyramid3d, streamgraph, sunburst,
		// and variablepie
		fillColor,
		fillOpacity, // area, arearange, areaspline,  areasplinerange, bellcurve, and streamgraph
		findNearestPointBy, // except gauge, sankey, solidgauge, timeline, venn, and xrange

		// except arcdiagram, gauge, networkgraph, sankey, solidgauge, timeline, venn, and xrange
		getExtremesFromAll,
		gradientForSides, // funnel3d and pyramid3d

		// bar, boxplot, bullet, column, columnpyramid, columnrange, cylinder, errorbar,
		// funnel3d, histogram, pyramid3d, variwide, waterfall, windbarb, and xrange
		grouping,

		// bar, boxplot, bullet, column, columnpyramid, columnrange, cylinder, dumbbell, errorbar,
		// funnel3d, histogram, lollipop, pyramid3d, variwide, waterfall, windbarb, and xrange
		groupPadding,

		// bar, bullet, column, columnrange, cylinder,
		// funnel3d, histogram, pyramid3d, waterfall, windbarb, and xrange
		groupZPadding,
		gapSize,
		hangingIndent, // organization
		hangingIndentTranslation, // organization
		height, // funnel, funnel3d, pyramid, and pyramid3d
		id, // all types
		ignoreHiddenPoint, // funnel, funnel3d, item, pie, pyramid, pyramid3d, timeline, treemap, variablepie
		includeInDataExport, // all types
		index, // all types
		innerRadius, // solidgauge
		innerSize, // item, pie, and variablepie,
		interactByLeaf, // treemap
		interpolation,
		intervals, // bellcurve
		itemPadding, // item
		jitter, // bubble, scatter, and scatter3d
		keys, // all types
		label, // except networkgraph and venn
		layout, // item
		layoutAlgorithm, // networkgraph, packedbubble, and treemap
		layoutStartingDirection, // treemap
		legendIndex, // except sankey
		levelIsConstant, // sunburst and treemap
		levels, // arcdiagram, dependencywheel, organization, sankey, sunburst, and treemap
		levelSize, // sunburst
		linecap, // except bar, column, columnpyramid, networkgraph, and venn,

		// area, arearange, areaspline, areasplinerange, bellcurve, dumbbell, lollipoup,
		// streamgraph, and waterfall
		lineColor,

		// except bar, column, columnpyramid, timeline, and venn
		lineWidth,
		link, // networkgraph
		linkColor, // organization
		linkedTo, // except venn
		linkLineWidth, // organization
		linkOpacity, // arcdiagram, dependencywheel, organization, and sankey
		linkRadius, // organization
		linkWeight, // arcdiagram
		lowColor, // dumbbell
		lowMarker, // dumbbell

		// except bar, boxplot, bullet, column, columnpyramid, gauge, solidgauge, tilemap, and venn
		marker,
		maxFontSize, // wordcloud
		maxPointSize, // variablepie

		// bar, boxplot, bullet, column, columnpyramid, columnrange, cylinder, errorbar,
		// funnel3d, histogram, pyramid3d, variwide, waterfall, windbarb, and xrange
		maxPointWidth,
		maxSize, // bubble and packedbubble
		medianColor, // boxplot and errorbar
		medianDashStyle, // boxplot and errorbar
		medianWidth, // boxplot and errorbar
		minFontSize, // wordcloud
		minLinkWidth, // arcdiagram, dependencywheel, organization, and sankey
		minNodeLength, // organization

		// bar, boxplot, bullet, column, columnpyramid, columnrange, cylinder, errorbar
		// funnel3d, histogram, pyramid3d, variwide, waterfall, windbarb, and xrange
		minPointLength,
		minPointSize, // variablepie
		minSize, // bubble, funnel, funnel3d, item, packedbubble, pie, pyramid, pyramid3d, variablepie,
		name, // all types
		neckHeight, // funnel, funnel3d, pyramid, and pyramid3d
		neckWidth, // funnel, funnel3d, pyramid, and pyramid3d
		negativeColor, // except gauge, networkgraph, sankey, solidgauge, timeline, venn, and xrange
		negativeFillColor, // area, arearange, areaspline, areasplinerange, bellcurve, dumbbell, lollipop, and streamgraph
		nodePadding, // dependencywheel, organization, and sankey
		nodes, // arcdiagram, dependencywheel, networkgraph, organization, and sankey
		nodeWidth, // arcdiagram, dependencywheel, organization, and sankey
		nullColor, // heatmap and tilemap
		offset, // arcdiagram
		onPoint, // all types
		onSeries, // windbarb
		opacity, // all types
		overshoot, // gauge and solidgauge
		parentNode, // packedbubble
		partialFill, // xrange
		pivot, // gauge
		placementStrategy, //wordcloud
		pointDescriptionFormatter, // all types
		pointInterval, // except arcdiagram, networkgraph, sankey, timeline, venn, and xrange,
		pointIntervalUnit, // except arcdiagram, networkgraph, sankey, timeline, venn, and xrange,

		// bar, boxplot, bullet, column, columnpyramid, columnrange, cylinder,
		// dumbbell, errorbar, funnel3d, heatmap, histogram, lollipop, pyramid3d,
		// tilemap, variwdie, waterfall, windbarb, and xrange
		pointPadding,

		// except arcdiagram, gauge, networkgraph, sankey, solidgauge, timeline, venn, and xrange
		pointPlacement,

		// bar, boxplot, bullet, column, columnpyramid, columnrange, cylinder, errorbar,
		// histogram, variwide, waterfall, and windbarb
		pointRange,
		pointsInInterval, // bellcurve

		// except arcdiagram, networkgraph, sankey, timeline, venn, and xrange
		pointStart,

		// bar, boxplot, bullet, column, columnpyramid, columnrange, cylinder,
		// errorbar, funnel3d, histogram, pyramid3d, variwide, waterfall, windbarb, and xrange
		pointWidth,
		radius, // solidgauge
		relativeXValue, // except arcdiagram
		reversed, // arcdiagram, funnel, funnel3d, pyramid, and pyramid3d
		rootId, // sunburst
		rotation, // wordcloud
		rounded, // solidgauge
		rows, // item
		rowsize, // heatmap and tilemap
		selected, // all types
		shadow, // except gauge, sankey, solidgauge, and tilemap,
		showCheckbox, // all types
		showInLegend, // all types
		size, // dependencywheel, item, pie, sunburst, variablepie,
		sizeBy, // bubble, packedbubble, and variablepie
		sizeByAbsoluteValue, // bubble
		skipKeyboardNavigation, // all types
		slicedOffset, // funnel, funnel3d, pie, pyramid, pyramid3d, sunburst, and variablepie

		// except arcdiagram, gauge, networkgraph, solidgauge, timeline, venn, and xrange
		softThreshold,
		sonification,
		sortIndex, // treemap
		spiral, // wordcloud

		// area, areaspline, bar, bellcurve, bullet, column, columnpyramid, cylinder,
		// dependencywheel, dumbbell, funnel3d, histogram, line, lollipop, organization,
		// pareto, pyramid3d, sankey, scatter, scatter3d, spline, streamgraph, variwide,
		// vector, waterfall, windbarb, wordcloud, and xrange
		stack,

		// except arcdaigram, arearange, boxplot, columnrange, errorbar, gauge,
		// networkgraph, sankey, solidgauge, timeline, venn, and xrange
		stacking,

		// dependencywheel, funnel, funnel3d, item, pie, pyramid, pyramid3d, sunburst,
		// variablepie
		startAngle,
		states, // except boxplot, gauge, and solidgauge
		stemDashStyle, // boxplot and errorbar
		stemWidth, // boxplot and errorbar

		// except arcdiagram, areaspline, areasplinerange, bar, column, columnpyramid,
		// gauge, networkgraph, pie, solidgauge, spline, streamgraph, timeline,
		//
		step,
		stickyTracking, // all types
		style, // wordcloud
		targetOptions, // bullet

		// funnel, funnel3d, item, pie, pyramid, pyramid3d, sunburst, and variablepie,
		thickness,
		threshold, // except gauge, networkgraph, sankey, solidgauge, timeline, venn, xrange
		tileShape, // tilemap
		tooltip, // all types
		trackByArea, // area, arearange, areaspline, areasplinerange, bellcurve, polygon, streamgraph
		traverseUpButton, // sunburst and treemap
		turboThreshold, // except gauge and solidgauge
		type, // all types
		upColor, // waterfall
		useSimulation, // packedbubble
		vectorLength, // windbarb
		whiskerColor, // boxplot and errorbar
		whiskerDashStyle, // boxplot and errorbar
		whiskerLength, // boxplot and errorbar
		whiskerWidth, // boxplot and errorbar
		width, // funnel, funnel3d, pyramid, and pyramid3d
		wrap, // gauge

		// except arcdiagram, funnel, item, networkgraph, pie, pyramid, variablepie,
		// and venn
		xAxis,
		xOffset, // windbarb

		// except arcdiagram, funnel, item, networkgraph, pie, pyramid, variablepie,
		// and venn
		yAxis,
		yOffset, // windbarb
		zIndex, // all types
		zMax, // bubble and variablepie
		zMin, // bubble and variablepie

		// except gauge, networkgraph, sankey, solidgauge, timeline, and venn
		zoneAxis,
		zones, // except gauge, sankey, solidgauge, timeline, and venn
		zThreshold // bubble and packedbubble
	}

	private enum Attrs implements PlotAttribute {
		data,
		visible,

		// internal use
		point
	}

	private SeriesPlotOptions plotOptions;

	public Series() {
	}

	/**
	 * Constructs the series with id and a list of point data.
	 */
	public Series(String id, List<Point> data) {
		setId(id);
		setData(data.toArray(new Point[0]));
	}

	/**
	 * Constructs the series with id and point data.
	 */
	public Series(String id, Point... data) {
		setId(id);
		setData(data);
	}

	/**
	 * Returns accessibility options for a series. If null, create a new one.
	 * <p> Default: an instance of {@link SeriesAccessibility}
	 * @since 10.2.1.0
	 */
	public SeriesAccessibility getAccessibility() {
		return (SeriesAccessibility) getAttr(DAttrs.accessibility, SeriesAccessibility.class).asValue();
	}

	/**
	 * Sets accessibility options for a series.
	 * @since 10.2.1.0
	 */
	public void setAccessibility(SeriesAccessibility accessibility) {
		setAttr(DAttrs.accessibility, accessibility);
	}

	/**
	 * For some series, there is a limit that shuts down initial animation by default
	 * when the total number of points in the chart is too high.
	 * For example, for a column chart and its derivatives, animation does not run
	 * if there is more than 250 points totally.
	 * To disable this cap, set animationLimit to Infinity.
	 * <p>Default: null
	 * @since 10.2.1.0
	 */
	public Number getAnimationLimit() {
		return getAttr(DAttrs.animationLimit, null).asNumber();
	}

	/**
	 * For some series, there is a limit that shuts down initial animation by default
	 * when the total number of points in the chart is too high.
	 * For example, for a column chart and its derivatives, animation does not run
	 * if there is more than 250 points totally.
	 * To disable this cap, set animationLimit to Infinity.
	 * <p>Default: null
	 * @since 10.2.1.0
	 */
	public void setAnimationLimit(Number animationLimit) {
		setAttr(DAttrs.animationLimit, animationLimit, null);
	}

	/**
	 * When enabled the user can click on a point which is a parent and zoom in
	 * on its children.
	 * <p>
	 * Default: false
	 * @since 10.2.1.0
	 */
	public boolean isAllowTraversingTree() {
		return getAttr(DAttrs.allowTraversingTree, false).asBoolean();
	}

	/**
	 * When enabled the user can click on a point which is a parent and zoom in
	 * on its children.
	 * @since 10.2.1.0
	 */
	public void setAllowTraversingTree(boolean allowTraversingTree) {
		setAttr(DAttrs.allowTraversingTree, allowTraversingTree);
	}

	/**
	 * Enabling this option will make the treemap alternate the drawing
	 * direction between vertical and horizontal. The next levels starting
	 * direction will always be the opposite of the previous.
	 * <p>Note: requires Treemap type</p>
	 * <p>
	 * Defaults to false.
	 * @since 10.2.1.0
	 */
	public boolean getAlternateStartingDirection() {
		return getAttr(DAttrs.alternateStartingDirection, false).asBoolean();
	}

	/**
	 * Enabling this option will make the treemap alternate the drawing
	 * direction between vertical and horizontal. The next levels starting
	 * direction will always be the opposite of the previous.
	 * <p>Note: requires Treemap type</p>
	 * <p>
	 * Defaults to false.
	 * @since 10.2.1.0
	 */
	public void setAlternateStartingDirection(boolean alternateStartingDirection) {
		setAttr(DAttrs.alternateStartingDirection, alternateStartingDirection, false);
	}

	/**
	 * Returns a preferable number of bins. It is a suggestion, so a histogram may
	 * have a different number of bins. By default it is set to the square root of
	 * the base series' data length.
	 * <p>Available options are: {@code "square-root"}, {@code "sturges}, {@code "rice"}.
	 * You can also define a function which takes a {@code "baseSeries"} as a parameter
	 * and should return a positive integer.
	 *
	 * <p>Defaults to {@code "square-root"}.
	 * @since 10.2.1.0
	 */
	public Object getBinsNumber() {
		return getAttr(DAttrs.binsNumber, "square-root").asValue();
	}

	/**
	 * Sets a preferable number of bins. It is a suggestion, so a histogram may
	 * have a different number of bins. By default it is set to the square root of
	 * the base series' data length.
	 * <p>Available options are: {@code "square-root"}, {@code "sturges}, {@code "rice"}.
	 * You can also define a function which takes a {@code "baseSeries"} as a parameter
	 * and should return a positive integer.
	 *
	 * <p>Defaults to {@code "square-root"}.
	 * @since 10.2.1.0
	 */
	public void setBinsNumber(JavaScriptValue binsNumber) {
		setAttr(DAttrs.binsNumber, binsNumber, NOT_NULL_VALUE);
	}

	/**
	 * Sets a preferable number of bins. It is a suggestion, so a histogram may
	 * have a different number of bins. By default it is set to the square root of
	 * the base series' data length.
	 * <p>Available options are: {@code "square-root"}, {@code "sturges}, {@code "rice"}.
	 * You can also define a function which takes a {@code "baseSeries"} as a parameter
	 * and should return a positive integer.
	 *
	 * <p>Defaults to {@code "square-root"}.
	 * @since 10.2.1.0
	 */
	public void setBinsNumber(String binsNumber) {
		if (!("square-root".equals(binsNumber) || "sturges".equals(binsNumber)
				|| "rice".equals(binsNumber))) {
			throw new IllegalArgumentException("Unknown value [" + binsNumber + "]");
		}
		setAttr(DAttrs.binsNumber, binsNumber, NOT_NULL_VALUE);
	}

	/**
	 * Sets a preferable number of bins. It is a suggestion, so a histogram may
	 * have a different number of bins. By default it is set to the square root of
	 * the base series' data length.
	 * <p>Available options are: {@code "square-root"}, {@code "sturges}, {@code "rice"}.
	 * You can also define a function which takes a {@code "baseSeries"} as a parameter
	 * and should return a positive integer.
	 *
	 * <p>Defaults to {@code "square-root"}.
	 * @since 10.2.1.0
	 */
	public void setBinsNumber(Number binsNumber) {
		setAttr(DAttrs.binsNumber, binsNumber, NOT_NULL_VALUE);
	}


	/**
	 * Returns the width of each bin. By default the bin's width is calculated
	 * as {@code (max - min) / number of bins}.
	 * This option takes precedence over binsNumber.
	 * <p>
	 * Default: null
	 * @since 10.2.1.0
	 */
	public Number getBinWidth() {
		return getAttr(DAttrs.binWidth, null).asNumber();
	}

	/**
	 * Sets the width of each bin. By default the bin's width is calculated
	 * as {@code (max - min) / number of bins}.
	 * This option takes precedence over binsNumber.
	 * @since 10.2.1.0
	 */
	public void setBinWidth(Number binWidth) {
		setAttr(DAttrs.binWidth, binWidth);
	}

	/**
	 * Returns the color blending in the boost module.
	 *
	 * <p>Defaults to null.
	 * @since 10.2.1.0
	 */
	public String getBoostBlending() {
		return getAttr(DAttrs.boostBlending, null).asString();
	}

	/**
	 * Sets the color blending in the boost module.
	 *
	 * <p>Defaults to null.
	 * @since 10.2.1.0
	 */
	public void setBoostBlending(String boostBlending) {
		setAttr(DAttrs.boostBlending, boostBlending, null);
	}

	/**
	 * Returns the point threshold for when a series should enter boost mode.
	 * Setting it to e.g. 2000 will cause the series to enter boost mode when
	 * there are 2000 or more points in the series.
	 * To disable boosting on the series, set the boostThreshold to 0.
	 * Setting it to 1 will force boosting.
	 *
	 * <p>Note that the cropThreshold also affects this setting.
	 * When zooming in on a series that has fewer points than the cropThreshold,
	 * all points are rendered although outside the visible plot area,
	 * and the boostThreshold won't take effect.
	 *
	 * <p>Defaults to 5000.
	 * @since 10.2.1.0
	 */
	public Number getBoostThreshold() {
		return getAttr(DAttrs.boostThreshold, 5000).asNumber();
	}

	/**
	 * Sets the point threshold for when a series should enter boost mode.
	 * Setting it to e.g. 2000 will cause the series to enter boost mode when
	 * there are 2000 or more points in the series.
	 * To disable boosting on the series, set the boostThreshold to 0.
	 * Setting it to 1 will force boosting.
	 *
	 * <p>Note that the cropThreshold also affects this setting.
	 * When zooming in on a series that has fewer points than the cropThreshold,
	 * all points are rendered although outside the visible plot area,
	 * and the boostThreshold won't take effect.
	 *
	 * <p>Defaults to 5000.
	 * @since 10.2.1.0
	 */
	public void setBoostThreshold(Number boostThreshold) {
		setAttr(DAttrs.boostThreshold, boostThreshold, null);
	}

	/**
	 * Returns an id for the series.
	 * <p>
	 * Default: null.
	 */
	public String getId() {
		return getAttr(DAttrs.id, null).asString();
	}

	/**
	 * Sets an id for the series.
	 */
	public void setId(String id) {
		setAttr(DAttrs.id, id);
	}


	/**
	 * Returns whether ignore the hidden point.
	 * <p>
	 * Equivalent to {@link Chart#isIgnoreHiddenSeries()}, this option tells
	 * whether the series shall be redrawn as if the hidden point were
	 * <code>null</code>.
	 * <p>
	 * The default value true.
	 * @since 10.2.1.0
	 */
	public boolean isIgnoreHiddenPoint() {
		return getAttr(DAttrs.ignoreHiddenPoint, true).asBoolean();
	}

	/**
	 * Sets whether ignore the hidden point.
	 * <p>
	 * Equivalent to {@link Chart#isIgnoreHiddenSeries()}, this option tells
	 * whether the series shall be redrawn as if the hidden point were
	 * <code>null</code>.
	 * <p>
	 * The default value true.
	 * @since 10.2.1.0
	 */
	public void setIgnoreHiddenPoint(boolean ignoreHiddenPoint) {
		setAttr(DAttrs.ignoreHiddenPoint, ignoreHiddenPoint);
	}

	/**
	 * When set to false will prevent the series data from being included in any form of data export.
	 * <p>
	 * Default: true
	 * @since 10.2.1.0
	 */
	public boolean isIncludeInDataExport() {
		return getAttr(DAttrs.includeInDataExport, true).asBoolean();
	}

	/**
	 * When set to false will prevent the series data from being included in any form of data export.
	 * @since 10.2.1.0
	 */
	public void setIncludeInDataExport(boolean includeInDataExport) {
		setAttr(DAttrs.includeInDataExport, includeInDataExport);
	}

	/**
	 * Returns whether to enable or disable the initial animation when a series is displayed.
	 * <p>
	 * Default: true.
	 * @since 7.2.1.0
	 */
	public boolean isAnimation() {
		return getAnimation() != Animation.NONE;
	}
	
	/**
	 * Sets whether to enable or disable the initial animation when a series is displayed.
	 * @see #setAnimation(Animation)
	 */
	public void setAnimation(boolean animation) {
		setAnimation(animation ? new Animation() : Animation.NONE);
	}

	/**
	 * Returns whether to allow this series' points to be selected by clicking on the graphic.
	 * <p>
	 * Default: false
	 * @since 7.2.2.0
	 */
	public boolean isAllowPointSelect() {
		return getAttr(DAttrs.allowPointSelect, false).asBoolean();
	}

	/**
	 * Sets whether to allow this series' points to be selected by clicking on the graphic.
	 * @since 7.2.2.0
	 */
	public void setAllowPointSelect(final boolean allowPointSelect) {
		setAttr(DAttrs.allowPointSelect, allowPointSelect);
	}

	/**
	 * Returns whether to enable or disable the initial animation when a series is displayed.
	 * <p>
	 * Default: an instance of {@link Animation}.
	 * @since 7.2.1.0
	 */
	public Animation getAnimation() {
		return (Animation) getAttr(DAttrs.animation, Animation.class).asValue();
	}

	/**
	 * Sets the initial animation when a series is displayed.
	 * Please note that this option only applies to the initial animation of the series itself.
	 * For other animations, see chart.animation and the animation parameter under the API methods.
	 * @see #setAnimation(boolean)
	 * @since 7.2.1.0
	 */
	public void setAnimation(Animation animation) {
		setAttr(DAttrs.animation, animation == null ? Animation.NONE : animation);
	}

	/**
	 * Returns an integer identifying the index to use for the base series,
	 * or a string representing the id of the series.
	 * <p>
	 * Default: null
	 * @since 7.2.1.0
	 */
	public Object getBaseSeries() {
		return getAttr(DAttrs.baseSeries);
	}

	/**
	 * Sets an integer identifying the index to use for the base series.
	 * @since 7.2.1.0
	 */
	public void setBaseSeries(int baseSeries) {
		setAttr(DAttrs.baseSeries, baseSeries);
	}

	/**
	 * Sets a string representing the id of the series to use for the base series.
	 * @since 7.2.1.0
	 */
	public void setBaseSeries(String baseSeries) {
		setAttr(DAttrs.baseSeries, baseSeries);
	}

	/**
	 * Returns the color of the border surrounding each slice.
	 * <p>
	 * Default: null
	 * @since 7.2.1.0
	 */
	public Color getBorderColor() {
		return (Color) getAttr(DAttrs.borderColor, null).asValue();
	}

	/**
	 * Sets the color of the border surrounding each slice.
	 * @since 7.2.1.0
	 */
	public void setBorderColor(Color borderColor) {
		setAttr(DAttrs.borderColor, borderColor);
	}

	/**
	 * Sets the color of the border surrounding each slice.
	 * @since 7.2.1.0
	 */
	public void setBorderColor(String borderColor) {
		setBorderColor(new Color(borderColor));
	}

	/**
	 * Sets the color of the border surrounding each slice.
	 * @since 7.2.1.0
	 */
	public void setBorderColor(LinearGradient borderColor) {
		setBorderColor(new Color(borderColor));
	}

	/**
	 * Sets the color of the border surrounding each slice.
	 * @since 7.2.1.0
	 */
	public void setBorderColor(RadialGradient borderColor) {
		setBorderColor(new Color(borderColor));
	}

	/**
	 * Returns the border dash style.
	 * <p>Note: requires venn type</p>
	 * <p>
	 * Default: "solid"
	 * @since 10.2.1.0
	 */
	public String getBorderDashStyle() {
		return getAttr(DAttrs.borderDashStyle, "solid").asString();
	}

	/**
	 * Sets the border dash style.
	 * <p>Note: requires venn type</p>
	 * <p>
	 * Default: "solid"
	 * @since 10.2.1.0
	 */
	public void setBorderDashStyle(String borderDashStyle) {
		setAttr(DAttrs.borderDashStyle, borderDashStyle);
	}

	/**
	 * Returns the options for the breadcrumbs, the navigation at the top leading
	 * the way up through the traversed levels.
	 * <p>Note: requires sunburst and treemap type.
	 * <p>Default: an instance of Breadcrumbs
	 * @since 10.2.1.0
	 */
	public Breadcrumbs getBreadcrumbs() {
		return Generics.cast(getAttr(DAttrs.breadcrumbs, Breadcrumbs.class).asValue());
	}

	/**
	 * Sets the options for the breadcrumbs, the navigation at the top leading
	 * the way up through the traversed levels.
	 * <p>Note: requires sunburst and treemap type.
	 * <p>Default: an instance of Breadcrumbs
	 * @since 10.2.1.0
	 */
	public void setBreadcrumbs(Breadcrumbs breadcrumbs) {
		setAttr(DAttrs.breadcrumbs, breadcrumbs);
	}

	/**
	 * Returns the brighten. (Requires venn type)
	 * <p>Default: {@code 0}
	 * @since 10.2.1.0
	 */
	public Number getBrighten() {
		return getAttr(DAttrs.brighten, 0).asNumber();
	}

	/**
	 * Sets the brighten. (Requires venn type)
	 * <p>Default: {@code 0}
	 * @since 10.2.1.0
	 */
	public void setBrighten(Number brighten) {
		setAttr(DAttrs.brighten, brighten);
	}

	/**
	 * Returns the border radius of the series.
	 * @since 7.2.1.0
	 */
	public Object getBorderRadius() {
		return getAttr(DAttrs.borderRadius, null).asValue();
	}

	/**
	 * Sets the border radius of the series.
	 * @since 7.2.1.0
	 */
	public void setBorderRadius(Number borderRadius) {
		setAttr(DAttrs.borderRadius, borderRadius);
	}

	/**
	 * Sets the border radius of the series.
	 * @since 11.2.0.0
	 */
	public void setBorderRadius(String borderRadius) {
		setAttr(DAttrs.borderRadius, borderRadius);
	}

	/**
	 * Returns the border width of the series.
	 */
	public Number getBorderWidth() {
		return getAttr(DAttrs.borderWidth, null).asNumber();
	}

	/**
	 * Sets the border width of the series.
	 */
	public void setBorderWidth(Number borderWidth) {
		setAttr(DAttrs.borderWidth, borderWidth);
	}

	/**
	 * Returns the dash style of the box.
	 *
	 * <p>Note: requires boxplot or errorbar type
	 * <p>Defaults to Solid.
	 * @since 10.2.1.
	 */
	public String getBoxDashStyle() {
		return getAttr(DAttrs.boxDashStyle, "Solid").asString();
	}

	/**
	 * Sets the dash style of the box.
	 *
	 * <p>Note: requires boxplot or errorbar type
	 * <p>Defaults to Solid.
	 * @since 10.2.1.
	 */
	public void setBoxDashStyle(String boxDashStyle) {
		if (!("Solid".equalsIgnoreCase(boxDashStyle)
				|| "ShortDash".equalsIgnoreCase(boxDashStyle)
				|| "ShortDot".equalsIgnoreCase(boxDashStyle)
				|| "ShortDashDot".equalsIgnoreCase(boxDashStyle)
				|| "ShortDashDotDot".equalsIgnoreCase(boxDashStyle)
				|| "Dot".equalsIgnoreCase(boxDashStyle)
				|| "Dash".equalsIgnoreCase(boxDashStyle)
				|| "LongDash".equalsIgnoreCase(boxDashStyle)
				|| "DashDot".equalsIgnoreCase(boxDashStyle)
				|| "LongDashDot".equalsIgnoreCase(boxDashStyle) || "LongDashDotDot"
				.equalsIgnoreCase(boxDashStyle)))
			throw new IllegalArgumentException("Unsupported style: ["
					+ boxDashStyle + "]");
		setAttr(DAttrs.boxDashStyle, boxDashStyle);
	}

	/**
	 * Returns the center of the pie chart relative to the plot area.
	 * Can be percentages or pixel values.
	 * The default behaviour is to center the pie so that all slices and data labels are within the plot area.
	 * As a consequence, the pie may actually jump around in a chart with dynamic values,
	 * as the data labels move. In that case, the center should be explicitly set,
	 * for example to ["50%", "50%"].
	 * <p>
	 * Default: [null, null]
	 * @since 7.2.1.0
	 */
	public List<String> getCenter() {
		if (!containsKey(DAttrs.center)) {
			setCenter(Arrays.asList(null, null));
		}
		return Generics.cast(getAttr(DAttrs.center));
	}

	/**
	 * Sets the center of the pie chart relative to the plot area.
	 * Can be percentages or pixel values.
	 * The default behaviour is to center the pie so that all slices and data labels are within the plot area.
	 * As a consequence, the pie may actually jump around in a chart with dynamic values,
	 * as the data labels move. In that case, the center should be explicitly set,
	 * for example to ["50%", "50%"].
	 * @since 7.2.1.0
	 */
	public void setCenter(List<Object> center) {
		setAttr(DAttrs.center, center, NOT_NULL_VALUE);
	}

	/**
	 * Returns the option to center links rather than position them one after another. (Requires arcdiagram type)
	 *
	 *
	 * <p>
	 * Default: {@code false}.
	 * @since 10.2.1.0
	 */
	public boolean isCenteredLinks() {
		return getAttr(DAttrs.centeredLinks, false).asBoolean();
	}

	/**
	 * Sets the option to center links rather than position them one after another. (Requires arcdiagram type)
	 * @since 10.2.1.0
	 */
	public void setCenteredLinks(boolean centeredLinks) {
		setAttr(DAttrs.centeredLinks, centeredLinks);
	}

	/**
	 * When true, the columns will center in the category, ignoring null or missing points.
	 * When false, space will be reserved for null or missing points.
	 *
	 * <p>
	 * Default: {@code false}.
	 * @since 10.2.1.0
	 */
	public boolean isCenterInCategory() {
		return getAttr(DAttrs.centerInCategory, false).asBoolean();
	}

	/**
	 * When true, the columns will center in the category, ignoring null or missing points.
	 * When false, space will be reserved for null or missing points.
	 * @since 10.2.1.0
	 */
	public void setCenterInCategory(boolean centerInCategory) {
		setAttr(DAttrs.centerInCategory, centerInCategory);
	}

	/**
	 * Returns the center position for arc diagram.
	 * <p>Default: {@code null}
	 * @since 10.2.1.0
	 */
	public String getCenterPos() {
		return getAttr(DAttrs.centerPos, null).asString();
	}

	/**
	 * Sets the center position for arc diagram.
	 * <p>Default: {@code null}
	 * @since 10.2.1.0
	 */
	public void setCenterPos(String centerPos) {
		setAttr(DAttrs.centerPos, centerPos);
	}

	/**
	 * Returns an additional class name to apply to the series' graphical elements.
	 * This option does not replace default class names of the graphical element.
	 * <p>Default: null
	 * @since 10.2.1.0
	 */
	public String getClassName() {
		return getAttr(DAttrs.className, null).asString();
	}

	/**
	 * Sets an additional class name to apply to the series' graphical elements.
	 * This option does not replace default class names of the graphical element.
	 * <p>Default: null
	 * @since 10.2.1.0
	 */
	public void setClassName(String className) {
		setAttr(DAttrs.className, className);
	}

	/**
	 * Disable this option to allow series rendering in the whole plotting area.
	 *
	 * <p>Note: Clipping should be always enabled when {@code chart.zoomType} is set
	 *
	 * <p>Defaults to true.
	 * @since 10.2.1.0
	 */
	public boolean isClip() {
		return getAttr(DAttrs.clip, true).asBoolean();
	}

	/**
	 * Disable this option to allow series rendering in the whole plotting area.
	 *
	 * <p>Note: Clipping should be always enabled when {@code chart.zoomType} is set
	 *
	 * <p>Defaults to true.
	 * @since 10.2.1.0
	 */
	public void setClip(boolean clip) {
		setAttr(DAttrs.clip, clip);
	}

	/**
	 * Returns the options for marker clusters, the concept of sampling the data values
	 * into larger blocks in order to ease readability and increase performance of
	 * the JavaScript charts.
	 * <p>Note: requires scatter, vector, or venn type</p>
	 * <p> Default: an instance of Cluster.
	 * @since 10.2.1.0
	 */
	public Cluster getCluster() {
		return (Cluster) getAttr(DAttrs.cluster, Cluster.class).asValue();
	}

	/**
	 * Sets the options for marker clusters, the concept of sampling the data values
	 * into larger blocks in order to ease readability and increase performance of
	 * the JavaScript charts.
	 * <p>Note: requires scatter, vector, or venn type</p>
	 * <p> Default: an instance of Cluster.
	 * @since 10.2.1.0
	 */
	public void setCluster(Cluster cluster) {
		setAttr(DAttrs.cluster, cluster);
	}

	/**
	 * Sets the center of the pie chart relative to the plot area.
	 * Can be percentages or pixel values.
	 * The default behaviour is to center the pie so that all slices and data labels are within the plot area.
	 * As a consequence, the pie may actually jump around in a chart with dynamic values,
	 * as the data labels move. In that case, the center should be explicitly set,
	 * for example to ["50%", "50%"].
	 * @since 7.2.1.0
	 */
	public void setCenter(Object... center) {
		setCenter(Arrays.asList(center));
	}

	/**
	 * Sets the plot options to the series.
	 */
	public void setPlotOptions(SeriesPlotOptions plotOptions) {
		SeriesPlotOptions old = this.plotOptions;
		if (!Objects.equals(old, plotOptions)) {
			if (old != null)
				old.removeOptionDataListener(this);

			if (plotOptions != null) {
				plotOptions.addOptionDataListener(this);
				this.plotOptions = plotOptions;
			}
		}
	}

	/**
	 * Returns the plot options to the series. If null, it will create a new
	 * one.
	 */
	public SeriesPlotOptions getPlotOptions() {
		if (plotOptions == null) {
			plotOptions = new SeriesPlotOptions();
			setPlotOptions(plotOptions);
		}
		return plotOptions;
	}

	/**
	 * Add a point at the end of the current points list.
	 * 
	 * @param point
	 */
	public void addPoint(Point point) {
		addPoint(point, true, false, true);
	}

	/**
	 * Add a point to the end of the current points list.
	 * 
	 * @param point
	 *            the point
	 * @param redraw
	 *            whether to redraw the chart
	 * @param shift
	 *            If shift is true, a point is shifted off the start of the
	 *            series as one is appended to the end.
	 * @param animation
	 *            Whether to apply animation, and optionally animation
	 *            configuration
	 */
	public void addPoint(Point point, boolean redraw, boolean shift,
			boolean animation) {
		addPoint(point, redraw, shift, animation ? new Animation()
				: Animation.NONE);
	}

	/**
	 * Add a point at the end of the current points list.
	 * 
	 * @param point
	 *            the point
	 * @param redraw
	 *            whether to redraw the chart
	 * @param shift
	 *            If shift is true, a point is shifted off the start of the
	 *            series as one is appended to the end.
	 * @param animation
	 *            Whether to apply animation, and optionally animation
	 *            configuration
	 */
	public void addPoint(final Point point, final boolean redraw,
			final boolean shift, final Animation animation) {
		List<Point> list = getData();
		if (list == null)
			list = new LinkedList<Point>();
		list.add(point);

		// monitor if data changed.
		point.addOptionDataListener(this);
		point.setSeries(this);
		setAttr(Attrs.data, list);

		fireEvent(EventType.ADDED, Attrs.point.toString(), point,
				new DeferredCall() {
					public void execute(JSFunction func) {
						func.callFunction("addPoint", point, redraw, shift,
								animation);
					}
				});
	}

	/**
	 * Add a number point to the end of the current points list.
	 */
	public void addPoint(Number y) {
		if (y == null)
			addPoint(new Point());
		else
			addPoint(new Point(y));
	}

	/**
	 * Add a number point to the end of the current points list.
	 */
	public void addPoint(double y) {
		addPoint(new Point(y));
	}

	/**
	 * Add a point with x and y numbers to the end of the current points list.
	 */
	public void addPoint(Number x, Number y) {
		addPoint(new Point(x, y));
	}

	/**
	 * Add a point with x and y numbers to the end of the current points list.
	 */
	public void addPoint(double x, double y) {
		addPoint(new Point(x, y));
	}

	/**
	 * Add a point with x, low, and high numbers to the end of the current
	 * points list.
	 */
	public void addPoint(Number x, Number low, Number high) {
		addPoint(new Point(x, low, high));
	}

	/**
	 * Add a point with x, low, and high numbers to the end of the current
	 * points list.
	 */
	public void addPoint(double x, double low, double high) {
		addPoint(new Point(x, low, high));
	}

	/**
	 * Add a point with name and number to the end of the current points list.
	 */
	public void addPoint(String name, Number y) {
		addPoint(new Point(name, y));
	}

	/**
	 * Add a point with name, Y number, and X index to the end of the current points list.
	 * @since 1.1
	 */
	public void addPoint(String name, Number y, Number x) {
		addPoint(new Point(name, y, x));
	}

	/**
	 * Add a point with name and number to the end of the current points list.
	 */
	public void addPoint(String name, double y) {
		addPoint(new Point(name, y));
	}

	/**
	 * Add a point with from and to to the end of the current points list.
	 * @since 7.2.1.0
	 */
	public void addPoint(String from, String to) {
		addPoint(new Point(from, to));
	}

	/**
	 * Add a point with from, to and weight to the end of the current points list.
	 * @since 7.2.1.0
	 */
	public void addPoint(String from, String to, Number weight) {
		addPoint(new Point(from, to, weight));
	}

	/**
	 * Add a point with id, parent and name to the end of the current points list.
	 * @since 11.2.0.0
	 */
	public void addPoint(String id, String parent, String name) {
		addPoint(new Point(id, parent, name));
	}

	/**
	 * Sets a list of point data
	 */
	public void setData(List<? extends Number> data) {
		Point[] points = new Point[data.size()];
		int i = 0;
		for (Number t : data) {
			if (t == null)
				points[i++] = new Point();
			else
				points[i++] = new Point(t);
		}
		setData(points);
	}

	/**
	 * Sets an array of point data
	 */
	public void setData(Point... data) {
		for (Point p : data) {
			// monitor if data changed.
			p.addOptionDataListener(this);
			p.setSeries(this);
		}

		// use ArrayList instead of fixed ArrayList
		final List<Point> fixedList = Arrays.asList(data);
		List<Point> list = new ArrayList<Point>(fixedList);
		setAttr(Attrs.data, list);

		fireEvent(EventType.CHANGED, Attrs.data.toString(), data,
				new DeferredCall() {
					public void execute(JSFunction func) {
						func.callFunction("setData", fixedList);
					}
				});
	}

	/**
	 * Sets an array of number data
	 */
	public void setData(Number... data) {
		setData(Arrays.asList(data));
	}

	/**
	 * Sets an array of number data
	 */
	public void setData(Double... data) {
		setData(Arrays.asList(data));
	}

	/**
	 * Sets an array of number data
	 */
	public void setData(Integer... data) {
		setData(Arrays.asList(data));
	}

	/**
	 * Returns the list of point data
	 */
	public List<Point> getData() {
		return Generics.cast(getAttr(Attrs.data, null).asValue());
	}

	/**
	 * Returns the point in the list from the given index.
	 */
	public Point getPoint(int index) {
		return getData().get(index);
	}

	/**
	 * Sets the index of the series in the chart, affecting the internal index
	 * in the chart.series array, the visible Z index as well as the order in
	 * the legend.
	 */
	public void setIndex(int index) {
		if (index < 0) {
			removeKey(DAttrs.index, true);
		} else {
			setAttr(DAttrs.index, index);
		}
	}

	/**
	 * Returns the index of the series in the chart, affecting the internal
	 * index in the chart.series array, the visible Z index as well as the order
	 * in the legend.
	 * <p>
	 * Default: -1, it means depended on the implementaion.
	 */
	public int getIndex() {
		return getAttr(DAttrs.index, -1).asInt();
	}

	/**
	 * Returns the inner radius for points in a solid gauge.
	 * Can be given as a number (pixels) or percentage string.
	 * <p>
	 * Default: 60
	 * @since 10.2.1.0
	 */
	public Object getInnerRadius() {
		return getAttr(DAttrs.innerRadius, 60).asValue();
	}

	/**
	 * Sets the inner radius for points in a solid gauge.
	 * Can be given as a number (pixels).
	 * @since 10.2.1.0
	 */
	public void setInnerRadius(Number innerRadius) {
		setAttr(DAttrs.innerRadius, innerRadius);
	}

	/**
	 * Returns the length of the bell curve.
	 * A unit of the length of the bell curve is standard deviation.
	 *
	 * Requires bellcurve.
	 * <p>
	 * Default: 3
	 * @since 7.2.1.0
	 */
	public Number getIntervals() {
		return getAttr(DAttrs.intervals, 3).asNumber();
	}

	/**
	 * Sets the length of the bell curve.
	 * A unit of the length of the bell curve is standard deviation.
	 *
	 * Requires bellcurve.
	 * @since 7.2.1.0
	 */
	public void setIntervals(Number intervals) {
		setAttr(DAttrs.intervals, intervals, 3);
	}

	/**
	 * Returns the padding between the items, given in relative size where the size
	 * of the item is 1.
	 * <p>Default: {@code 0.1}
	 * @since 10.2.1.0
	 */
	public Number getItemPadding() {
		return getAttr(DAttrs.itemPadding, 0.1).asNumber();
	}

	/**
	 * Sets the padding between the items, given in relative size where the size
	 * of the item is 1.
	 * <p>Default: {@code 0.1}
	 * @since 10.2.1.0
	 */
	public void setItemPadding(Number itemPadding) {
		setAttr(DAttrs.itemPadding, itemPadding);
	}

	/**
	 * Apply a jitter effect for the rendered markers.
	 * When plotting discrete values, a little random noise may help telling the points apart.
	 * The jitter setting applies a random displacement of up to n axis units in either direction.
	 * So for example on a horizontal X axis, setting the jitter.x to 0.24 will
	 * render the point in a random position between 0.24 units to the left
	 * and 0.24 units to the right of the true axis position.
	 * On a category axis, setting it to 0.5 will fill up the bin and make the data appear continuous.
	 *
	 * When rendered on top of a box plot or a column series,
	 * a jitter value of 0.24 will correspond to the underlying series' default groupPadding and pointPadding settings.
	 * @since 10.2.1.0
	 */
	public Jitter getJitter() {
		Jitter jitter = (Jitter) getAttr(DAttrs.jitter);
		if (jitter == null) {
			jitter = new Jitter();
			setJitter(jitter);
		}
		return jitter;
	}

	/**
	 * Apply a jitter effect for the rendered markers.
	 * When plotting discrete values, a little random noise may help telling the points apart.
	 * The jitter setting applies a random displacement of up to n axis units in either direction.
	 * So for example on a horizontal X axis, setting the jitter.x to 0.24 will
	 * render the point in a random position between 0.24 units to the left
	 * and 0.24 units to the right of the true axis position.
	 * On a category axis, setting it to 0.5 will fill up the bin and make the data appear continuous.
	 *
	 * When rendered on top of a box plot or a column series,
	 * a jitter value of 0.24 will correspond to the underlying series' default groupPadding and pointPadding settings.
	 * @since 10.2.1.0
	 */
	public void setJitter(Jitter jitter) {
		setAttr(DAttrs.jitter, jitter);
	}


	/**
	 * In circular view, returns the size of the inner diameter of the circle.
	 * Can be a percentage or pixel value.
	 * Percentages are relative to the outer perimeter.
	 * Pixel values are given as integers.
	 *
	 * If the {@link #getRows()} option is set, it overrides the innerSize setting.
	 * <p>
	 * Default: "40%"
	 * @since 7.2.1.0
	 */
	public Object getInnerSize() {
		return getAttr(DAttrs.innerSize, "40%").asValue();
	}

	/**
	 * In circular view, sets the size of the inner diameter of the circle.
	 * Can be a percentage.
	 * Percentages are relative to the outer perimeter.
	 *
	 * If the {@link #getRows()} option is set, it overrides the innerSize setting.
	 * @since 7.2.1.0
	 */
	public void setInnerSize(String innerSize) {
		setAttr(DAttrs.innerSize, innerSize);
	}

	/**
	 * This option decides if the user can interact with the parent nodes or
	 * just the leaf nodes. When this option is undefined, it will be true by
	 * default. However when allowTraversingTree is true,
	 * then it will be false by default.
	 * <p>Requires treemap
	 * @since 10.2.1.0
	 */
	public boolean getInteractByLeaf() {
		if (isAllowTraversingTree()) {
			return getAttr(DAttrs.interactByLeaf, false).asBoolean();
		} else {
			return getAttr(DAttrs.interactByLeaf, true).asBoolean();
		}
	}
	/**
	 * This option decides if the user can interact with the parent nodes or
	 * just the leaf nodes. When this option is undefined, it will be true by
	 * default. However when allowTraversingTree is true,
	 * then it will be false by default.
	 * <p>Requires treemap
	 * @since 10.2.1.0
	 */
	public void setInteractByLeaf(boolean interactByLeaf) {
		setAttr(DAttrs.interactByLeaf, interactByLeaf);
	}

	/**
	 * Returns the boolean value of interpolated.
	 * <p>Make the heatmap render its data points as an interpolated image.
	 * <p> Default: false
	 * @since 11.2.0.0
	 */
	public boolean isInterpolation() {
		return getAttr(DAttrs.interpolation, false).asBoolean();
	}

	/**
	 * Sets the boolean value of interpolated.
	 * <p>Make the heatmap render its data points as an interpolated image.
	 * @since 11.2.0.0
	 */
	public void setInterpolation(boolean interpolation) {
		setAttr(DAttrs.interpolation, interpolation);
	}
	/**
	 * In circular view, sets the size of the inner diameter of the circle.
	 * Can be a pixel value.
	 * Pixel values are given as integers.
	 *
	 * If the {@link #getRows()} option is set, it overrides the innerSize setting.
	 */
	public void setInnerSize(int innerSize) {
		setAttr(DAttrs.innerSize, innerSize);
	}

	/**
	 * Returns the first or single option of data labels.
	 * 
	 * @see SeriesPlotOptions#getDataLabels()
	 */
	public DataLabels getDataLabels() {
		return getPlotOptions().getDataLabels();
	}

	/**
	 * Sets the first or single option of data labels.
	 *
	 * @param dataLabels
	 * @see SeriesPlotOptions#setDataLabels(DataLabels)
	 */
	public void setDataLabels(DataLabels dataLabels) {
		getPlotOptions().setDataLabels(dataLabels);
	}

	/**
	 * Returns the series data sorting.
	 * <p>Default: an instance of DataSoring.
	 * @see DataSorting
	 * @since 10.2.1.0
	 */
	public DataSorting getDataSorting() {
		return (DataSorting) getAttr(DAttrs.dataSorting, DataSorting.class).asValue();
	}

	/**
	 * Sets the series data sorting.
	 * <p>Default: an instance of DataSoring.
	 * @see DataSorting
	 * @since 10.2.1.0
	 */
	public void setDataSorting(DataSorting dataSorting) {
		setAttr(DAttrs.dataSorting, dataSorting);
	}

	/**
	 * Returns the list of options of data labels.
	 *
	 * @see SeriesPlotOptions#getDataLabelsList()
	 * @since 7.2.2.0
	 */
	public List<DataLabels> getDataLabelsList() {
		return getPlotOptions().getDataLabelsList();
	}

	/**
	 * Sets the list of options of data labels.
	 * 
	 * @see SeriesPlotOptions#setDataLabels(DataLabels)
	 * @since 7.2.2.0
	 */
	public void setDataLabels(List<DataLabels> dataLabels) {
		getPlotOptions().setDataLabels(dataLabels);
	}

	/**
	 * Add an option of data labels to the list.
	 * 
	 * @see SeriesPlotOptions#addDataLabels(DataLabels)
	 * @since 7.2.2.0
	 */
	public void addDataLabels(DataLabels dataLabels) {
		getPlotOptions().addDataLabels(dataLabels);
	}

	/**
	 * Returns the depth of the 3D chart.
	 * <p>Default: {@code null}
	 * @since 10.2.1.0
	 */
	public Number getDepth() {
		return getAttr(DAttrs.depth, null).asNumber();
	}

	/**
	 * Sets the depth of the 3D chart.
	 * <p>Default: {@code null}
	 * @since 10.2.1.0
	 */
	public void setDepth(Number depth) {
		setAttr(DAttrs.depth, depth);
	}

	/**
	 * A description of the series to add to the screen reader information about the series.
	 * <p>Default: {@code null}
	 * @since 10.2.1.0
	 */
	public String getDescription() {
		return getAttr(DAttrs.description, null).asString();
	}

	/**
	 * A description of the series to add to the screen reader information about the series.
	 * <p>Default: {@code null}
	 * @since 10.2.1.0
	 */
	public void setDescription(String description) {
		setAttr(DAttrs.description, description);
	}



	/**
	 * Returns options for the dial or arrow pointer of the gauge.
	 * <p>
	 * Default: an instance of {@link GaugeDialPlotOptions}.
	 * @since 10.2.1.0
	 */
	public GaugeDialPlotOptions getDial() {
		GaugeDialPlotOptions dial = (GaugeDialPlotOptions) getAttr(DAttrs.dial);
		if (dial == null) {
			dial = new GaugeDialPlotOptions();
			setDial(dial);
		}
		return dial;
	}

	/**
	 * Sets options for the dial or arrow pointer of the gauge.
	 * @since 10.2.1.0
	 */
	public void setDial(GaugeDialPlotOptions dial) {
		setAttr(DAttrs.dial, dial);
	}



	/**
	 * Returns whether to display negative sized bubbles. The threshold is given
	 * by the {@link #setZThreshold(Number)} option, and negative bubbles can be
	 * visualized by setting {@link #setNegativeColor(String)}
	 * <p>
	 * Default: true
	 * @since 10.2.1.0
	 */
	public boolean isDisplayNegative() {
		return getAttr(DAttrs.displayNegative, true).asBoolean();
	}

	/**
	 * Sets whether to display negative sized bubbles. The threshold is given by
	 * the {@link #setZThreshold(Number)} option, and negative bubbles can be
	 * visualized by setting {@link #setNegativeColor(String)}
	 * <p>
	 * Default: true
	 * @since 10.2.1.0
	 */
	public void setDisplayNegative(boolean displayNegative) {
		setAttr(DAttrs.displayNegative, displayNegative);
	}

	/**
	 * Returns the options applied to collapse Button for treegraph type.
	 * @since 10.3.3.0
	 */
	public CollapseButton getCollapseButton() {
		CollapseButton options = (CollapseButton) getAttr(
				DAttrs.collapseButton);
		if (options == null) {
			options = new CollapseButton();
			setCollapseButton(options);
		}
		return options;
	}

	/**
	 * Sets the options applied to collapse Button for treegraph type.
	 * @since 10.3.3.0
	 */
	public void setCollapseButton(CollapseButton collapseButton) {
		setAttr(DAttrs.collapseButton, collapseButton);
	}

	/**
	 * Returns individual color for the series. By default the color is pulled
	 * from the global <code>colors</code> array.
	 * <p>
	 * Default: null.
	 */
	public Color getColor() {
		return (Color) getAttr(DAttrs.color, null).asValue();
	}

	/**
	 * Sets individual color for the series. By default the color is pulled from
	 * the global <code>colors</code> array.
	 */
	public void setColor(Color color) {
		setAttr(DAttrs.color, color);
	}

	/**
	 * Sets individual color for the series. By default the color is pulled from
	 * the global <code>colors</code> array.
	 */
	public void setColor(String color) {
		setColor(new Color(color));
	}

	/**
	 * Sets individual color for the series. By default the color is pulled from
	 * the global <code>colors</code> array.
	 */
	public void setColor(LinearGradient color) {
		setColor(new Color(color));
	}

	/**
	 * Sets individual color for the series. By default the color is pulled from
	 * the global <code>colors</code> array.
	 */
	public void setColor(RadialGradient color) {
		setColor(new Color(color));
	}

	/**
	 * When using dual or multiple color axes, this defines which colorAxis
	 * the particular series is connected to. It refers to either the {@link ColorAxis#getId()}
	 * or the index of the axis in the colorAxis array, with 0 being the first.
	 * Set this option to false to prevent a series from connecting to the default color axis.
	 * <p>
	 * Default: 0
	 * @since 7.2.1.0
	 */
	public Object getColorAxis() {
		return getAttr(DAttrs.colorAxis, 0).asValue();
	}

	/**
	 * When using dual or multiple color axes, this defines which colorAxis
	 * the particular series is connected to.
	 * It refers to the index of the axis in the colorAxis array, with 0 being the first.
	 * @since 7.2.1.0
	 */
	public void setColorAxis(Number colorAxis) {
		setAttr(DAttrs.colorAxis, colorAxis, 0);
	}

	/**
	 * When using dual or multiple color axes, this defines which colorAxis
	 * the particular series is connected to. It refers to the {@link ColorAxis#getId()}.
	 * @since 7.2.1.0
	 */
	public void setColorAxis(String colorAxis) {
		setAttr(DAttrs.colorAxis, colorAxis);
	}

	/**
	 * When using dual or multiple color axes, this defines which colorAxis
	 * the particular series is connected to.
	 * Set this option to false to prevent a series from connecting to the default color axis.
	 * @since 7.2.1.0
	 */
	public void setColorAxis(boolean colorAxis) {
		setAttr(DAttrs.colorAxis, colorAxis);
	}

	/**
	 * Returns what data value should be used to calculate point color if colorAxis is used.
	 * Requires to set min and max if some custom point property is used or if
	 * approximation for data grouping is set to 'sum'.
	 * <p>
	 * Default: "y"
	 * @since 7.2.1.0
	 */
	public String getColorKey() {
		return getAttr(DAttrs.colorKey, "y").asString();
	}

	/**
	 * Sets what data value should be used to calculate point color if colorAxis is used.
	 * Requires to set min and max if some custom point property is used or if
	 * approximation for data grouping is set to 'sum'.
	 * @since 7.2.1.0
	 */
	public void setColorKey(String colorKey) {
		setAttr(DAttrs.colorKey, colorKey, "y");
	}

	/**
	 * Returns whether color by point option is enabled or not
	 */
	public boolean isColorByPoint() {
		return getAttr(DAttrs.colorByPoint, false).asBoolean();
	}
	
	/**
	 * Sets whether to enable the color by point option or not
	 */
	public void setColorByPoint(boolean colorByPoint) {
		setAttr(DAttrs.colorByPoint, colorByPoint);
	}

	/**
	 * Returns a specific color index to use for the series, so its graphic
	 * representations are given the class name {@code "highcharts-color-{n}"}.
	 * <p>Default: null
	 * @since 10.2.1.0
	 */
	public Number getColorIndex() {
		return getAttr(DAttrs.colorIndex, null).asNumber();
	}

	/**
	 * Sets a specific color index to use for the series, so its graphic
	 * representations are given the class name {@code "highcharts-color-{n}"}.
	 * <p>Default: null
	 * @since 10.2.1.0
	 */
	public void setColorIndex(Number colorIndex) {
		setAttr(DAttrs.colorIndex, colorIndex);
	}

	/**
	 * Returns a series specific or series type specific color set to apply
	 * instead of the global {@link Charts#getColors()} when
	 * {@link #setColorByPoint(boolean)} is true.
	 * @since 7.2.1.0
	 */
	public List<Color> getColors() {
		return Generics.cast(getAttr(DAttrs.colors, null).asValue());
	}

	/**
	 * Sets a series specific or series type specific color set to apply
	 * instead of the global {@link Charts#getColors()} when
	 * {@link #setColorByPoint(boolean)} is true.
	 * @since 7.2.1.0
	 */
	public void setColors(List<Color> colors) {
		setAttr(DAttrs.colors, colors);
	}

	/**
	 * Sets a series specific or series type specific color set to apply
	 * instead of the global {@link Charts#getColors()} when
	 * {@link #setColorByPoint(boolean)} is true.
	 * @since 7.2.1.0
	 */
	public void setColors(String... colors) {
		setColors(Arrays.stream(colors).map(Color::new)
				.collect(Collectors.toList()));
	}

	/**
	 * Sets a series specific or series type specific color set to apply
	 * instead of the global {@link Charts#getColors()} when
	 * {@link #setColorByPoint(boolean)} is true.
	 * @since 10.2.1.0
	 */
	public void setColors(Color... colors) {
		setColors(Arrays.asList(colors));
	}

	/**
	 * Returns the column size - how many X axis units each column in the heatmap
	 * should span. (Requires heatmap or tilemap)
	 * <p> Default: 1
	 * @since 10.2.1.0
	 */
	public Number getColsize() {
		return getAttr(DAttrs.colsize, 1).asNumber();
	}

	/**
	 * Sets the column size - how many X axis units each column in the heatmap
	 * should span. (Requires heatmap or tilemap)
	 * <p> Default: 1
	 * @since 10.2.1.0
	 */
	public void setColsize(Number colsize) {
		setAttr(DAttrs.colsize, colsize);
	}

	/**
	 * Returns whether to connect the ends of a line series plot across the extremes.
	 * (Polar charts only).
	 * <p>Default: false.
	 * @since 10.2.1.0
	 */
	public boolean isConnectEnds() {
		return getAttr(DAttrs.connectEnds, false).asBoolean();
	}

	/**
	 * Sets whether to connect the ends of a line series plot across the extremes.
	 * (Polar charts only).
	 * <p>Default: false.
	 * @since 10.2.1.0
	 */
	public void setConnectEnds(boolean connectEnds) {
		setAttr(DAttrs.connectEnds, connectEnds);
	}

	/**
	 * Returns whether to connect a graph line across null points, or render a gap
	 * between the two points on either side of the null.
	 * <p>Default: false.
	 * @since 10.2.1.0
	 */
	public boolean isConnectNulls() {
		return getAttr(DAttrs.connectNulls, false).asBoolean();
	}

	/**
	 * Sets whether to connect a graph line across null points, or render a gap
	 * between the two points on either side of the null.
	 * <p>Default: false.
	 * @since 10.2.1.0
	 */
	public void setConnectNulls(boolean connectNulls) {
		setAttr(DAttrs.connectNulls, connectNulls);
	}

	/**
	 * Returns the color of the line that connects the dumbbell point's values.
	 * By default it is the series' color.
	 *
	 * <p>Note: requires dumbbell or lollipop type</p>
	 *
	 * <p>Defaults to null.
	 * @since 10.2.1.0
	 */
	public String getConnectorColor() {
		return getAttr(DAttrs.connectorColor, null).asString();
	}

	/**
	 * Sets the color of the line that connects the dumbbell point's values.
	 * By default it is the series' color.
	 *
	 * <p>Note: requires dumbbell or lollipop type</p>
	 *
	 * <p>Defaults to null.
	 * @since 10.2.1.0
	 */
	public void setConnectorColor(String connectorColor) {
		setAttr(DAttrs.connectorColor, connectorColor);
	}

	/**
	 * Returns the pixel width of the line that connects the dumbbell point's values.
	 *
	 * <p>Note: requires dumbbell or lollipop type</p>
	 * <p>Default: 1
	 * @since 10.2.1.0
	 */
	public Number getConnectorWidth() {
		return getAttr(DAttrs.connectorWidth, 1).asNumber();
	}

	/**
	 * Sets the pixel width of the line that connects the dumbbell point's values.
	 *
	 * <p>Note: requires dumbbell or lollipop type</p>
	 * <p>Default: 1
	 * @since 10.2.1.0
	 * @param connectorWidth
	 */
	public void setConnectorWidth(Number connectorWidth) {
		setAttr(DAttrs.connectorWidth, connectorWidth);
	}

	/**
	 * When true, each point or column edge is rounded to its nearest pixel in order
	 * to render sharp on screen. In some cases, when there are a lot of densely
	 * packed columns, this leads to visible difference in column widths or distance
	 * between columns. In these cases, setting crisp to false may look better,
	 * even though each column is rendered blurry.
	 * <p>Default: true.
	 * @since 10.2.1.0
	 */
	public boolean isCrisp() {
		return getAttr(DAttrs.crisp, true).asBoolean();
	}

	/**
	 * When true, each point or column edge is rounded to its nearest pixel in order
	 * to render sharp on screen. In some cases, when there are a lot of densely
	 * packed columns, this leads to visible difference in column widths or distance
	 * between columns. In these cases, setting crisp to false may look better,
	 * even though each column is rendered blurry.
	 * <p>Default: true.
	 * @since 10.2.1.0
	 */
	public void setCrisp(boolean crisp) {
		setAttr(DAttrs.crisp, crisp);
	}

	/**
	 * When the series contains less points than the crop threshold, all points
	 * are drawn, even if the points fall outside the visible plot area at the
	 * current zoom. The advantage of drawing all points (including markers and
	 * columns), is that animation is performed on updates. On the other hand,
	 * when the series contains more points than the crop threshold, the series
	 * data is cropped to only contain points that fall within the plot area.
	 * The advantage of cropping away invisible points is to increase performance
	 * on large series.
	 *
	 * <p> Defaults to {@code 300}.
	 * @since 10.2.1.0
	 */
	public Number getCropThreshold() {
		return getAttr(DAttrs.cropThreshold, 300).asNumber();
	}

	/**
	 * When the series contains less points than the crop threshold, all points
	 * are drawn, even if the points fall outside the visible plot area at the
	 * current zoom. The advantage of drawing all points (including markers and
	 * columns), is that animation is performed on updates. On the other hand,
	 * when the series contains more points than the crop threshold, the series
	 * data is cropped to only contain points that fall within the plot area.
	 * The advantage of cropping away invisible points is to increase performance
	 * on large series.
	 *
	 * <p> Defaults to {@code 300}.
	 * @since 10.2.1.0
	 */
	public void setCropThreshold(Number cropThreshold) {
		setAttr(DAttrs.cropThreshold, cropThreshold);
	}

	/**
	 * Higher numbers makes the links in a sankey diagram or dependency wheelrender
	 * more curved. A {@code curveFactor} of 0 makes the lines straight.
	 * <p>Default: null. (depends on its implementation)
	 * @since 10.2.1.0
	 */
	public Number getCurveFactor() {
		return getAttr(DAttrs.curveFactor, null).asNumber();
	}

	/**
	 * Higher numbers makes the links in a sankey diagram or dependency wheelrender
	 * more curved. A {@code curveFactor} of 0 makes the lines straight.
	 * <p>Default: null. (depends on its implementation)
	 * @since 10.2.1.0
	 */
	public void setCurveFactor(Number curveFactor) {
		setAttr(DAttrs.curveFactor, curveFactor);
	}

	/**
	 * Returns a reserved subspace to store options and values for customized functionality.
	 * Here you can add additional data for your own event callbacks and formatter callbacks.
	 * <p>Default: null.
	 * @since 10.2.1.0
	 */
	public <K, V> Map<K, V> getCustom() {
		return Generics.cast(getAttr(DAttrs.custom, null).asValue());
	}

	/**
	 * Sets a reserved subspace to store options and values for customized functionality.
	 * Here you can add additional data for your own event callbacks and formatter callbacks.
	 * <p>Default: null.
	 * @since 10.2.1.0
	 */
	public <K, V> void setCustom(Map<K, V> custom) {
		setAttr(DAttrs.custom, custom);
	}

	/**
	 * Returns you can set the cursor to "pointer" if you have click events
	 * attached to the series, to signal to the user that the points and lines
	 * can be clicked.
	 * <p>
	 * Default: null
	 * @since 7.2.1.0
	 */
	public String getCursor() {
		return getAttr(DAttrs.cursor, null).asString();
	}

	/**
	 * Sets you can set the cursor to "pointer" if you have click events
	 * attached to the series, to signal to the user that the points and lines
	 * can be clicked.
	 * @since 7.2.1.0
	 */
	public void setCursor(String cursor) {
		setAttr(DAttrs.cursor, cursor);
	}

	/**
	 * Returns a name for the dash style to use for the graph. Applies only to
	 * series type having a graph, like <code>line</code>, <code>spline</code>,
	 * <code>area</code> and <code>scatter</code> in case it has a
	 * <code>lineWidth</code>. The value for the <code>dashStyle</code> include:
	 * <ul>
	 * <li>Solid</li>
	 * <li>ShortDash</li>
	 * <li>ShortDot</li>
	 * <li>ShortDashDot</li>
	 * <li>ShortDashDotDot</li>
	 * <li>Dot</li>
	 * <li>Dash</li>
	 * <li>LongDash</li>
	 * <li>DashDot</li>
	 * <li>LongDashDot</li>
	 * <li>LongDashDotDot</li>
	 * </ul>
	 * @since 10.2.1.0
	 */
	public String getDashStyle() {
		return getAttr(DAttrs.dashStyle, "Solid").asString();
	}

	/**
	 * Sets a name for the dash style to use for the graph. Applies only to
	 * series type having a graph, like <code>line</code>, <code>spline</code>,
	 * <code>area</code> and <code>scatter</code> in case it has a
	 * <code>lineWidth</code>. The value for the <code>dashStyle</code> include:
	 * <ul>
	 * <li>Solid</li>
	 * <li>ShortDash</li>
	 * <li>ShortDot</li>
	 * <li>ShortDashDot</li>
	 * <li>ShortDashDotDot</li>
	 * <li>Dot</li>
	 * <li>Dash</li>
	 * <li>LongDash</li>
	 * <li>DashDot</li>
	 * <li>LongDashDot</li>
	 * <li>LongDashDotDot</li>
	 * </ul>
	 * @since 10.2.1.0
	 */
	public void setDashStyle(String dashStyle) {
		if (!("Solid".equalsIgnoreCase(dashStyle)
				|| "ShortDash".equalsIgnoreCase(dashStyle)
				|| "ShortDot".equalsIgnoreCase(dashStyle)
				|| "ShortDashDot".equalsIgnoreCase(dashStyle)
				|| "ShortDashDotDot".equalsIgnoreCase(dashStyle)
				|| "Dot".equalsIgnoreCase(dashStyle)
				|| "Dash".equalsIgnoreCase(dashStyle)
				|| "LongDash".equalsIgnoreCase(dashStyle)
				|| "DashDot".equalsIgnoreCase(dashStyle)
				|| "LongDashDot".equalsIgnoreCase(dashStyle) || "LongDashDotDot"
					.equalsIgnoreCase(dashStyle)))
			throw new IllegalArgumentException("Unsupported style: ["
					+ dashStyle + "]");
		setAttr(DAttrs.dashStyle, dashStyle);
	}



	/**
	 * Returns the data grouping options for the wind barbs, if any.
	 * Otherwise, create a new one.
	 * @since 10.2.1.0
	 */
	public WindBarbDataGrouping getDataGrouping() {
		WindBarbDataGrouping dataGrouping = (WindBarbDataGrouping) getAttr(DAttrs.dataGrouping);
		if (dataGrouping == null) {
			dataGrouping = new WindBarbDataGrouping();
			setDataGrouping(dataGrouping);
		}
		return dataGrouping;
	}

	/**
	 * Sets the data grouping options for the wind barbs.
	 * @since 10.2.1.0
	 */
	public void setDataGrouping(WindBarbDataGrouping dataGrouping) {
		setAttr(DAttrs.dataGrouping, dataGrouping);
	}

	/**
	 * Indicates data is structured as columns instead of rows.
	 * <p>Default: false.
	 * @since 10.2.1.0
	 */
	public boolean isDataAsColumns() {
		return getAttr(DAttrs.dataAsColumns, false).asBoolean();
	}

	/**
	 * Indicates data is structured as columns instead of rows.
	 * <p>Default: false.
	 * @since 10.2.1.0
	 */
	public void setDataAsColumns(boolean dataAsColumns) {
		setAttr(DAttrs.dataAsColumns, dataAsColumns);
	}

	/**
	 * Returns the options of the draggable-points module allows points
	 * to be moved around or modified in the chart, if any. Otherwise, create a new one.
	 * In addition to the options mentioned under the dragDrop API structure,
	 * the module fires three events, point.dragStart, point.drag and point.drop.
	 * @since 7.2.1.0
	 */
	public DragDrop getDragDrop() {
		DragDrop dragDrop = (DragDrop) getAttr(DAttrs.dragDrop);
		if (dragDrop == null) {
			dragDrop = new DragDrop();
			setDragDrop(dragDrop);
		}
		return dragDrop;
	}

	/**
	 * Sets the options of the draggable-points module allows points
	 * to be moved around or modified in the chart.
	 * In addition to the options mentioned under the dragDrop API structure,
	 * the module fires three events, point.dragStart, point.drag and point.drop.
	 * @since 7.2.1.0
	 */
	public void setDragDrop(DragDrop dragDrop) {
		setAttr(DAttrs.dragDrop, dragDrop);
	}

	/**
	 * Flag to determine if nodes are draggable or not. (Requires networkgraph or packedbubble)
	 * <p>Default: {@code true}
	 * @since 10.2.1.0
	 */
	public boolean isDraggable() {
		return getAttr(DAttrs.draggable, true).asBoolean();
	}

	/**
	 * Flag to determine if nodes are draggable or not. (Requires networkgraph or packedbubble)
	 * <p>Default: {@code true}
	 * @since 10.2.1.0
	 */
	public void setDraggable(boolean draggable) {
		setAttr(DAttrs.draggable, draggable);
	}

	/**
	 * 3D columns only. The color of the edges. Similar to {@code borderColor},
	 * except it defaults to the same color as the column.
	 * <p>Default: {@code null}
	 * @since 10.2.1.0
	 */
	public Color getEdgeColor() {
		return (Color) getAttr(DAttrs.edgeColor, null).asValue();
	}

	/**
	 * 3D columns only. The color of the edges. Similar to {@code borderColor},
	 * except it defaults to the same color as the column.
	 * <p>Default: {@code null}
	 * @since 10.2.1.0
	 */
	public void setEdgeColor(Color color) {
		setAttr(DAttrs.edgeColor, color);
	}

	/**
	 * 3D columns only.
	 * Returns the width of the colored edges.
	 * <p>
	 * Default: 1
	 * @since 12.2.1.0
	 */
	public Number getEdgeWidth() {
		return getAttr(DAttrs.edgeWidth, 1).asNumber();
	}

	/**
	 * 3D columns only.
	 * Sets the width of the colored edges.
	 * @since 12.2.1.0
	 */
	public void setEdgeWidth(Number edgeWidth) {
		setAttr(DAttrs.edgeWidth, edgeWidth);
	}

	/**
	 * 3D columns only. The color of the edges. Similar to {@code borderColor},
	 * except it defaults to the same color as the column.
	 * <p>Default: {@code null}
	 * @since 10.2.1.0
	 */
	public void setEdgeColor(String color) {
		setColor(new Color(color));
	}

	/**
	 * Returns enable or disable the mouse tracking for a specific series. This
	 * includes point tooltips and click events on graphs and points. For large
	 * datasets it improves performance.
	 * @since 7.2.1.0
	 */
	public boolean isEnableMouseTracking() {
		return getAttr(DAttrs.enableMouseTracking, true).asBoolean();
	}

	/**
	 * Sets enable or disable the mouse tracking for a specific series. This
	 * includes point tooltips and click events on graphs and points. For large
	 * datasets it improves performance.
	 * @since 7.2.1.0
	 */
	public void setEnableMouseTracking(boolean enableMouseTracking) {
		setAttr(DAttrs.enableMouseTracking, enableMouseTracking);
	}

	/**
	 * Returns whether nodes with different values should have the same size.
	 * If set to true, all nodes are calculated based on the nodePadding and current
	 * plotArea. (Requires arcdiagram)
	 *
	 * <p>
	 * Default: {@code false}.
	 * @since 10.2.1.0
	 */
	public boolean isEqualNodes() {
		return getAttr(DAttrs.equalNodes, false).asBoolean();
	}

	/**
	 * Sets whether nodes with different values should have the same size.
	 * If set to true, all nodes are calculated based on the nodePadding and current
	 * plotArea. (Requires arcdiagram)
	 * @since 10.2.1.0
	 */
	public void setEqualNodes(boolean equalNodes) {
		setAttr(DAttrs.equalNodes, equalNodes);
	}

	/**
	 * In circular view, returns the end angle of the item layout, in degrees where 0 is up.
	 * Default: null
	 * @since 7.2.1.0
	 */
	public Number getEndAngle() {
		return getAttr(DAttrs.endAngle, null).asNumber();
	}

	/**
	 * In circular view, sets the end angle of the item layout, in degrees where 0 is up.
	 * @since 7.2.1.0
	 */
	public void setEndAngle(Number endAngle) {
		setAttr(DAttrs.endAngle, endAngle);
	}

	/**
	 * Returns general event handlers for the series items.
	 * @since 11.2.0.0
	 */
	public Map<String, JavaScriptValue> getEventsMap() {
		Map<String, JavaScriptValue> map = Generics.cast(getAttr(DAttrs.events, null).asValue());
		if (map == null) {
			map = new LinkedHashMap<String, JavaScriptValue>();
			setEvents(map);
		}
		return Generics.cast(getAttr(DAttrs.events, null).asValue());
	}

	/**
	 * Sets a map of general event handlers for the series items.
	 * @since 11.2.0.0
	 */
	public void setEvents(Map<String, JavaScriptValue> events) {
		setAttr(DAttrs.events, events, NOT_NULL_VALUE);
	}

	/**
	 * Sets general event handlers for the series items.
	 * @since 11.2.0.0
	 */
	@SuppressWarnings("nls")
	public void addEvent(String name, JavaScriptValue event) {
		if (!("afterAnimate".equals(name) || "checkboxClick".equals(name) ||
				"click".equals(name) || "hide".equals(name) ||
				"legendItemClick".equals(name) || "mouseOut".equals(name) ||
				"mouseOver".equals(name) || "show".equals(name)))
			throw new IllegalArgumentException("Unsupported event type: [" + name + "]");

		getEventsMap().put(name, event);
	}

	/**
	 * Returns fill color or gradient for the area.
	 * When null, the series' color is used with the series' fillOpacity.
	 * <p>
	 * Default: null
	 * @since 7.2.1.0
	 */
	public Color getFillColor() {
		return (Color) getAttr(DAttrs.fillColor, null).asValue();
	}

	/**
	 * Sets fill color for the area.
	 * When null, the series' color is used with the series' fillOpacity.
	 * @since 7.2.1.0
	 */
	public void setFillColor(Color color) {
		setAttr(DAttrs.fillColor, color, NOT_NULL_VALUE);
	}

	/**
	 * Sets fill color for the area.
	 * When null, the series' color is used with the series' fillOpacity.
	 * @since 7.2.1.0
	 */
	public void setFillColor(String color) {
		setFillColor(new Color(color));
	}

	/**
	 * Sets fill gradient for the area.
	 * When null, the series' color is used with the series' fillOpacity.
	 * @since 7.2.1.0
	 */
	public void setFillColor(LinearGradient color) {
		setFillColor(new Color(color));
	}

	/**
	 * Sets fill gradient for the area.
	 * When null, the series' color is used with the series' fillOpacity.
	 * @since 7.2.1.0
	 */
	public void setFillColor(RadialGradient color) {
		setFillColor(new Color(color));
	}

	/**
	 * Returns fill opacity for the area, arearange, areaspline,  areasplinerange, bellcurve, and streamgraph.
	 * <p>
	 * Default: 0.75.
	 * @since 10.2.1.0
	 */
	public Number getFillOpacity() {
		return getAttr(DAttrs.fillOpacity, 0.75).asNumber();
	}

	/**
	 * Sets fill opacity for the area, arearange, areaspline,  areasplinerange, bellcurve, and streamgraph.
	 * @since 10.2.1.0
	 */
	public void setFillOpacity(Number fillOpacity) {
		setAttr(DAttrs.fillOpacity, fillOpacity);
	}

	/**
	 * Returns whether the series should look for the nearest point in both
	 * dimensions or just the x-dimension when hovering the series.
	 * Defaults to 'xy' for scatter series and 'x' for most other series.
	 * If the data has duplicate x-values, it is recommended to set this
	 * to 'xy' to allow hovering over all points.
	 *
	 * Applies only to series types using nearest neighbor search (not direct hover) for tooltip.
	 * <p>
	 * Default: "x"
	 * @since 10.2.1.0
	 */
	public String getFindNearestPointBy() {
		return getAttr(DAttrs.findNearestPointBy, "x").asString();
	}

	/**
	 * Sets whether the series should look for the nearest point in both
	 * dimensions or just the x-dimension when hovering the series.
	 * Defaults to 'xy' for scatter series and 'x' for most other series.
	 * If the data has duplicate x-values, it is recommended to set this
	 * to 'xy' to allow hovering over all points.
	 *
	 * Applies only to series types using nearest neighbor search (not direct hover) for tooltip.
	 * @since 10.2.1.0
	 */
	public void setFindNearestPointBy(String findNearestPointBy) {
		setAttr(DAttrs.findNearestPointBy, findNearestPointBy);
	}

	/**
	 * Returns whether to use the Y extremes of the total chart width or
	 * only the zoomed area when zooming in on parts of the X axis.
	 * By default, the Y axis adjusts to the min and max of the visible data. Cartesian series only.
	 * <p>
	 * Default: false
	 * @since 10.2.1.0
	 */
	public boolean isGetExtremesFromAll() {
		return getAttr(DAttrs.getExtremesFromAll, false).asBoolean();
	}

	/**
	 * Sets whether to use the Y extremes of the total chart width or
	 * only the zoomed area when zooming in on parts of the X axis.
	 * By default, the Y axis adjusts to the min and max of the visible data. Cartesian series only.
	 * @since 10.2.1.0
	 */
	public void setGetExtremesFromAll(boolean getExtremesFromAll) {
		setAttr(DAttrs.getExtremesFromAll, getExtremesFromAll);
	}

	/**
	 * By default sides fill is set to a gradient through this option being set to true.
	 * Set to false to get solid color for the sides. (Requires funnel3d or pyramid3d)
	 * <p>
	 * Default: true
	 * @since 10.2.1.0
	 */
	public boolean isGradientForSides() {
		return getAttr(DAttrs.gradientForSides, true).asBoolean();
	}

	/**
	 * By default sides fill is set to a gradient through this option being set to true.
	 * Set to false to get solid color for the sides. (Requires funnel3d or pyramid3d)
	 * @since 10.2.1.0
	 */
	public void setGradientForSides(boolean gradientForSides) {
		setAttr(DAttrs.gradientForSides, gradientForSides);
	}

	/**
	 * Returns whether to group non-stacked columns or to let them render independent
	 * of each other. Non-grouped columns will be laid out individually and overlap each other.
	 * <p>Default: {@code true}
	 * @since 10.2.1.0
	 */
	public boolean isGrouping() {
		return getAttr(DAttrs.grouping, true).asBoolean();
	}

	/**
	 * Sets whether to group non-stacked columns or to let them render independent
	 * of each other. Non-grouped columns will be laid out individually and overlap each other.
	 * <p>Default: {@code true}
	 * @since 10.2.1.0
	 */
	public void setGrouping(boolean grouping) {
		setAttr(DAttrs.grouping, grouping);
	}

	/**
	 * Returns padding between each value groups, in x axis units.
	 * <p>Default: 0.2
	 * @since 10.2.1.0
	 */
	public Number getGroupPadding() {
		return getAttr(DAttrs.groupPadding, 0.2).asNumber();
	}

	/**
	 * Sets padding between each value groups, in x axis units.
	 * @since 10.2.1.0
	 */
	public void setGroupPadding(Number groupPadding) {
		setAttr(DAttrs.groupPadding, groupPadding);
	}

	/**
	 * Returns the spacing between columns on the Z Axis in a 3D chart.
	 * <p>
	 * Default: 1
	 * @since 10.2.1.0
	 */
	public Number getGroupZPadding() {
		return getAttr(DAttrs.groupZPadding, 1).asNumber();
	}

	/**
	 * Sets the spacing between columns on the Z Axis in a 3D chart.
	 * <p>
	 * Default: 1
	 * @since 10.2.1.0
	 */
	public void setGroupZPadding(Number groupZPadding) {
		setAttr(DAttrs.groupZPadding, groupZPadding);
	}

	public int getGapSize() {
		return getAttr(DAttrs.gapSize, 0).asInt();
	}

	public void setGapSize(int gapSize) {
		setAttr(DAttrs.gapSize, gapSize, 0);
	}

	/**
	 * Returns the indentation in pixels of hanging nodes, nodes which parent has layout set to hanging.
	 * (Requires organization)
	 * <p>
	 * Default: 20
	 * @since 10.2.1.0
	 */
	public Number getHangingIndent() {
		return getAttr(DAttrs.hangingIndent, 20).asNumber();
	}

	/**
	 * Sets the indentation in pixels of hanging nodes, nodes which parent has layout set to hanging.
	 * (Requires organization)
	 * @since 10.2.1.0
	 */
	public void setHangingIndent(Number hangingIndent) {
		setAttr(DAttrs.hangingIndent, hangingIndent);
	}

	/**
	 * Defines the indentation of a hanging layout parent's children. (Requires organization)
	 * <p>
	 * Possible options:
	 * <ul><li>
	 * {@code "inherit"} (default): Only the first child adds the indentation,
	 * children of a child with indentation inherit the indentation.
	 * </li><li>
	 * {@code "cumulative"}: All children of a child with indentation add its own indent.
	 * The option may cause overlapping of nodes. Then use shrink option:
	 * </li><li>
	 * {@code "shrink"}: Nodes shrink by the hangingIndent value until they reach the minNodeLength.
	 * </li></ul>
	 * <p>
	 * Defaults to {@code "inherit"}.
	 * @since 10.2.1.0
	 */
	public String getHangingIndentTranslation() {
		return getAttr(DAttrs.hangingIndentTranslation, "inherit").asString();
	}

	/**
	 * Defines the indentation of a hanging layout parent's children. (Requires organization)
	 * <p>
	 * Possible options:
	 * <ul><li>
	 * {@code "inherit"} (default): Only the first child adds the indentation,
	 * children of a child with indentation inherit the indentation.
	 * </li><li>
	 * {@code "cumulative"}: All children of a child with indentation add its own indent.
	 * The option may cause overlapping of nodes. Then use shrink option:
	 * </li><li>
	 * {@code "shrink"}: Nodes shrink by the hangingIndent value until they reach the minNodeLength.
	 * </li></ul>
	 * <p>
	 * Defaults to {@code "inherit"}.
	 * @since 10.2.1.0
	 */
	public void setHangingIndentTranslation(String hangingIndentTranslation) {
		if (!("inherit".equals(hangingIndentTranslation) || "cumulative".equals(hangingIndentTranslation)
		|| "shrink".equals(hangingIndentTranslation))) {
			throw new IllegalArgumentException("Not supported value. [" + hangingIndentTranslation + "]");
		}
		setAttr(DAttrs.hangingIndentTranslation, hangingIndentTranslation);
	}

	/**
	 * Returns the height of the neck, the lower part of a funnel. If it is a
	 * number it defines the pixel height, if it is a percentage string it is
	 * the percentage of the plot area height. (Requires funnel, funnel3d, pyramid, or pyramid3d)
	 * <p>
	 * Default: 100%
	 * @since 10.2.1.0
	 */
	public String getHeight() {
		return getAttr(DAttrs.height, "100%").asString();
	}

	/**
	 * Sets the height of the neck, the lower part of a funnel. If it is a
	 * number it defines the pixel height, if it is a percentage string it is
	 * the percentage of the plot area height. (Requires funnel, funnel3d, pyramid, or pyramid3d)
	 * <p>
	 * Default: 100%
	 * @since 10.2.1.0
	 */
	public void setHeight(String height) {
		setAttr(DAttrs.height, height, "100%");
	}


	/**
	 * Returns the layout of the items in rectangular view.
	 * Can be either horizontal or vertical
	 * <p>
	 * Default: "vertical"
	 * @since 7.2.1.0
	 */
	public String getLayout() {
		return getAttr(DAttrs.layout, "vertical").asString();
	}

	/**
	 * Sets the layout of the items in rectangular view.
	 * Can be either horizontal or vertical
	 * @since 7.2.1.0
	 */
	public void setLayout(String layout) {
		if (!("horizontal".equals(layout) || "vertical".equals(layout)))
			throw new IllegalArgumentException("Unsupported layout: [ " + layout + " ]");
		setAttr(DAttrs.layout, layout);
	}

	/**
	 * Get the option that decides which algorithm is used for setting position and dimensions of the points.
	 * (Requires treemap)
	 * <p>
	 * Default: "sliceAndDice"
	 * @return either "sliceAndDice", "stripes", "squarified", or "strip"
	 * @since 10.2.1.0
	 */
	public String getLayoutAlgorithm() {
		return getAttr(DAttrs.layoutAlgorithm, "sliceAndDice").asString();
	}
	/**
	 * Set the option decides which algorithm is used for setting position and dimensions of the points.
	 * (Requires treemap)
	 * <p>
	 * @param layoutAlgorithm
	 * @since 10.2.1.0
	 */
	public void setLayoutAlgorithm(String layoutAlgorithm) {
		if (!("sliceAndDice".equals(layoutAlgorithm) ||
			"stripes".equals(layoutAlgorithm) ||
			"squarified".equals(layoutAlgorithm) ||
			"strip".equals(layoutAlgorithm))) {
			throw new IllegalArgumentException("Unsupported layout algorithm: [" + layoutAlgorithm + "]");
		}
		setAttr(DAttrs.layoutAlgorithm, layoutAlgorithm, "sliceAndDice");
	}

	/**
	 * Return options for layout algorithm when simulation is enabled, if any. Otherwise, create a new one.
	 * (Requires packedbubble type)
	 * @since 10.2.1.0
	 */
	public PackedBubbleLayoutAlgorithm getPackedBubbleLayoutAlgorithm() {
		Object layoutAlgorithm = getAttr(DAttrs.layoutAlgorithm);
		if (!(layoutAlgorithm instanceof PackedBubbleLayoutAlgorithm)) {
			layoutAlgorithm = new PackedBubbleLayoutAlgorithm();
			setLayoutAlgorithm((PackedBubbleLayoutAlgorithm) layoutAlgorithm);
		}
		return (PackedBubbleLayoutAlgorithm) layoutAlgorithm;
	}

	/**
	 * Sets options for layout algorithm when simulation is enabled. for packedbubble type
	 * @since 10.2.1.0
	 */
	public void setLayoutAlgorithm(PackedBubbleLayoutAlgorithm layoutAlgorithm) {
		setAttr(DAttrs.layoutAlgorithm, layoutAlgorithm);
	}

	/**
	 * Return options for layout algorithm for networkgraph type, if any. Otherwise, create a new one.
	 * (Requires networkgraph type)
	 * @since 10.2.1.0
	 */
	public NetworkGraphLayoutAlgorithm getNetworkGraphLayoutAlgorithm() {
		Object layoutAlgorithm = getAttr(DAttrs.layoutAlgorithm);
		if (!(layoutAlgorithm instanceof NetworkGraphLayoutAlgorithm)) {
			layoutAlgorithm = new NetworkGraphLayoutAlgorithm();
			setLayoutAlgorithm((NetworkGraphLayoutAlgorithm) layoutAlgorithm);
		}
		return (NetworkGraphLayoutAlgorithm) layoutAlgorithm;
	}

	/**
	 * Sets options for layout algorithm for networkgraph type
	 * @since 10.2.1.0
	 */
	public void setLayoutAlgorithm(NetworkGraphLayoutAlgorithm layoutAlgorithm) {
		setAttr(DAttrs.layoutAlgorithm, layoutAlgorithm);
	}

	/**
	 * Defines which direction the layout algorithm will start drawing. (Requires treemap)
	 * <p>
	 * Default: "vertical"
	 * @return either "vertical" or "horizontal
	 * @since 10.2.1.0
	 */
	public String getLayoutStartingDirection() {
		return getAttr(DAttrs.layoutStartingDirection, "vertical").asString();
	}

	/**
	 * Set which direction the layout algorithm will start drawing. (Requires treemap)
	 * <p>
	 * Possible values are "vertical" and "horizontal".
	 * @param layoutStartingDirection
	 * @see #getLayoutStartingDirection()
	 * @since 10.2.1.0
	 */
	public void setLayoutStartingDirection(String layoutStartingDirection) {
		if (!("vertical".equalsIgnoreCase(layoutStartingDirection) || "horizontal".equalsIgnoreCase(layoutStartingDirection))) {
			throw new IllegalArgumentException("Unsopported layout starting direction:[" + layoutStartingDirection + "]");
		}
		setAttr(DAttrs.layoutStartingDirection, layoutStartingDirection, "vertical");
	}

	/**
	 * Sets the sequential index of the series in the legend.
	 */
	public void setLegendIndex(int legendIndex) {
		setAttr(DAttrs.legendIndex, legendIndex);
	}

	/**
	 * Returns the sequential index of the series in the legend.
	 */
	public int getLegendIndex() {
		return getAttr(DAttrs.legendIndex, -1).asInt();
	}

	/**
	 * Used together with the levels and allowDrillToNode options. When set to
	 * false the first level visible when drilling is considered to be level
	 * one. Otherwise the level will be the same as the tree structure. (Requires
	 * treemap or sunburst)
	 * <p>
	 * Defaults to true.
	 * @since 10.2.1.0
	 */
	public boolean isLevelIsConstant() {
		return getAttr(DAttrs.levelIsConstant, true).asBoolean();
	}

	/**
	 * Used together with the levels and allowDrillToNode options. When set to
	 * false the first level visible when drilling is considered to be level
	 * one. Otherwise the level will be the same as the tree structure. (Requires
	 * treemap or sunburst)
	 * <p>
	 * Defaults to true.
	 * @since 10.2.1.0
	 */
	public void setLevelIsConstant(boolean levelIsConstant) {
		setAttr(DAttrs.levelIsConstant, levelIsConstant, true);
	}

	/**
	 * Returns options on specific levels. Takes precedence over series options,
	 * but not node and link options.
	 * <p>
	 * Default: null
	 * @since 10.2.1.0
	 */
	public List<Level> getLevels() {
		return Generics.cast(getAttr(DAttrs.levels, null).asValue());
	}

	/**
	 * Set options on specific levels. Takes precedence over series options,
	 * @since 10.2.1.0
	 * but not node and link options.
	 */
	public void setLevels(List<Level> levels) {
		setAttr(DAttrs.levels, levels);
	}

	/**
	 * Set options on specific levels. Takes precedence over series options,
	 * but not node and link options.
	 * @since 10.2.1.0
	 */
	public void setLevels(Level... levels) {
		setLevels(Arrays.asList(levels));
	}

	/**
	 * Returns options of determining the width of the ring per level, if any.
	 * Otherwise, create a new one. (Requires sunburst)
	 * @since 10.2.1.0
	 */
	public LevelSize getLevelSize() {
		LevelSize levelSize = (LevelSize) getAttr(DAttrs.levelSize);
		if (levelSize == null) {
			levelSize = new LevelSize();
			setLevelSize(levelSize);
		}
		return levelSize;
	}

	/**
	 * Sets options of determining the width of the ring per level. (Requires sunburst)
	 * @since 10.2.1.0
	 */
	public void setLevelSize(LevelSize levelSize) {
		setAttr(DAttrs.levelSize, levelSize);
	}
	/**
	 * Returns the line cap used for line ends and line joins on the graph.
	 * <p>
	 * Default: {@code "round"}
	 * @since 10.2.1.0
	 */
	public String getLinecap() {
		return getAttr(DAttrs.linecap, "round").asString();
	}

	/**
	 * Sets the line cap used for line ends and line joins on the graph.
	 * @since 10.2.1.0
	 */
	public void setLinecap(String linecap) {
		setAttr(DAttrs.linecap, linecap);
	}

	/**
	 * Returns a separate color for the graph line. By default the line takes
	 * the <code>color</code> of the series, but the lineColor setting allows
	 * setting a separate color for the line without altering the
	 * <code>fillColor</code>.
	 * <p>
	 * Default: null.
	 * @since 10.2.1.0
	 */
	public Color getLineColor() {
		return (Color) getAttr(DAttrs.lineColor, null).asValue();
	}

	/**
	 * Sets a separate color for the graph line. By default the line takes the
	 * <code>color</code> of the series, but the lineColor setting allows
	 * setting a separate color for the line without altering the
	 * <code>fillColor</code>.
	 * @since 10.2.1.0
	 */
	public void setLineColor(Color color) {
		setAttr(DAttrs.lineColor, color);
	}

	/**
	 * Sets a separate color for the graph line. By default the line takes the
	 * <code>color</code> of the series, but the lineColor setting allows
	 * setting a separate color for the line without altering the
	 * <code>fillColor</code>.
	 * @since 10.2.1.0
	 */
	public void setLineColor(String color) {
		setLineColor(new Color(color));
	}

	/**
	 * Sets a separate color for the graph line. By default the line takes the
	 * <code>color</code> of the series, but the lineColor setting allows
	 * setting a separate color for the line without altering the
	 * <code>fillColor</code>.
	 * @since 10.2.1.0
	 */
	public void setLineColor(LinearGradient color) {
		setLineColor(new Color(color));
	}

	/**
	 * Sets a separate color for the graph line. By default the line takes the
	 * <code>color</code> of the series, but the lineColor setting allows
	 * setting a separate color for the line without altering the
	 * <code>fillColor</code>.
	 * @since 10.2.1.0
	 */
	public void setLineColor(RadialGradient color) {
		setLineColor(new Color(color));
	}

	/**
	 * Returns the width of the point marker's outline.
	 * <p>Default: 2
	 */
	public Number getLineWidth() {
		return getAttr(DAttrs.lineWidth, 2).asNumber();
	}

	/**
	 * Sets the width of the point marker's outline.
	 * <p>Default: 2
	 */
	public void setLineWidth(Number lineWidth) {
		setAttr(DAttrs.lineWidth, lineWidth, 2);
	}

	/**
	 * Returns the link style options, if any. Otherwise, create a new one.
	 * @since 7.2.1.0
	 */
	public NetworkGraphLink getLink() {
		NetworkGraphLink link = (NetworkGraphLink) getAttr(DAttrs.link);
		if (link == null) {
			link = new NetworkGraphLink();
			setLink(link);
		}
		return link;
	}

	/**
	 * Sets the link style options.
	 * @since 7.2.1.0
	 */
	public void setLink(NetworkGraphLink link) {
		setAttr(DAttrs.link, link);
	}

	/**
	 * Returns the tree graph link style options, if any. Otherwise, create a new one.
	 * @since 10.3.3.0
	 */
	public TreeGraphLink getTreeGraphLink() {
		TreeGraphLink link = (TreeGraphLink) getAttr(DAttrs.link);
		if (link == null) {
			link = new TreeGraphLink();
			setTreeGraphLink(link);
		}
		return link;
	}

	/**
	 * Sets the tree graph link style options.
	 * @since 10.3.3.0
	 */
	public void setTreeGraphLink(TreeGraphLink link) {
		setAttr(DAttrs.link, link);
	}

	/**
	 * Returns the color of the links between nodes.(Requires organization)
	 * <p>
	 * Default: #666666
	 * @since 10.2.1.0
	 */
	public Color getLinkColor() {
		return (Color) getAttr(DAttrs.linkColor, new Color("#666666")).asValue();
	}

	/**
	 * Sets the color of the links between nodes.(Requires organization)
	 * @since 10.2.1.0
	 */
	public void setLinkColor(Color linkColor) {
		setAttr(DAttrs.linkColor, linkColor);
	}

	/**
	 * Sets the color of the links between nodes. (Requires organization)
	 * @since 10.2.1.0
	 */
	public void setLinkColor(String linkColor) {
		setLinkColor(new Color(linkColor));
	}

	/**
	 * Returns the id of another series to link to. Additionally, the value can be ":previous" to link to the
	 * previous series. When two series are linked, only the first one appears in the legend. Toggling the
	 * visibility of this also toggles the linked series.
	 * 
	 * If master series uses data sorting and linked series does not have its own sorting definition, the linked
	 * series will be sorted in the same order as the master one.
	 * 
	 * <p>
	 * Default: null.
	 * @since 7.2.2.0
	 */
	public String getLinkedTo() {
		return getAttr(DAttrs.linkedTo, null).asString();
	}

	/**
	 * Sets the id of another series to link to. Additionally, the value can be ":previous" to link to the
	 * previous series. When two series are linked, only the first one appears in the legend. Toggling the
	 * visibility of this also toggles the linked series.
	 * 
	 * If master series uses data sorting and linked series does not have its own sorting definition, the linked
	 * series will be sorted in the same order as the master one.
	 * @since 7.2.2.0
	 */
	public void setLinkedTo(String linkedTo) {
		setAttr(DAttrs.linkedTo, linkedTo);
	}

	/**
	 * Returns the line width of the links connecting nodes, in pixels.(Requires organization)
	 * <p>
	 * Default: 1
	 * @since 10.2.1.0
	 */
	public Number getLinkLineWidth() {
		return getAttr(DAttrs.linkLineWidth, 1).asNumber();
	}

	/**
	 * Sets the line width of the links connecting nodes, in pixels. (Requires organization)
	 * @since 10.2.1.0
	 */
	public void setLinkLineWidth(Number linkLineWidth) {
		setAttr(DAttrs.linkLineWidth, linkLineWidth);
	}
	/**
	 * Returns linkOpacity on all points which lay on the same level. (Requires
	 * arcdiagram or sankey)
	 * <p>
	 * Default: {@code 0.5}
	 * @since 10.2.1.0
	 */
	public Number getLinkOpacity() {
		return getAttr(DAttrs.linkOpacity, 0.5).asNumber();
	}

	/**
	 * Sets linkOpacity on all points which lay on the same level.
	 * (Requires arcdiagram or sankey)
	 * @since 10.2.1.0
	 */
	public void setLinkOpacity(Number linkOpacity) {
		setAttr(DAttrs.linkOpacity, linkOpacity);
	}

	/**
	 * Returns radius for the rounded corners of the links between nodes.
	 * (Requires organization)
	 * <p>
	 * Default: 10
	 * @since 10.2.1.0
	 */
	public Number getLinkRadius() {
		return getAttr(DAttrs.linkRadius, 10).asNumber();
	}

	/**
	 * Sets radius for the rounded corners of the links between nodes.
	 * (Requires organization)
	 * @since 10.2.1.0
	 */
	public void setLinkRadius(Number linkRadius) {
		setAttr(DAttrs.linkRadius, linkRadius);
	}


	/**
	 * The global link weight. If not set, width is calculated per link,
	 * depending on the weight value. (Requires arcdiagram)
	 *
	 * <p>Defaults to {@code null}.
	 * @since 10.2.1.0
	 */
	public Number getLinkWeight() {
		return getAttr(DAttrs.linkWeight, null).asNumber();
	}

	/**
	 * The global link weight. If not set, width is calculated per link,
	 * depending on the weight value. (Requires arcdiagram)
	 *
	 * <p>Defaults to {@code null}.
	 * @since 10.2.1.0
	 */
	public void setLinkWeight(Number linkWeight) {
		setAttr(DAttrs.linkWeight, linkWeight);
	}

	/**
	 * Returns color of the start markers in a dumbbell graph.
	 * <p>
	 * Default: {@code "#333333"}.
	 * @since 10.2.1.0
	 */
	public Color getLowColor() {
		return (Color) getAttr(DAttrs.lowColor, new Color("#333333")).asValue();
	}

	/**
	 * Sets color of the start markers in a dumbbell graph.
	 * <p>
	 * Default: {@code "#333333"}.
	 * @since 10.2.1.0
	 */
	public void setLowColor(Color color) {
		setAttr(DAttrs.lowColor, color);
	}

	/**
	 * Sets color of the start markers in a dumbbell graph.
	 * <p>
	 * Default: {@code "#333333"}.
	 * @since 10.2.1.0
	 */
	public void setLowColor(String color) {
		setLowColor(new Color(color));
	}

	/**
	 * Sets color of the start markers in a dumbbell graph.
	 * <p>
	 * Default: {@code "#333333"}.
	 * @since 10.2.1.0
	 */
	public void setLowColor(LinearGradient color) {
		setLowColor(new Color(color));
	}

	/**
	 * Sets color of the start markers in a dumbbell graph.
	 * <p>
	 * Default: {@code "#333333"}.
	 * @since 10.2.1.0
	 */
	public void setLowColor(RadialGradient color) {
		setLowColor(new Color(color));
	}

	/**
	 * Returns options for the lower markers of the dumbbell-like series.
	 * When lowMarker is not defined, options inherit form the marker.
	 * @since 11.2.0.0
	 */
	public LowMarker getLowMarker() {
		LowMarker lowMarker = (LowMarker) getAttr(DAttrs.lowMarker);
		if (lowMarker == null) {
			lowMarker = new LowMarker();
			setAttr(DAttrs.lowMarker, lowMarker);
		}
		return lowMarker;
	}

	/**
	 * Sets options for the lower markers of the dumbbell-like series.
	 * When lowMarker is not defined, options inherit form the marker.
	 * @since 11.2.0.0
	 */
	public void setLowMarker(LowMarker lowMarker) {
		setAttr(DAttrs.lowMarker, lowMarker);
	}

	/**
	 * Returns the marker for this series.
	 */
	public Marker getMarker() {
		Marker marker = (Marker) getAttr(DAttrs.marker);
		if (marker == null) {
			marker = new Marker();
			setAttr(DAttrs.marker, marker);
		}
		return marker;
	}

	/**
	 * Sets the marker for this series.
	 */
	public void setMarker(Marker marker) {
		setAttr(DAttrs.marker, marker);
	}

	/**
	 * Returns the word with the largest weight will have a font size equal to this value.
	 * The font size of a word is the ratio between its weight and the largest occurring weight,
	 * multiplied with the value of maxFontSize. (Requires wordcloud)
	 * <p>
	 * Default: 25
	 * @since 10.2.1.0
	 */
	public Number getMaxFontSize() {
		return getAttr(DAttrs.maxFontSize, 25).asNumber();
	}

	/**
	 * Sets the word with the largest weight will have a font size equal to this value.
	 * The font size of a word is the ratio between its weight and the largest occurring weight,
	 * multiplied with the value of maxFontSize. (Requires wordcloud)
	 * @since 10.2.1.0
	 */
	public void setMaxFontSize(Number maxFontSize) {
		setAttr(DAttrs.maxFontSize, maxFontSize);
	}

	/**
	 * Returns the maximum size of the points' radius related to chart's plotArea.
	 * If a number is set, it applies in pixels. (Requires variablepie)
	 * <p>
	 * Default: "100%"
	 * @since 10.2.1.0
	 */
	public Object getMaxPointSize() {
		return getAttr(DAttrs.maxPointSize, "100%").asValue();
	}

	/**
	 * Sets the maximum size of the points' radius related to chart's plotArea.
	 * If a number is set, it applies in pixels. (Requires variablepie)
	 * @since 10.2.1.0
	 */
	public void setMaxPointSize(Number maxPointSize) {
		setAttr(DAttrs.maxPointSize, maxPointSize);
	}

	/**
	 * Returns the maximum allowed pixel width for a column,
	 * translated to the height of a bar in a bar chart.
	 * This prevents the columns from becoming too wide when there is a small
	 * number of points in the chart.
	 * <p>Default: null
	 * @since 10.2.1.0
	 */
	public Number getMaxPointWidth() {
		return getAttr(DAttrs.maxPointWidth, null).asNumber();
	}

	/**
	 * Sets the maximum allowed pixel width for a column,
	 * translated to the height of a bar in a bar chart.
	 * This prevents the columns from becoming too wide when there is a small
	 * number of points in the chart.
	 * @since 10.2.1.0
	 */
	public void setMaxPointWidth(Number maxPointWidth) {
		setAttr(DAttrs.maxPointWidth, maxPointWidth);
	}

	/**
	 * Returns maximum bubble size.
	 * Bubbles will automatically size between the minSize and maxSize to reflect the z value of each bubble.
	 * Can be either pixels (when no unit is given), or a percentage of the smallest one of the plot width and height.
	 * <p>
	 * Default: "20%"
	 * @since 7.2.1.0
	 */
	public Object getMaxSize() {
		return getAttr(DAttrs.maxSize, "20%");
	}

	/**
	 * Sets maximum bubble size.
	 * Bubbles will automatically size between the minSize and maxSize to reflect the z value of each bubble.
	 * Can be pixels (when no unit is given).
	 * @since 7.2.1.0
	 */
	public void setMaxSize(Number maxSize) {
		setAttr(DAttrs.maxSize, maxSize);
	}

	/**
	 * Sets maximum bubble size.
	 * Bubbles will automatically size between the minSize and maxSize to reflect the z value of each bubble.
	 * Can be a percentage of the smallest one of the plot width and height.
	 * @since 7.2.1.0
	 */
	public void setMaxSize(String maxSize) {
		setAttr(DAttrs.maxSize, maxSize);
	}

	/**
	 * Returns the color of the median line. If <code>null</code>, the general
	 * series color applies. (Require boxplot)
	 * <p>Default: {@code null}
	 * @since 10.2.1.0
	 */
	public String getMedianColor() {
		return getAttr(DAttrs.medianColor, null).asString();
	}

	/**
	 * Sets the color of the median line. If <code>null</code>, the general
	 * series color applies. 'medianColor the color of the median line. If
	 * <code>null</code>, the general series color applies. (Require boxplot)
	 * <p>Default: {@code null}
	 * @since 10.2.1.0
	 */
	public void setMedianColor(String medianColor) {
		setMedianColor(new Color(medianColor));
	}

	/**
	 * Sets the color of the median line. If <code>null</code>, the general
	 * series color applies. 'medianColor the color of the median line. If
	 * <code>null</code>, the general series color applies. (Require boxplot)
	 * <p>Default: {@code null}
	 * @since 10.2.1.0
	 */
	public void setMedianColor(Color medianColor) {
		setAttr(DAttrs.medianColor, medianColor);
	}

	/**
	 * Sets the color of the median line. If <code>null</code>, the general
	 * series color applies. 'medianColor the color of the median line. If
	 * <code>null</code>, the general series color applies. (Require boxplot)
	 * <p>Default: {@code null}
	 * @since 10.2.1.0
	 */
	public void setMedianColor(LinearGradient medianColor) {
		setMedianColor(new Color(medianColor));
	}

	/**
	 * Sets the color of the median line. If <code>null</code>, the general
	 * series color applies. 'medianColor the color of the median line. If
	 * <code>null</code>, the general series color applies. (Require boxplot)
	 * <p>Default: {@code null}
	 * @since 10.2.1.0
	 */
	public void setMedianColor(RadialGradient medianColor) {
		setMedianColor(new Color(medianColor));
	}

	/**
	 * Returns a name for the dash style of the median. (Requires boxplot)
	 * The value for the <code>medianDashStyle</code> include:
	 * <ul>
	 * <li>Solid</li>
	 * <li>ShortDash</li>
	 * <li>ShortDot</li>
	 * <li>ShortDashDot</li>
	 * <li>ShortDashDotDot</li>
	 * <li>Dot</li>
	 * <li>Dash</li>
	 * <li>LongDash</li>
	 * <li>DashDot</li>
	 * <li>LongDashDot</li>
	 * <li>LongDashDotDot</li>
	 * </ul>
	 * <p>Default: {@code "Solid"}
	 * @since 10.2.1.0
	 */
	public String getMedianDashStyle() {
		return getAttr(DAttrs.medianDashStyle, "Solid").asString();
	}

	/**
	 * Sets a name for the dash style of the median. (Requires boxplot)
	 * The value for the <code>medianDashStyle</code> include:
	 * <ul>
	 * <li>Solid</li>
	 * <li>ShortDash</li>
	 * <li>ShortDot</li>
	 * <li>ShortDashDot</li>
	 * <li>ShortDashDotDot</li>
	 * <li>Dot</li>
	 * <li>Dash</li>
	 * <li>LongDash</li>
	 * <li>DashDot</li>
	 * <li>LongDashDot</li>
	 * <li>LongDashDotDot</li>
	 * </ul>
	 * @since 10.2.1.0
	 */
	public void setMedianDashStyle(String medianDashStyle) {
		if (!("Solid".equalsIgnoreCase(medianDashStyle)
				|| "ShortDash".equalsIgnoreCase(medianDashStyle)
				|| "ShortDot".equalsIgnoreCase(medianDashStyle)
				|| "ShortDashDot".equalsIgnoreCase(medianDashStyle)
				|| "ShortDashDotDot".equalsIgnoreCase(medianDashStyle)
				|| "Dot".equalsIgnoreCase(medianDashStyle)
				|| "Dash".equalsIgnoreCase(medianDashStyle)
				|| "LongDash".equalsIgnoreCase(medianDashStyle)
				|| "DashDot".equalsIgnoreCase(medianDashStyle)
				|| "LongDashDot".equalsIgnoreCase(medianDashStyle) || "LongDashDotDot"
					.equalsIgnoreCase(medianDashStyle)))
			throw new IllegalArgumentException("Unsupported style: ["
					+ medianDashStyle + "]");
		setAttr(DAttrs.medianDashStyle, medianDashStyle);
	}

	/**
	 * Returns the pixel width of the median line. If <code>null</code>, the
	 * {@link #setLineWidth(Number)} is used. (Requires boxplot)
	 * <p>
	 * Default: 2
	 * @since 10.2.1.0
	 */
	public Number getMedianWidth() {
		return getAttr(DAttrs.medianWidth, 2).asNumber();
	}

	/**
	 * Sets the pixel width of the median line. If <code>null</code>, the
	 * {@link #setLineWidth(Number)} is used. (Requires boxplot)
	 * <p>
	 * Default: 2
	 * @since 10.2.1.0
	 */
	public void setMedianWidth(Number medianWidth) {
		setAttr(DAttrs.medianWidth, medianWidth, 2);
	}

	/**
	 * Returns a threshold determining the minimum font size that can be applied to a word.
	 * (Requires wordcloud)
	 * <p>
	 * Default: 1
	 * @since 10.2.1.0
	 */
	public Number getMinFontSize() {
		return getAttr(DAttrs.minFontSize, 1).asNumber();
	}

	/**
	 * Sets a threshold determining the minimum font size that can be applied to a word.
	 * (Requires wordcloud)
	 * @since 10.2.1.0
	 */
	public void setMinFontSize(Number minFontSize) {
		setAttr(DAttrs.minFontSize, minFontSize);
	}

	/**
	 * Returns the minimal width for a line of a sankey. By default, 0 values are not shown.
	 * (Requires arcdiagram or sankey)
	 * <p>
	 * Default: 0
	 * @since 10.2.1.0
	 */
	public Number getMinLinkWidth() {
		return getAttr(DAttrs.minLinkWidth, 0).asNumber();
	}

	/**
	 * Sets the minimal width for a line of a sankey. By default, 0 values are not shown.
	 * (Requires arcdiagram or sankey)
	 * @since 10.2.1.0
	 */
	public void setMinLinkWidth(Number minLinkWidth) {
		setAttr(DAttrs.minLinkWidth, minLinkWidth);
	}

	/**
	 * In a horizontal chart, the minimum width of the hanging nodes only, in pixels.
	 * In a vertical chart, the minimum height of the hanging nodes only, in pixels too.
	 * (Requires organization)
	 * <p>
	 * Note: Used only when {@link #getHangingIndentTranslation()} is set to {@code "shrink").
	 * <p>
	 * Defaults to {@code 10}.
	 * @since 10.2.1.0
	 */
	public Number getMinNodeLength() {
		return getAttr(DAttrs.minNodeLength, 10).asNumber();
	}

	/**
	 * In a horizontal chart, the minimum width of the hanging nodes only, in pixels.
	 * In a vertical chart, the minimum height of the hanging nodes only, in pixels too.
	 * (Requires organization)
	 * <p>
	 * Note: Used only when {@link #getHangingIndentTranslation()} is set to {@code "shrink").
	 * <p>
	 * Defaults to {@code 10}.
	 * @since 10.2.1.0
	 */
	public void setMinNodeLength(Number minNodeLength) {
		setAttr(DAttrs.minNodeLength, minNodeLength);
	}

	/**
	 * Returns the minimal height for a column or width for a bar. By default, 0
	 * values are not shown. To visualize a 0 (or close to zero) point, set the
	 * minimal point length to a pixel value like 3. In stacked column charts,
	 * minPointLength might not be respected for tightly packed values.
	 * <p>Default: 0
	 * @since 10.2.1.0
	 */
	public Number getMinPointLength() {
		return getAttr(DAttrs.minPointLength, 0).asNumber();
	}

	/**
	 * Sets the minimal height for a column or width for a bar. By default, 0
	 * values are not shown. To visualize a 0 (or close to zero) point, set the
	 * minimal point length to a pixel value like 3. In stacked column charts,
	 * minPointLength might not be respected for tightly packed values.
	 * @since 10.2.1.0
	 */
	public void setMinPointLength(Number minPointLength) {
		setAttr(DAttrs.minPointLength, minPointLength, 0);
	}

	/**
	 * Returns the minimum size of the points' radius related to chart's plotArea.
	 * If a number is set, it applies in pixels. (Requires variablepie)
	 * <p>
	 * Default: {@code "10%"}
	 * @since 10.2.1.0
	 */
	public Object getMinPointSize() {
		return getAttr(DAttrs.minPointSize, "10%").asValue();
	}

	/**
	 * Sets the minimum size of the points' radius related to chart's plotArea.
	 * If a number is set, it applies in pixels. (Requires variablepie)
	 * @since 10.2.1.0
	 */
	public void setMinPointSize(Number minPointSize) {
		setAttr(DAttrs.minPointSize, minPointSize);
	}

	/**
	 * Returns the minimum size.
	 * @since 7.2.1.0
	 */
	public Object getMinSize() {
		return getAttr(DAttrs.minSize);
	}

	/**
	 * Sets the minimum size.
	 * @since 7.2.1.0
	 */
	public void setMinSize(Number minSize) {
		setAttr(DAttrs.minSize, minSize);
	}

	/**
	 * Sets the minimum size.
	 * @since 7.2.1.0
	 */
	public void setMinSize(String minSize) {
		setAttr(DAttrs.minSize, minSize);
	}

	/**
	 * Sets the name of the series as shown in the legend, tooltip etc.
	 */
	public void setName(String name) {
		setAttr(DAttrs.name, name);
	}

	/**
	 * Returns the height of the neck, the lower part of the funnel. A number
	 * defines pixel width, a percentage string defines a percentage of the plot
	 * area height. (Requires funnel)
	 * @since 10.2.1.0
	 */
	public String getNeckHeight() {
		return getAttr(DAttrs.neckHeight, null).asString();
	}

	/**
	 * Sets the height of the neck, the lower part of the funnel. A number
	 * defines pixel width, a percentage string defines a percentage of the plot
	 * area height. (Requires funnel)
	 * @since 10.2.1.0
	 */
	public void setNeckHeight(String neckHeight) {
		setAttr(DAttrs.neckHeight, neckHeight);
	}

	/**
	 * Returns the width of the neck, the lower part of the funnel. A number
	 * defines pixel width, a percentage string defines a percentage of the plot
	 * area width. (Requires funnel)
	 * @since 10.2.1.0
	 */
	public String getNeckWidth() {
		return getAttr(DAttrs.neckWidth, null).asString();
	}

	/**
	 * Sets the width of the neck, the lower part of the funnel. A number
	 * defines pixel width, a percentage string defines a percentage of the plot
	 * area width. (Requires funnel)
	 * @since 10.2.1.0
	 */
	public void setNeckWidth(String neckWidth) {
		setAttr(DAttrs.neckWidth, neckWidth);
	}


	/**
	 * Returns the color for the parts of the graph or points that are
	 * below the {@link #setThreshold(Number)}.
	 * <p> Defaults to null.
	 * @since 10.2.1.0
	 */
	public Color getNegativeColor() {
		return (Color) getAttr(DAttrs.negativeColor, null).asValue();
	}

	/**
	 * Sets the color for the parts of the graph or points that are
	 * below the {@link #setThreshold(Number)}.
	 * <p> Defaults to null.
	 * @since 10.2.1.0
	 */
	public void setNegativeColor(Color color) {
		setAttr(DAttrs.negativeColor, color);
	}

	/**
	 * Sets the color for the parts of the graph or points that are
	 * below the {@link #setThreshold(Number)}.
	 * <p> Defaults to null.
	 * @since 10.2.1.0
	 */
	public void setNegativeColor(String color) {
		setNegativeColor(new Color(color));
	}

	/**
	 * Sets the color for the parts of the graph or points that are
	 * below the {@link #setThreshold(Number)}.
	 * <p> Defaults to null.
	 * @since 10.2.1.0
	 */
	public void setNegativeColor(LinearGradient color) {
		setNegativeColor(new Color(color));
	}

	/**
	 * Sets the color for the parts of the graph or points that are
	 * below the {@link #setThreshold(Number)}.
	 * <p> Defaults to null.
	 * @since 10.2.1.0
	 */
	public void setNegativeColor(RadialGradient color) {
		setNegativeColor(new Color(color));
	}

	/**
	 * Returns a separate color for the negative part of the area.
	 * <p>
	 * Default: null.
	 * @since 10.2.1.0
	 */
	public Color getNegativeFillColor() {
		return (Color) getAttr(DAttrs.negativeFillColor, null).asValue();
	}

	/**
	 * Sets a separate color for the negative part of the area.
	 * <p>
	 * Default: null.
	 * @since 10.2.1.0
	 */
	public void setNegativeFillColor(Color color) {
		setAttr(DAttrs.negativeFillColor, color);
	}

	/**
	 * Sets a separate color for the negative part of the area.
	 * <p>
	 * Default: null.
	 * @since 10.2.1.0
	 */
	public void setNegativeFillColor(String color) {
		setNegativeFillColor(new Color(color));
	}

	/**
	 * Sets a separate color for the negative part of the area.
	 * <p>
	 * Default: null.
	 * @since 10.2.1.0
	 */
	public void setNegativeFillColor(LinearGradient color) {
		setNegativeFillColor(new Color(color));
	}

	/**
	 * Sets a separate color for the negative part of the area.
	 * <p>
	 * Default: null.
	 * @since 10.2.1.0
	 */
	public void setNegativeFillColor(RadialGradient color) {
		setNegativeFillColor(new Color(color));
	}

	/**
	 * Returns the Series on point feature. Only {@code pie} and {@code sunburst}
	 * series are supported at this moment.
	 * <p>Default: an instance of OnPoint
	 * @since 10.2.1.0
	 * @see OnPoint
	 */
	public OnPoint getOnPoint() {
		return (OnPoint) getAttr(DAttrs.onPoint, OnPoint.class).asValue();
	}

	/**
	 * Sets the Series on point feature. Only {@code pie} and {@code sunburst}
	 * series are supported at this moment.
	 * <p>Default: an instance of OnPoint
	 * @since 10.2.1.0
	 * @see OnPoint
	 */
	public void setOnPoint(OnPoint onPoint) {
		setAttr(DAttrs.onPoint, onPoint);
	}

	/**
	 * Returns the pixel width of each node in a sankey diagram or dependency wheel,
	 * or the height in case the chart is inverted. (Requires arcdiagram or sankey)
	 * @since 10.2.1.0
	 */
	public Number getNodeWidth() {
		return getAttr(DAttrs.nodeWidth, null).asNumber();
	}

	/**
	 * Sets the pixel width of each node in a sankey diagram or dependency wheel,
	 * or the height in case the chart is inverted. (Requires arcdiagram or sankey)
	 * @since 10.2.1.0
	 */
	public void setNodeWidth(String nodeWidth) {
		setAttr(DAttrs.nodeWidth, nodeWidth);
	}

	/**
	 * Sets the pixel width of each node in a sankey diagram or dependency wheel,
	 * or the height in case the chart is inverted. (Requires arcdiagram or sankey)
	 * @since 10.2.1.0
	 */
	public void setNodeWidth(Number nodeWidth) {
		setAttr(DAttrs.nodeWidth, nodeWidth, 20);
	}

	/**
	 * Returns the color applied to null points. (Requires heatmap)
	 * <p>
	 * Default: {@code "#f7f7f7"}.
	 * @since 10.2.1.0
	 */
	public Color getNullColor() {
		return (Color) getAttr(DAttrs.nullColor, new Color("#f7f7f7")).asValue();
	}

	/**
	 * Sets the color applied to null points. (Requires heatmap)
	 * @since 10.2.1.0
	 */
	public void setNullColor(Color color) {
		setAttr(DAttrs.nullColor, color, NOT_NULL_VALUE);
	}

	/**
	 * Sets the color applied to null points. (Requires heatmap)
	 * @since 11.2.0.0
	 */
	public void setNullColor(String color) {
		setNullColor(new Color(color));
	}

	/**
	 * Sets the color applied to null points. (Requires heatmap)
	 * @since 11.2.0.0
	 */
	public void setNullColor(LinearGradient color) {
		setNullColor(new Color(color));
	}

	/**
	 * Sets the color applied to null points. (Requires heatmap)
	 * @since 11.2.0.0
	 */
	public void setNullColor(RadialGradient color) {
		setNullColor(new Color(color));
	}

	/**
	 * Returns the offset of an arc diagram nodes column in relation to the {@code plotArea}.
	 * The offset equal to 50% places nodes in the center of a chart. By default
	 * the series is placed so that the biggest node is touching the bottom border
	 * of the {@code plotArea}. (Requires arcdiagram)
	 * <p>Default: {@code "100%"}
	 * @since 10.2.1.0
	 */
	public String getOffset() {
		return getAttr(DAttrs.offset, "100%").asString();
	}

	/**
	 * Sets the offset of an arc diagram nodes column in relation to the {@code plotArea}.
	 * The offset equal to 50% places nodes in the center of a chart. By default
	 * the series is placed so that the biggest node is touching the bottom border
	 * of the {@code plotArea}. (Requires arcdiagram)
	 * <p>Default: {@code "100%"}
	 * @since 10.2.1.0
	 */
	public void setOffset(String offset) {
		setAttr(DAttrs.offset, offset);
	}

	/**
	 * Returns the id of another series in the chart that the wind barbs are projected on.
	 * When null, the wind symbols are drawn on the X axis, but offset up or down by the yOffset setting.
	 * <p>
	 * Default: null
	 * @since 7.2.1.0
	 */
	public String getOnSeries() {
		return getAttr(DAttrs.onSeries, null).asString();
	}

	/**
	 * Sets the id of another series in the chart that the wind barbs are projected on.
	 * When null, the wind symbols are drawn on the X axis, but offset up or down by the yOffset setting.
	 * @since 7.2.1.0
	 */
	public void setOnSeries(String onSeries) {
		setAttr(DAttrs.onSeries, onSeries);
	}

	/**
	 * Returns opacity of a series parts: line, fill (e.g. area) and dataLabels.
	 * <p>
	 * Default: 1
	 * @since 10.2.1.0
	 */
	public Number getOpacity() {
		return getAttr(DAttrs.opacity, 1).asNumber();
	}

	/**
	 * Sets opacity of a series parts: line, fill (e.g. area) and dataLabels.
	 * @since 10.2.1.0
	 */
	public void setOpacity(Number opacity) {
		setAttr(DAttrs.opacity, opacity);
	}

	/**
	 * Returns the amount of degrees that are allowed to overshoot (Requires gauge)
	 * Default: {@code null}
	 */
	public Number getOvershoot() {
		return getAttr(DAttrs.overshoot, null).asNumber();
	}
	
	/**
	 * Sets the amount of degrees that are allowed to overshoot (Requires gauge)
	 */
	public void setOvershoot(Number overshoot) {
		setAttr(DAttrs.overshoot, overshoot);
	}

	/**
	 * Returns series options for parent nodes.
	 * <p>Default: an instance of {@link PackedBubbleParentNode}
	 * @since 10.2.1.0
	 */
	public PackedBubbleParentNode getParentNode() {
		return (PackedBubbleParentNode) getAttr(DAttrs.parentNode, PackedBubbleParentNode.class).asValue();
	}

	/**
	 * Sets series options for parent nodes.
	 * @since 10.2.1.0
	 */
	public void setParentNode(PackedBubbleParentNode parentNode) {
		setAttr(DAttrs.parentNode, parentNode);
	}

	/**
	 * Returns a partial fill for each point, if any. Otherwise, create a new one.
	 * Typically used to visualize how much of a task is performed.
	 * The partial fill object can be set either on series or point level.
	 * @since 10.2.1.0
	 */
	public XRangePartialFill getPartialFill() {
		XRangePartialFill partialFill = (XRangePartialFill) getAttr(DAttrs.partialFill);
		if (partialFill == null) {
			partialFill = new XRangePartialFill();
			setPartialFill(partialFill);
		}
		return partialFill;
	}

	/**
	 * Sets a partial fill for each point.
	 * Typically used to visualize how much of a task is performed.
	 * The partial fill object can be set either on series or point level.
	 * @since 10.2.1.0
	 */
	public void setPartialFill(XRangePartialFill partialFill) {
		setAttr(DAttrs.partialFill, partialFill);
	}

	/**
	 * Sets a partial fill with an amount for each point.
	 * Typically used to visualize how much of a task is performed.
	 * The partial fill object can be set either on series or point level.
	 * @since 10.2.1.0
	 */
	public void setPartialFill(Number amount) {
		setPartialFill(new XRangePartialFill(amount));
	}

	/**
	 * Returns options for the pivot or the center point of the gauge.
	 * <p>
	 * Default: an instance of {@link GaugePivotPlotOptions}.
	 * @since 10.2.1.0
	 */
	public GaugePivotPlotOptions getPivot() {
		GaugePivotPlotOptions pivot = (GaugePivotPlotOptions) getAttr(DAttrs.pivot);
		if (pivot == null) {
			pivot = new GaugePivotPlotOptions();
			setPivot(pivot);
		}
		return pivot;
	}

	/**
	 * Sets options for the pivot or the center point of the gauge.
	 * @since 10.2.1.0
	 */
	public void setPivot(GaugePivotPlotOptions pivot) {
		setAttr(DAttrs.pivot, pivot);
	}

	/**
	 * Returns which algorithm is used for placement, and rotation of a word.
	 * The choice of algorithm is therefore a crucial part of the resulting layout of the wordcloud.
	 * It is possible for users to add their own custom placement strategies for use in word cloud.
	 * Read more about it in <a href="https://www.highcharts.com/docs/chart-and-series-types/word-cloud-series#custom-placement-strategies">documentation</a>.
	 * <p>
	 * Default: {@code "center"}
	 * @since 10.2.1.0
	 */
	public String getPlacementStrategy() {
		return getAttr(DAttrs.placementStrategy, "center").asString();
	}

	/**
	 * Sets which algorithm is used for placement, and rotation of a word.
	 * The choice of algorithm is therefore a crucial part of the resulting layout of the wordcloud.
	 * It is possible for users to add their own custom placement strategies for use in word cloud.
	 * Read more about it in <a href="https://www.highcharts.com/docs/chart-and-series-types/word-cloud-series#custom-placement-strategies">documentation</a>.
	 * @since 10.2.1.0
	 */
	public void setPlacementStrategy(String placementStrategy) {
		setAttr(DAttrs.placementStrategy, placementStrategy);
	}

	/**
	 * Returns the formatter function to use instead of the default for series
	 * descriptions. Receives one argument, series, referring to the series to describe.
	 * Should return a string with the description of the series for a screen reader user.
	 * If false is returned, the default formatter will be used for that series.
	 * <p> Default: null.
	 * @since 10.2.1.0
	 */
	public JavaScriptValue getPointDescriptionFormatter() {
		return (JavaScriptValue) getAttr(DAttrs.pointDescriptionFormatter, null).asValue();
	}

	/**
	 * Sets the formatter function to use instead of the default for series
	 * descriptions. Receives one argument, series, referring to the series to describe.
	 * Should return a string with the description of the series for a screen reader user.
	 * If false is returned, the default formatter will be used for that series.
	 * <p> Default: null.
	 * @since 10.2.1.0
	 */
	public void setPointDescriptionFormatter(JavaScriptValue pointDescriptionFormatter) {
		setAttr(DAttrs.pointDescriptionFormatter, pointDescriptionFormatter);
	}

	/**
	 * Returns the point interval of the series.
	 * Default: null
	 */
	public Number getPointInterval() {
		return getAttr(DAttrs.pointInterval, null).asNumber();
	}
	
	/**
	 * Sets the point interval of the series.
	 */
	public void setPointInterval(Number pointInterval) {
		setAttr(DAttrs.pointInterval, pointInterval);
	}
	
	/**
	 * On datetime series, this allows for setting the pointInterval to the two
	 * irregular time units, month and year. Combine it with pointInterval to
	 * draw quarters, 6 months, 10 years etc.
	 * <p>
	 * Default: null
	 * @since 2.1.0
	 */
	public String getPointIntervalUnit() {
		return getAttr(DAttrs.pointIntervalUnit, null).asString();
	}
	/**
	 * Set the irregular time unit for point interval, only "day", "month", or "year" is allowed.
	 * @param pointIntervalUnit "day", "month", or "year"
	 * @since 2.1.0
	 */
	public void setPointIntervalUnit(String pointIntervalUnit) {
		if (!("day".equalsIgnoreCase(pointIntervalUnit) || "month".equalsIgnoreCase(pointIntervalUnit)
				|| "year".equalsIgnoreCase(pointIntervalUnit))) {
			throw new IllegalArgumentException("Unsupported point interval unit:[" + pointIntervalUnit + "]");
		}
		setAttr(DAttrs.pointIntervalUnit, pointIntervalUnit);
	}
	
	/**
	 * Returns the padding between each column or bar, in x axis units.
	 * Default: 0.1
	 */
	public Number getPointPadding() {
		return getAttr(DAttrs.pointPadding, 0.1).asNumber();
	}
	
	/**
	 * Sets the padding between each column or bar, in x axis units.
	 */
	public void setPointPadding(Number pointPadding) {
		setAttr(DAttrs.pointPadding, pointPadding, 0.1);
	}
	
	/**
	 * Returns the point placement.
	 * <p>
	 * Possible values: null, "on", "between" or numeric.
	 * </p>
	 * <p>
	 * In a column chart, when pointPlacement is "on", the point will not create
	 * any padding of the X axis. In a polar column chart this means that the
	 * first column points directly north. If the pointPlacement is "between",
	 * the columns will be laid out between ticks. This is useful for example
	 * for visualising an amount between two points in time or in a certain
	 * sector of a polar chart.
	 * </p>
	 * <p>
	 * The point placement can also be numeric, where 0 is on the axis value,
	 * -0.5 is between this value and the previous, and 0.5 is between this
	 * value and the next. Unlike the textual options, numeric point placement
	 * options won't affect axis padding.
	 * </p>
	 * <p>
	 * Defaults to <code>null</code> in cartesian charts, <code>"between"</code>
	 * in polar charts.
	 */
	public Object getPointPlacement() {
		return getAttr(DAttrs.pointPlacement, null).asValue();
	}

	/**
	 * Sets the point placement.
	 * <p>
	 * Possible values: null, "on", "between".
	 * </p>
	 * <p>
	 * In a column chart, when pointPlacement is "on", the point will not create
	 * any padding of the X axis. In a polar column chart this means that the
	 * first column points directly north. If the pointPlacement is "between",
	 * the columns will be laid out between ticks. This is useful for example
	 * for visualising an amount between two points in time or in a certain
	 * sector of a polar chart.
	 * <p>
	 * Defaults to <code>null</code> in cartesian charts, <code>"between"</code>
	 * in polar charts.
	 */
	public void setPointPlacement(String pointPlacement) {
		if (!("on".equals(pointPlacement) || "between".equals(pointPlacement)))
			throw new IllegalArgumentException("Unsupported pointPlacement: ["
					+ pointPlacement + "]");
		setAttr(DAttrs.pointPlacement, pointPlacement);
	}

	/**
	 * Sets the point placement with a number
	 * <p>
	 * The point placement can also be numeric, where 0 is on the axis value,
	 * -0.5 is between this value and the previous, and 0.5 is between this
	 * value and the next. Unlike the textual options, numeric point placement
	 * options won't affect axis padding.
	 * </p>
	 */
	public void setPointPlacement(Number pointPlacement) {
		setAttr(DAttrs.pointPlacement, pointPlacement);
	}

	/**
	 * Returns how many points should be plotted within 1 interval.
	 * See plotOptions.bellcurve.intervals.
	 *
	 * Requires bellcurve.
	 * <p>
	 * Default: 3
	 * @since 7.2.1.0
	 */
	public Number getPointsInInterval() {
		return getAttr(DAttrs.pointsInInterval, 3).asNumber();
	}

	/**
	 * Sets how many points should be plotted within 1 interval.
	 * See plotOptions.bellcurve.intervals.
	 *
	 * Requires bellcurve.
	 * @since 7.2.1.0
	 */
	public void setPointsInInterval(Number pointsInInterval) {
		setAttr(DAttrs.pointsInInterval, pointsInInterval, 3);
	}

	/**
	 * If no x values are given for the points in a series, pointStart defines
	 * on what value to start. For example, if a series contains one yearly
	 * value starting from 1945, set pointStart to 1945.
	 *
	 * Requires bellcurve.
	 * <p>
	 * Default: 0
	 * @since 7.2.1.0
	 */
	public Number getPointStart() {
		return getAttr(DAttrs.pointStart, 0).asNumber();
	}

	/**
	 * If no x values are given for the points in a series, pointStart defines
	 * on what value to start. For example, if a series contains one yearly
	 * value starting from 1945, set pointStart to 1945.
	 *
	 * Requires bellcurve.
	 * @since 7.2.1.0
	 */
	public void setPointStart(Number pointStart) {
		setAttr(DAttrs.pointStart, pointStart, 0);
	}

	/**
	 * Default: null
	 * @since 7.2.1.0
	 */
	public Number getPointRange() {
		return getAttr(DAttrs.pointRange, null).asNumber();
	}

	/**
	 * @since 7.2.1.0
	 */
	public void setPointRange(Number pointRange) {
		setAttr(DAttrs.pointRange, pointRange);
	}

	/**
	 * Returns a pixel value specifying a fixed width for each column or bar.
	 * When <code>null</code>, the width is calculated from the
	 * <code>pointPadding</code> and <code>groupPadding</code>.
	 * <p>Default: null
	 * @since 10.2.1.0
	 */
	public Number getPointWidth() {
		return getAttr(DAttrs.pointWidth, null).asNumber();
	}

	/**
	 * Sets a pixel value specifying a fixed width for each column or bar. When
	 * <code>null</code>, the width is calculated from the
	 * <code>pointPadding</code> and <code>groupPadding</code>.
	 * @since 10.2.1.0
	 */
	public void setPointWidth(Number pointWidth) {
		setAttr(DAttrs.pointWidth, pointWidth);
	}

	/**
	 * Returns the outer radius for points in a solid gauge.
	 * Can be given as a number (pixels) or percentage string.
	 * <p>
	 * Default: {@code 100}
	 * @since 10.2.1.0
	 */
	public Object getRadius() {
		return getAttr(DAttrs.radius, 100).asValue();
	}

	/**
	 * Sets the outer radius for points in a solid gauge.
	 * Can be given as a number (pixels).
	 * @since 10.2.1.0
	 */
	public void setRadius(Number radius) {
		setAttr(DAttrs.radius, radius);
	}

	/**
	 * Sets the outer radius for points in a solid gauge.
	 * Can be given as a percentage string.
	 * @since 10.2.1.0
	 */
	public void setRadius(String radius) {
		setAttr(DAttrs.radius, radius);
	}


	/**
	 * When true, X values in the data set are relative to the current {@code pointStart},
	 * {@code pointInterval} and {@code pointIntervalUnit} settings. This allows
	 * compression of the data for datasets with irregular X values.
	 * <p>
	 * The real X values are computed on the formula {@code f(x) = ax + b}, where
	 * {@code a} is the {@code pointInterval} (optionally with a time unit given
	 * by {@code pointIntervalUnit}), and {@code b} is the {@code pointStart}.
	 * <p> Defaults to {@code false}.
	 * @since 10.2.1.0
	 */
	public boolean isRelativeXValue() {
		return getAttr(DAttrs.relativeXValue, false).asBoolean();
	}

	/**
	 * When true, X values in the data set are relative to the current {@code pointStart},
	 * {@code pointInterval} and {@code pointIntervalUnit} settings. This allows
	 * compression of the data for datasets with irregular X values.
	 * <p>
	 * The real X values are computed on the formula {@code f(x) = ax + b}, where
	 * {@code a} is the {@code pointInterval} (optionally with a time unit given
	 * by {@code pointIntervalUnit}), and {@code b} is the {@code pointStart}.
	 * <p> Defaults to {@code false}.
	 * @since 10.2.1.0
	 */
	public void setRelativeXValue(boolean relativeXValue) {
		setAttr(DAttrs.relativeXValue, relativeXValue);
	}

	/**
	 * Returns the name of the series as shown in the legend, tooltip etc.
	 */
	public String getName() {
		return getAttr(DAttrs.name, null).asString();
	}

	/**
	 * Returns the padding between nodes in a sankey diagram or dependency wheel, in pixels.
	 * <p>
	 * Default: {@code 10}
	 * @since 10.2.1.0
	 */
	public Number getNodePadding() {
		return getAttr(DAttrs.nodePadding, 10).asNumber();
	}

	/**
	 * Sets the padding between nodes in a sankey diagram or dependency wheel, in pixels.
	 * @since 10.2.1.0
	 */
	public void setNodePadding(Number nodePadding) {
		setAttr(DAttrs.nodePadding, nodePadding);
	}

	/**
	 * Returns the size of nodes.
	 * @since 7.2.1.0
	 */
	public int getNodesSize() {
		List<Series> list = Generics.cast(getAttr(DAttrs.nodes));
		return list == null ? 0 : list.size();
	}

	/**
	 * Returns the Node options, if any. Otherwise, create a new one.
	 * <p>
	 * Returns the first node (index 0) by default.
	 *
	 * @see Node
	 * @see #getNodes(int)
	 * @since 7.2.1.0
	 */
	public Node getNodes() {
		return getNodes(0);
	}

	/**
	 * Returns the Node options from the given index, if any. Otherwise,
	 * create a new one.
	 *
	 * @see Node
	 * @since 7.2.1.0
	 */
	public Node getNodes(int index) {
		List<Node> list = Generics.cast(getAttr(DAttrs.nodes));
		if (list == null) {
			list = new LinkedList<Node>();
			setAttr(DAttrs.nodes, list);
		}

		int size = list.size();
		if (size <= index) {
			for (int i = index - size + 1; i > 0; i--) {
				list.add(new Node());
			}
		}
		return list.get(index);
	}

	/**
	 * Returns whether the series should be placed on the other side of the plotArea.
	 * (Requires arcdiagram or funnel)
	 * <p>
	 * Default: {@code false}.
	 * @since 10.2.1.0
	 */
	public boolean isReversed() {
		return getAttr(DAttrs.reversed, false).asBoolean();
	}

	/**
	 * Sets whether the series should be placed on the other side of the plotArea.
	 * (Requires arcdiagram or funnel)
	 * @since 10.2.1.0
	 */
	public void setReversed(boolean reversed) {
		setAttr(DAttrs.reversed, reversed);
	}

	/**
	 * Returns which point to use as a root in the visualization.
	 * (Requires sunburst)
	 * <p>
	 * Default: null
	 * @since 10.2.1.0
	 */
	public String getRootId() {
		return getAttr(DAttrs.rootId, null).asString();
	}

	/**
	 * Sets which point to use as a root in the visualization.
	 * (Requires sunburst)
	 * @since 10.2.1.0
	 */
	public void setRootId(String rootId) {
		setAttr(DAttrs.rootId, rootId);
	}

	/**
	 * Returns the rotation options for the words in the wordcloud, if any.
	 * Otherwise, create a new one.
	 * @since 10.2.1.0
	 */
	public WordCloudRotation getRotation() {
		WordCloudRotation rotation = (WordCloudRotation) getAttr(DAttrs.rotation);
		if (rotation == null) {
			rotation = new WordCloudRotation();
			setRotation(rotation);
		}
		return rotation;
	}

	/**
	 * Sets the rotation options for the words in the wordcloud.
	 * @since 10.2.1.0
	 */
	public void setRotation(WordCloudRotation rotation) {
		setAttr(DAttrs.rotation, rotation);
	}

	/**
	 * Returns whether to draw rounded edges on the gauge. (Requires solidgauge)
	 * <p>
	 * Default: {@code false}
	 * @since 10.2.1.0
	 */
	public boolean isRounded() {
		return getAttr(DAttrs.rounded, false).asBoolean();
	}

	/**
	 * Sets whether to draw rounded edges on the gauge. (Requires solidgauge)
	 * @since 10.2.1.0
	 */
	public void setRounded(boolean rounded) {
		setAttr(DAttrs.rounded, rounded);
	}

	/**
	 * Returns the number of rows to display in the rectangular or circular view.
	 * If the {@link #getInnerSize()} is set, it will be overridden by the rows setting.
	 * <p>
	 * Default: null
	 * @since 7.2.1.0
	 */
	public Number getRows() {
		return getAttr(DAttrs.rows, null).asNumber();
	}

	/**
	 * Sets the number of rows to display in the rectangular or circular view.
	 * If the {@link #getInnerSize()} is set, it will be overridden by the rows setting.
	 * @since 7.2.1.0
	 */
	public void setRows(Number rows) {
		setAttr(DAttrs.rows, rows);
	}

	/**
	 * Returns the row size - how many Y axis units each heatmap row should span.
	 * <p>
	 * Default: 1.
	 *
	 * @return row size - how many Y axis units each heatmap row should span.
	 * @since 10.2.1.0
	 */
	public Number getRowsize() {
		return getAttr(DAttrs.rowsize, 1).asNumber();
	}

	/**
	 * Sets the row size - how many Y axis units each heatmap row should span.
	 * @since 10.2.1.0
	 */
	public void setRowsize(Number rowsize) {
		setAttr(DAttrs.rowsize, rowsize);
	}

	/**
	 * Returns whether to select the series initially. If
	 * <code>showCheckbox</code> is true, the checkbox next to the series name
	 * will be checked for a selected series.
	 * <p>Default: {@code false}
	 * @since 10.2.1.0
	 */
	public boolean isSelected() {
		return getAttr(DAttrs.selected, false).asBoolean();
	}

	/**
	 * Sets whether to select the series initially. If <code>showCheckbox</code>
	 * is true, the checkbox next to the series name will be checked for a
	 * selected series.
	 * <p>Default: {@code false}
	 * @since 10.2.1.0
	 */
	public void setSelected(boolean selected) {
		setAttr(DAttrs.selected, selected);
	}

	/**
	 * Returns the Shadow object or boolean value of shadow, depend on what was set.
	 * To apply a drop shadow to the graph line. Since 2.3 the
	 * shadow can be an object configuration containing <code>color</code>,
	 * <code>offsetX</code>, <code>offsetY</code>, <code>opacity</code> and
	 * <code>width</code>.
	 * @since 11.2.0.0
	 */
	public Object getShadow() {
		Object shadow = this.getAttr(DAttrs.shadow);
		if (shadow == null) {
			shadow = new Shadow();
			setShadow((Shadow) shadow);
		}
		return shadow;
	}

	/**
	 * Returns whether to apply a drop shadow to the graph line. Since 2.3 the
	 * shadow can be an object configuration containing <code>color</code>,
	 * <code>offsetX</code>, <code>offsetY</code>, <code>opacity</code> and
	 * <code>width</code>.
	 * <p>Default: {@code false}
	 * @since 10.2.1.0
	 *
	 */
	public boolean isShadow() {
		return getAttr(DAttrs.shadow, false).asBoolean();
	}

	/**
	 * Sets whether to apply a drop shadow to the graph line. Since 2.3 the
	 * shadow can be an object configuration containing <code>color</code>,
	 * <code>offsetX</code>, <code>offsetY</code>, <code>opacity</code> and
	 * <code>width</code>.
	 * <p>Default: {@code false}
	 * @since 10.2.1.0
	 */
	public void setShadow(boolean shadow) {
		setAttr(DAttrs.shadow, shadow);
	}

	/**
	 * Sets whether to apply a drop shadow to the graph line. Since 2.3 the
	 * shadow can be an object configuration containing <code>color</code>,
	 * <code>offsetX</code>, <code>offsetY</code>, <code>opacity</code> and
	 * <code>width</code>.
	 * @since 11.2.0.0
	 */
	public void setShadow(Shadow shadow) {
		setAttr(DAttrs.shadow, shadow);
	}

	/**
	 * Returns if true, a checkbox is displayed next to the legend item to allow
	 * selecting the series. The state of the checkbox is determined by the
	 * <code>selected</code> option.
	 * <p>Default: {@code false}
	 * @since 10.2.1.0
	 */
	public boolean isShowCheckbox() {
		return getAttr(DAttrs.showCheckbox, false).asBoolean();
	}

	/**
	 * Sets if true, a checkbox is displayed next to the legend item to allow
	 * selecting the series. The state of the checkbox is determined by the
	 * <code>selected</code> option.
	 * <p>Default: {@code false}
	 * @since 10.2.1.0
	 */
	public void setShowCheckbox(boolean showCheckbox) {
		setAttr(DAttrs.showCheckbox, showCheckbox);
	}

	/**
	 * Returns whether to display this particular series or series type in the legend.
	 * Standalone series are shown in legend by default, and linked series are not.
	 * It is possible to show series that use colorAxis by setting this option to true.
	 * <p>
	 * Default: true
	 * @since 7.2.1.0
	 */
	public boolean isShowInLegend() {
		return getAttr(DAttrs.showInLegend, true).asBoolean();
	}

	/**
	 * Sets whether to display this particular series or series type in the legend.
	 * Standalone series are shown in legend by default, and linked series are not.
	 * It is possible to show series that use colorAxis by setting this option to true.
	 * @since 7.2.1.0
	 */
	public void setShowInLegend(boolean showInLegend) {
		setAttr(DAttrs.showInLegend, showInLegend);
	}

	/**
	 * Returns the diameter of the pie relative to the plot area.
	 * Can be a percentage or pixel value. Pixel values are given as integers.
	 * The default behaviour is to scale to the plot area and give room for data labels
	 * within the plot area. slicedOffset is also included in the default size calculation.
	 * As a consequence, the size of the pie may vary when points are updated and data labels more around.
	 * In that case it is best to set a fixed value, for example "75%".
	 * <p>
	 * Default: null
	 * @since 7.2.1.0
	 */
	public Object getSize() {
		return getAttr(DAttrs.size);
	}

	/**
	 * Sets the diameter of the pie relative to the plot area.
	 * Can be a percentage or pixel value. Pixel values are given as integers.
	 * The default behaviour is to scale to the plot area and give room for data labels
	 * within the plot area. slicedOffset is also included in the default size calculation.
	 * As a consequence, the size of the pie may vary when points are updated and data labels more around.
	 * In that case it is best to set a fixed value, for example "75%".
	 * @since 7.2.1.0
	 */
	public void setSize(Number size) {
		setAttr(DAttrs.size, size);
	}

	/**
	 * Sets the diameter of the pie relative to the plot area.
	 * Can be a percentage or pixel value. Pixel values are given as integers.
	 * The default behaviour is to scale to the plot area and give room for data labels
	 * within the plot area. slicedOffset is also included in the default size calculation.
	 * As a consequence, the size of the pie may vary when points are updated and data labels more around.
	 * In that case it is best to set a fixed value, for example "75%".
	 * @since 7.2.1.0
	 */
	public void setSize(String size) {
		setAttr(DAttrs.size, size);
	}

	/**
	 * Returns what the plot area value should be represented by.
	 * The default, area, corresponds best to the human perception of the size of each area.
	 * <p>
	 * Default: "area"
	 * @since 7.2.1.0
	 */
	public String getSizeBy() {
		return getAttr(DAttrs.sizeBy, "area").asString();
	}

	/**
	 * Sets what the plot area value should be represented by.
	 * @since 7.2.1.0
	 */
	public void setSizeBy(String sizeBy) {
		setAttr(DAttrs.sizeBy, sizeBy);
	}

	/**
	 * When this is true, the absolute value of z determines the size of the bubble.
	 * This means that with the default zThreshold of 0, a bubble of value -1 will
	 * have the same size as a bubble of value 1, while a bubble of value 0 will
	 * have a smaller size according to minSize.
	 * <p>
	 * Defaults to {@code false}.
	 * @since 10.2.1.0
	 */
	public boolean isSizeByAbsoluteValue() {
		return getAttr(DAttrs.sizeByAbsoluteValue, false).asBoolean();
	}

	/**
	 * When this is true, the absolute value of z determines the size of the bubble.
	 * This means that with the default zThreshold of 0, a bubble of value -1 will
	 * have the same size as a bubble of value 1, while a bubble of value 0 will
	 * have a smaller size according to minSize.
	 * <p>
	 * Defaults to {@code false}.
	 * @since 10.2.1.0
	 */
	public void setSizeByAbsoluteValue(boolean sizeByAbsoluteValue) {
		setAttr(DAttrs.sizeByAbsoluteValue, sizeByAbsoluteValue);
	}

	/**
	 * If set to true, the accessibility module will skip past the points in this series for keyboard navigation.
	 * <p>
	 * Default: false
	 * @since 10.2.1.0
	 */
	public boolean isSkipKeyboardNavigation() {
		return getAttr(DAttrs.skipKeyboardNavigation, false).asBoolean();
	}

	/**
	 * If set to true, the accessibility module will skip past the points in this series for keyboard navigation.
	 * @since 10.2.1.0
	 */
	public void setSkipKeyboardNavigation(boolean skipKeyboardNavigation) {
		setAttr(DAttrs.skipKeyboardNavigation, skipKeyboardNavigation);
	}

	/**
	 * Returns if a point is sliced, moved out from the center, how many pixels
	 * should it be moved?
	 * <p> Default: 10
	 * @since 10.2.1.0
	 */
	public Number getSlicedOffset() {
		return getAttr(DAttrs.slicedOffset, 10).asNumber();
	}

	/**
	 * Sets if a point is sliced, moved out from the center, how many pixels
	 * should it be moved?
	 * @since 10.2.1.0
	 */
	public void setSlicedOffset(Number slicedOffset) {
		setAttr(DAttrs.slicedOffset, slicedOffset, 10);
	}

	/**
	 * When this is true, the series will not cause the Y axis to cross the
	 * zero plane (or threshold option) unless the data actually crosses the plane.
	 *
	 * For example, if softThreshold is false, a series of 0, 1, 2, 3 will make
	 * the Y axis show negative values according to the minPadding option.
	 * If softThreshold is true, the Y axis starts at 0.
	 * <p>
	 * Default: true
	 * @since 10.2.1.0
	 */
	public boolean isSoftThreshold() {
		return getAttr(DAttrs.softThreshold, true).asBoolean();
	}

	/**
	 * When this is true, the series will not cause the Y axis to cross the
	 * zero plane (or threshold option) unless the data actually crosses the plane.
	 *
	 * For example, if softThreshold is false, a series of 0, 1, 2, 3 will make
	 * the Y axis show negative values according to the minPadding option.
	 * If softThreshold is true, the Y axis starts at 0.
	 * @since 10.2.1.0
	 */
	public void setSoftThreshold(boolean softThreshold) {
		setAttr(DAttrs.softThreshold, softThreshold, true);
	}

	/**
	 * Returns the Sonification/audio chart options for a series.
	 * @since 11.2.0.0
	 */
	public SeriesSonification getSonification() {
		SeriesSonification sonification = (SeriesSonification) getAttr(DAttrs.sonification);
		if (sonification == null) {
			sonification = new SeriesSonification();
			setSonification(sonification);
		}
		return sonification;
	}

	/**
	 * Sets the Sonification/audio chart options for a series.
	 * @since 11.2.0.0
	 */
	public void setSonification(SeriesSonification sonification) {
		setAttr(DAttrs.sonification, sonification);
	}

	/**
	 * Returns the sort index of the point inside the treemap level.
	 * <p>Default: {@code null}
	 * @since 10.2.1.0
	 */
	public Number getSortIndex() {
		return getAttr(DAttrs.sortIndex, null).asNumber();
	}

	/**
	 * Sets the sort index of the point inside the treemap level.
	 * <p>Default: {@code null}
	 * @since 10.2.1.0
	 */
	public void setSortIndex(Number sortIndex) {
		setAttr(DAttrs.sortIndex, sortIndex);
	}

	/**
	 * Returns the spiral used for placing a word after the initial position
	 * experienced a collision with either another word or the borders.
	 * It is possible for users to add their own custom spiralling algorithms
	 * for use in word cloud. Read more about it in <a href="https://www.highcharts.com/docs/chart-and-series-types/word-cloud-series#custom-spiralling-algorithm">documentation</a>.
	 * <p>
	 * Default: {@code "rectangular"}
	 * @since 10.2.1.0
	 */
	public String getSpiral() {
		return getAttr(DAttrs.spiral, "rectangular").asString();
	}

	/**
	 * Sets the spiral used for placing a word after the initial position
	 * experienced a collision with either another word or the borders.
	 * It is possible for users to add their own custom spiralling algorithms
	 * for use in word cloud. Read more about it in <a href="https://www.highcharts.com/docs/chart-and-series-types/word-cloud-series#custom-spiralling-algorithm">documentation</a>.
	 * @since 10.2.1.0
	 */
	public void setSpiral(String spiral) {
		setAttr(DAttrs.spiral, spiral);
	}

	/**
	 * Sets the option allows grouping series in a stacked chart. The stack
	 * option can be a string or a number or anything else, as long as the
	 * grouped series' stack options match each other.
	 */
	public void setStack(String stack) {
		setAttr(DAttrs.stack, stack);
	}
	
	/**
	 * Sets the option allows grouping series in a stacked chart. The stack
	 * option can be a string or a number or anything else, as long as the
	 * grouped series' stack options match each other.
	 */
	public void setStack(Number stack) {
		setAttr(DAttrs.stack, stack);
	}

	/**
	 * Returns the option allows grouping series in a stacked chart. The stack
	 * option can be a string or a number or anything else, as long as the
	 * grouped series' stack options match each other.
	 */
	public Object getStack() {
		return getAttr(DAttrs.stack, null).asValue();
	}

	/**
	 * Returns whether to stack the values of each series on top of each other.
	 * Possible values are null to disable, "normal" to stack by value or
	 * "percent".
	 * @since 10.2.1.0
	 */
	public String getStacking() {
		return getAttr(DAttrs.stacking, null).asString();
	}

	/**
	 * Returns whether to stack the values of each series on top of each other or not.
	 *
	 * @since 10.2.1.0
	 */
	public boolean isStacking() {
		return getAttr(DAttrs.stacking, null).asBoolean();
	}

	/**
	 * Sets whether to stack the values of each series on top of each other.
	 * Possible values are null to disable, "normal" to stack by value or
	 * "percent".
	 * @since 10.2.1.0
	 */
	public void setStacking(String stacking) {
		if (stacking != null
				&& !("normal".equals(stacking) || "percent".equals(stacking)))
			throw new IllegalArgumentException("Unsupported value: ["
					+ stacking + "]");
		setAttr(DAttrs.stacking, stacking);
	}

	/**
	 * Sets whether to stack the values of each series on top of each other.
	 * Possible values are null to disable, "normal" to stack by value or
	 * "percent".
	 * @since 10.2.1.0
	 */
	public void setStacking(Boolean stacking) {
		setAttr(DAttrs.stacking, stacking);
	}

	/**
	 * In circular view, returns the start angle of the item layout, in degrees where 0 is up.
	 * Default: null
	 * @since 7.2.1.0
	 */
	public Number getStartAngle() {
		return getAttr(DAttrs.startAngle, null).asNumber();
	}

	/**
	 * In circular view, sets the start angle of the item layout, in degrees where 0 is up.
	 * @since 7.2.1.0
	 */
	public void setStartAngle(Number startAngle) {
		setAttr(DAttrs.startAngle, startAngle);
	}

	/**
	 * Returns a wrapper object for all the series options in specific states.
	 * <p>Default: an instance of {@link States}
	 * @since 10.2.1.0
	 */
	public States getStates() {
		States states = (States) getAttr(DAttrs.states);
		if (states == null) {
			states = new States();
			setStates(states);
		}
		return states;
	}

	/**
	 * Sets a wrapper object for all the series options in specific states.
	 * @since 10.2.1.0
	 */
	public void setStates(States states) {
		setAttr(DAttrs.states, states);
	}

	/**
	 * Returns the dash style of the stem, the vertical line extending from the
	 * box to the whiskers. (Requires boxplot)
	 * @since 10.2.1.0
	 */
	public String getStemDashStyle() {
		return getAttr(DAttrs.stemDashStyle, "Solid").asString();
	}

	/**
	 * Sets the dash style of the stem, the vertical line extending from the box
	 * to the whiskers. (Requires boxplot)
	 * @since 10.2.1.0
	 */
	public void setStemDashStyle(String stemDashStyle) {
		if (!("Solid".equalsIgnoreCase(stemDashStyle)
				|| "ShortDash".equalsIgnoreCase(stemDashStyle)
				|| "ShortDot".equalsIgnoreCase(stemDashStyle)
				|| "ShortDashDot".equalsIgnoreCase(stemDashStyle)
				|| "ShortDashDotDot".equalsIgnoreCase(stemDashStyle)
				|| "Dot".equalsIgnoreCase(stemDashStyle)
				|| "Dash".equalsIgnoreCase(stemDashStyle)
				|| "LongDash".equalsIgnoreCase(stemDashStyle)
				|| "DashDot".equalsIgnoreCase(stemDashStyle)
				|| "LongDashDot".equalsIgnoreCase(stemDashStyle) || "LongDashDotDot"
					.equalsIgnoreCase(stemDashStyle)))
			throw new IllegalArgumentException("Unsupported style: ["
					+ stemDashStyle + "]");
		setAttr(DAttrs.stemDashStyle, stemDashStyle, "Solid");
	}

	/**
	 * Returns the width of the stem, the vertical line extending from the box
	 * to the whiskers. If <code>null</code>, the width is inherited from the
	 * {@link #setLineWidth(Number)} option. (Requires boxplot)
	 * @since 10.2.1.0
	 */
	public Number getStemWidth() {
		return getAttr(DAttrs.stemWidth, null).asNumber();
	}

	/**
	 * Sets the width of the stem, the vertical line extending from the box to
	 * the whiskers. If <code>null</code>, the width is inherited from the
	 * {@link #setLineWidth(Number)} option. (Requires boxplot)
	 * @since 10.2.1.0
	 */
	public void setStemWidth(Number stemWidth) {
		setAttr(DAttrs.stemWidth, stemWidth);
	}

	/**
	 * Returns whether to apply steps to the line. Possible values are
	 * <code>left</code>, <code>center</code> and <code>right</code>.
	 * <p>
	 * Default: null
	 * @since 10.2.1.0
	 */
	public String getStep() {
		return getAttr(DAttrs.step, null).asString();
	}

	/**
	 * Sets whether to apply steps to the line. Possible values are
	 * <code>left</code>, <code>center</code> and <code>right</code>.
	 * @since 10.2.1.0
	 */
	public void setStep(String step) {
		if (!("left".equals(step) || "center".equals(step) || "right".equals(step)))
			throw new IllegalArgumentException("Unsupported step: [" + step + "]");
		setAttr(DAttrs.step, step) ;
	}

	/**
	 * Returns sticky tracking of mouse events. When true, the
	 * <code>mouseOut</code> event on a series isn't triggered until the mouse
	 * moves over another series, or out of the plot area. When false, the
	 * <code>mouseOut</code> event on a series is triggered when the mouse
	 * leaves the area around the series' graph or markers. This also implies
	 * the tooltip. When <code>stickyTracking</code> is false and
	 * {@link Tooltip#isShared()} is false, the tooltip will be hidden when
	 * moving the mouse between series. Defaults to true for line and area type
	 * series, but to false for columns, pies etc.
	 * <p>Default: {@code false}
	 * @since 10.2.1.0
	 */
	public boolean isStickyTracking() {
		return getAttr(DAttrs.stickyTracking, false).asBoolean();
	}

	/**
	 * Sets sticky tracking of mouse events. When true, the
	 * <code>mouseOut</code> event on a series isn't triggered until the mouse
	 * moves over another series, or out of the plot area. When false, the
	 * <code>mouseOut</code> event on a series is triggered when the mouse
	 * leaves the area around the series' graph or markers. This also implies
	 * the tooltip. When <code>stickyTracking</code> is false and
	 * {@link Tooltip#isShared()} is false, the tooltip will be hidden when
	 * moving the mouse between series. Defaults to true for line and area type
	 * series, but to false for columns, pies etc.
	 * @since 10.2.1.0
	 */
	public void setStickyTracking(boolean stickyTracking) {
		setAttr(DAttrs.stickyTracking, stickyTracking);
	}
	/**
	 * Returns CSS styles for the words. (Requires wordcloud)
	 * <p>
	 * Default: "fontFamily: 'sans-serif'; fontWeight: 900"
	 * @param <K> type for the key
	 * <p>
	 * if style was set using {@link #setStyle(String)}, type will be String
	 * <p>
	 * if style was set using {@link #setStyle(Map)}, type will be the same as the type from the key of the map
	 * <p>
	 * if style was not set, default style will be applied, and type will be String
	 * @param <V> type for the value
	 * <p>
	 * if style was set using {@link #setStyle(String)}, type will be String
	 * <p>
	 * if style was set using {@link #setStyle(Map)}, type will be the same as the type from the key of the map
	 * <p>
	 * if style was not set, default style will be applied, and type will be String
	 * @since 10.2.1.0
	 */
	public <K, V> Map<K, V> getStyle() {
		if (!containsKey(DAttrs.style)) {
			setStyle("fontFamily: 'sans-serif'; fontWeight: 900");
		}
		return Generics.cast(getAttr(DAttrs.style));
	}

	/**
	 * Sets CSS styles for the words. (Requires wordcloud)
	 * <p>
	 * This method allows <b>simple</b> settings like
	 * <pre>{@code
	 * label.setStyle("fontWeight: 'bold'; color: 'white'; textShadow: '0px 1px 2px black'");
	 * }</pre>
	 * <p>
	 * In the format of CSS key:value, and ";" separating multiple key:value pairs,
	 * if you have more a complex CSS styling, we recommend using {@link #setStyle(Map)}
	 * @param style one or many CSS key:value pairs in a single line string format
	 * @since 10.2.1.0
	 */
	public void setStyle(String style) {
		setStyle(
				MapsHelper.parse(new LinkedHashMap<String, String>(), style, ':', ';', '\''));
	}

	/**
	 * Sets CSS styles for the words. (Requires wordcloud)
	 * <p>
	 * This method allows <b>complex</b> styling like:
	 * <pre>
	 * {@code
	 * Map<String, Object> styles = new HashMap<String, Object>();
	 * styles.put("fill", "#E8E8EA");
	 * styles.put("stroke", "#6E6E78");
	 * styles.put("stroke-width", "1");
	 * styles.put("r", "0");
	 * Map<String, Object> states = new HashMap<String, Object>();
	 * Map<String, String> hover = new HashMap<String, String>();
	 * hover.put("fill", "#F4F4F6");
	 * hover.put("stroke", "#6E6E78");
	 * hover.put("stroke-width", "1");
	 * hover.put("r", "0");
	 * states.put("hover", hover);
	 * styles.put("states", states);
	 * Map<String, String> style = new HashMap<String, String>();
	 * style.put("color", "#4D5663");
	 * style.put("fontSize", "12px");
	 * style.put("fontFamily", "'Arial'");
	 * style.put("fontWeight", "normal");
	 * style.put("padding", "10px");
	 * styles.put("style", style);
	 * }</pre>
	 * which will generate the final results like:
	 * <pre>{@code
	 * {
	 *     "fill": "#E8E8EA",
	 *     "stroke": "#6E6E78",
	 *     "stroke-width": "1",
	 *     "r": "0",
	 *     "states":
	 *     {
	 *         "hover":
	 *         {
	 *             "fill": "#F4F4F6",
	 *             "stroke": "#6E6E78",
	 *             "stroke-width": "1",
	 *             "r": "0"
	 *         }
	 *     },
	 *     "style":
	 *     {
	 *         "color": "#4D5663",
	 *         "fontSize": "12px",
	 *         "fontFamily": "'Arial'",
	 *         "fontWeight": "normal",
	 *         "padding": "10px"
	 *     }
	 * }
	 * }</pre>
	 * @param <K> Type for the key, usually String
	 * @param <V> Type for the value, usually String. For nested maps, use Object or any other type.
	 * @param style the style object
	 * @since 10.2.1.0
	 */
	public <K, V> void setStyle(Map<K, V> style) {
		setAttr(DAttrs.style, style, NOT_NULL_VALUE);
	}

	/**
	 * Returns all options related with look and position of targets, if any.
	 * Otherwise, create a new one.
	 * @since 7.2.1.0
	 */
	public TargetOptions getTargetOptions() {
		TargetOptions targetOptions = (TargetOptions) getAttr(DAttrs.targetOptions);
		if (targetOptions == null) {
			targetOptions = new TargetOptions();
			setTargetOptions(targetOptions);
		}
		return targetOptions;
	}

	/**
	 * Sets all options related with look and position of targets.
	 * @since 7.2.1.0
	 */
	public void setTargetOptions(TargetOptions targetOptions) {
		setAttr(DAttrs.targetOptions, targetOptions);
	}

	/**
	 * Returns thickness describing the ring size for a donut type chart,
	 * overriding {@link #getInnerSize()}. (Requires pie or item)
	 * <p>Default: {@code null}
	 * @since 10.2.1.0
	 */
	public Number getThickness() {
		return getAttr(DAttrs.thickness, null).asNumber();
	}

	/**
	 * Sets thickness describing the ring size for a donut type chart,
	 * overriding {@link #getInnerSize()}. (Requires pie or item)
	 * <p>Default: {@code null}
	 * @since 10.2.1.0
	 */
	public void setThickness(Number thickness) {
		setAttr(DAttrs.thickness, thickness);
	}

	/**
	 * Returns the threshold, also called zero level or base level. For line type
	 * series this is only used in conjunction with {@code negativeColor}.
	 * <p>Default: {@code 0}
	 * @since 10.2.1.0
	 */
	public Number getThreshold() {
		return getAttr(DAttrs.threshold, 0).asNumber();
	}

	/**
	 * Sets the threshold, also called zero level or base level. For line type
	 * series this is only used in conjunction with {@code negativeColor}.
	 * <p>Default: {@code 0}
	 * @since 10.2.1.0
	 */
	public void setThreshold(Number threshold) {
		setAttr(DAttrs.threshold, threshold, 0);
	}

	/**
	 * Returns the shape of the tiles in the tilemap.
	 * Possible values are {@code hexagon}, {@code circle}, {@code diamond}, and {@code square}.
	 * <p>
	 * Default: {@code "hexagon"}
	 * @since 10.2.1.0
	 */
	public String getTileShape() {
		return getAttr(DAttrs.tileShape, "hexagon").asString();
	}

	/**
	 * Sets the shape of the tiles in the tilemap.
	 * Possible values are {@code hexagon}, {@code circle}, {@code diamond}, and {@code square}.
	 * @since 10.2.1.0
	 */
	public void setTileShape(String tileShape) {
		if (!("hexagon".equals(tileShape) || "circle".equals(tileShape)
				|| "diamond".equals(tileShape) || "square".equals(tileShape)))
			throw new IllegalArgumentException("Unsupported tileShape: [" + tileShape + "]");
		setAttr(DAttrs.tileShape, tileShape, "hexagon");
	}

	/**
	 * Returns the tooltip options.
	 */
	public Tooltip getTooltip() {
		Tooltip tooltip = (Tooltip) getAttr(DAttrs.tooltip);
		if (tooltip == null) {
			tooltip = new Tooltip();
			setTooltip(tooltip);
		}
		return tooltip;
	}
	
	/**
	 * Sets the tooltip options.
	 */
	public void setTooltip(Tooltip tooltip) {
		setAttr(DAttrs.tooltip, tooltip);
	}

	/**
	 * Returns whether the whole area or just the line should respond to
	 * mouseover tooltips and other mouse or touch events.
	 * <p>
	 * Default: false.
	 * @since 10.2.1.0
	 */
	public boolean getTrackByArea() {
		return getAttr(DAttrs.trackByArea, false).asBoolean();
	}

	/**
	 * Sets whether the whole area or just the line should respond to mouseover
	 * tooltips and other mouse or touch events.
	 * @since 10.2.1.0
	 */
	public void setTrackByArea(boolean trackByArea) {
		setAttr(DAttrs.trackByArea, trackByArea);
	}

	/**
	 * Returns options for the button appearing when traversing down in a treemap, if any.
	 * Otherwise, create a new one. (Requires treemap or sunburst)
	 * @since 10.2.1.0
	 */
	public TraverseUpButton getTraverseUpButton() {
		TraverseUpButton traverseUpButton = (TraverseUpButton) getAttr(DAttrs.traverseUpButton);
		if (traverseUpButton == null) {
			traverseUpButton = new TraverseUpButton();
			setTraverseUpButton(traverseUpButton);
		}
		return traverseUpButton;
	}

	/**
	 * Sets options for the button appearing when traversing down in a treemap.
	 * (Requires treemap or sunburst)
	 * @since 10.2.1.0
	 */
	public void setTraverseUpButton(TraverseUpButton traverseUpButton) {
		setAttr(DAttrs.traverseUpButton, traverseUpButton);
	}

	/**
	 * Returns when a series contains a data array that is longer than this,
	 * only one dimensional arrays of numbers, or two dimensional arrays with x
	 * and y values are allowed. Also, only the first point is tested, and the
	 * rest are assumed to be the same format. This saves expensive data
	 * checking and indexing in long series. Set it to <code>0</code> disable.
	 * @since 10.2.1.0
	 */
	public Number getTurboThreshold() {
		return getAttr(DAttrs.turboThreshold, 1000).asNumber();
	}

	/**
	 * Sets when a series contains a data array that is longer than this, only
	 * one dimensional arrays of numbers, or two dimensional arrays with x and y
	 * values are allowed. Also, only the first point is tested, and the rest
	 * are assumed to be the same format. This saves expensive data checking and
	 * indexing in long series. Set it to <code>0</code> disable.
	 * @since 10.2.1.0
	 */
	public void setTurboThreshold(Number turboThreshold) {
		setAttr(DAttrs.turboThreshold, turboThreshold, NOT_NULL_VALUE);
	}
	
	/**
	 * The type of series. Can be one of area, areaspline, bar, column, line,
	 * pie, scatter, spline, arearange, areasplinerange and columnrange.
	 */
	public void setType(String type) {
		setAttr(DAttrs.type, type);
	}

	/**
	 * The type of series. Can be one of area, areaspline, bar, column, line,
	 * pie, scatter, spline, arearange, areasplinerange and columnrange.
	 */
	public String getType() {
		return getAttr(DAttrs.type, null).asString();
	}

	/**
	 * Returns the color of up point in a waterfall series.
	 * series.
	 * <p>
	 * Default: null
	 * @since 10.2.1.0
	 */
	public Color getUpColor() {
		return (Color) getAttr(DAttrs.upColor);
	}

	/**
	 * Sets the color of up point in a waterfall series.
	 * @since 10.2.1.0
	 */
	public void setUpColor(Color color) {
		setAttr(DAttrs.upColor, color, NOT_NULL_VALUE);
	}

	/**
	 * Returns an option is giving a possibility to choose between using simulation
	 * for calculating bubble positions.
	 * These reflects in both animation and final position of bubbles.
	 * Simulation is also adding options to the series graph based on used layout.
	 * In case of big data sets, with any performance issues, it is possible to
	 * disable animation and pack bubble in a simple circular way.
	 * <p>
	 * Default: {@code true}
	 * @since 10.2.1.0
	 */
	public boolean isUseSimulation() {
		return getAttr(DAttrs.useSimulation, true).asBoolean();
	}

	/**
	 * Sets an option is giving a possibility to choose between using simulation
	 * for calculating bubble positions.
	 * These reflects in both animation and final position of bubbles.
	 * Simulation is also adding options to the series graph based on used layout.
	 * In case of big data sets, with any performance issues, it is possible to
	 * disable animation and pack bubble in a simple circular way.
	 * @since 10.2.1.0
	 */
	public void setUseSimulation(boolean useSimulation) {
		setAttr(DAttrs.useSimulation, useSimulation);
	}

	/**
	 * Returns the pixel length of the stems. (Requires windbarb)
	 * <p>
	 * Default: {@code 20}
	 * @since 10.2.1.0
	 */
	public Number getVectorLength() {
		return getAttr(DAttrs.vectorLength, 20).asNumber();
	}

	/**
	 * Sets the pixel length of the stems. (Requires windbarb)
	 * @since 10.2.1.0
	 */
	public void setVectorLength(Number vectorLength) {
		setAttr(DAttrs.vectorLength, vectorLength, NOT_NULL_VALUE);
	}

	/**
	 * Returns the color of the whiskers, the horizontal lines marking low and
	 * high values. When <code>null</code>, the general series color is used.
	 * (Requires boxplot)
	 * <p>
	 * Default: null
	 * @since 10.2.1.0
	 */
	public Color getWhiskerColor() {
		return (Color) getAttr(DAttrs.whiskerColor, null).asValue();
	}

	/**
	 * Sets the color of the whiskers, the horizontal lines marking low and high
	 * values. When <code>null</code>, the general series color is used. (Requires boxplot)
	 * @since 10.2.1.0
	 */
	public void setWhiskerColor(Color color) {
		setAttr(DAttrs.whiskerColor, color);
	}

	/**
	 * Sets the color of the whiskers, the horizontal lines marking low and high
	 * values. When <code>null</code>, the general series color is used. (Requires boxplot)
	 * @since 10.2.1.0
	 */
	public void setWhiskerColor(String color) {
		setWhiskerColor(new Color(color));
	}

	/**
	 * Sets the color of the whiskers, the horizontal lines marking low and high
	 * values. When <code>null</code>, the general series color is used. (Requires boxplot)
	 * @since 10.2.1.0
	 */
	public void setWhiskerColor(LinearGradient color) {
		setWhiskerColor(new Color(color));
	}

	/**
	 * Sets the color of the whiskers, the horizontal lines marking low and high
	 * values. When <code>null</code>, the general series color is used. (Requires boxplot)
	 * @since 10.2.1.0
	 */
	public void setWhiskerColor(RadialGradient color) {
		setWhiskerColor(new Color(color));
	}

	/**
	 * Returns the dash style of the whisker.
	 *
	 * <p>Note: requires boxplot or errorbar type
	 * <p>Defaults to Solid.
	 * @since 10.2.1.
	 */
	public String getWhiskerDashStyle() {
		return getAttr(DAttrs.whiskerDashStyle, "Solid").asString();
	}

	/**
	 * Sets the dash style of the whisker.
	 *
	 * <p>Note: requires boxplot or errorbar type
	 * <p>Defaults to Solid.
	 * @since 10.2.1.
	 */
	public void setWhiskerDashStyle(String whiskerDashStyle) {
		if (!("Solid".equalsIgnoreCase(whiskerDashStyle)
				|| "ShortDash".equalsIgnoreCase(whiskerDashStyle)
				|| "ShortDot".equalsIgnoreCase(whiskerDashStyle)
				|| "ShortDashDot".equalsIgnoreCase(whiskerDashStyle)
				|| "ShortDashDotDot".equalsIgnoreCase(whiskerDashStyle)
				|| "Dot".equalsIgnoreCase(whiskerDashStyle)
				|| "Dash".equalsIgnoreCase(whiskerDashStyle)
				|| "LongDash".equalsIgnoreCase(whiskerDashStyle)
				|| "DashDot".equalsIgnoreCase(whiskerDashStyle)
				|| "LongDashDot".equalsIgnoreCase(whiskerDashStyle) || "LongDashDotDot"
				.equalsIgnoreCase(whiskerDashStyle)))
			throw new IllegalArgumentException("Unsupported style: ["
					+ whiskerDashStyle + "]");
		setAttr(DAttrs.whiskerDashStyle, whiskerDashStyle, "Solid");
	}

	/**
	 * Returns the length of the whiskers, the horizontal lines marking low and
	 * high values. It can be a numerical pixel value, or a percentage value of
	 * the box width. Set <code>0</code> to disable whiskers. (Requires boxplot)
	 * <p>
	 * Default: {@code "50%"}
	 * @since 10.2.1.0
	 */
	public String getWhiskerLength() {
		return getAttr(DAttrs.whiskerLength, "50%").asString();
	}

	/**
	 * Sets the length of the whiskers, the horizontal lines marking low and
	 * high values. It can be a numerical pixel value, or a percentage value of
	 * the box width. Set <code>0</code> to disable whiskers. (Requires boxplot)
	 * @since 10.2.1.0
	 */
	public void setWhiskerLength(String whiskerLength) {
		setAttr(DAttrs.whiskerLength, whiskerLength, NOT_NULL_VALUE);
	}

	/**
	 * Sets the length of the whiskers, the horizontal lines marking low and
	 * high values. It can be a numerical pixel value, or a percentage value of
	 * the box width. Set <code>0</code> to disable whiskers. (Requires boxplot)
	 * @since 10.2.1.0
	 */
	public void setWhiskerLength(Number whiskerLength) {
		setAttr(DAttrs.whiskerLength, whiskerLength);
	}

	/**
	 * Returns the line width of the whiskers, the horizontal lines marking low
	 * and high values. When <code>null</code>, the general
	 * {@link #setLineWidth(Number)} applies. (Requires boxplot)
	 * <p>Default: {@code 2}
	 * @since 10.2.1.0
	 */
	public Number getWhiskerWidth() {
		return getAttr(DAttrs.whiskerWidth, 2).asNumber();
	}

	/**
	 * Sets the line width of the whiskers, the horizontal lines marking low and
	 * high values. When <code>null</code>, the general
	 * {@link #setLineWidth(Number)} applies. (Requires boxplot)
	 * @since 10.2.1.0
	 */
	public void setWhiskerWidth(Number whiskerWidth) {
		setAttr(DAttrs.whiskerWidth, whiskerWidth, NOT_NULL_VALUE);
	}

	/**
	 * Returns the width of the funnel compared to the width of the plot area,
	 * or the pixel width if it is a number. (Requires funnel)
	 * <p> Default: {@code "90%"}
	 * @since 10.2.1.0
	 */
	public String getWidth() {
		return getAttr(DAttrs.width, "90%").asString();
	}

	/**
	 * Sets the width of the funnel compared to the width of the plot area, or
	 * the pixel width if it is a number. (Requires funnel)
	 * @since 10.2.1.0
	 */
	public void setWidth(String width) {
		setAttr(DAttrs.width, width, NOT_NULL_VALUE);
	}

	/**
	 * Returns when this option is <code>true</code>, the dial will wrap around
	 * the axes. For instance, in a full-range gauge going from 0 to 360, a
	 * value of 400 will point to 40. When <code>wrap</code> is
	 * <code>false</code>, the dial stops at 360. (Requires gauge)
	 * <p>
	 * Default: true.
	 * @since 10.2.1.0
	 */
	public boolean isWrap() {
		return getAttr(DAttrs.wrap, true).asBoolean();
	}

	/**
	 * Sets when this option is <code>true</code>, the dial will wrap around the
	 * axes. For instance, in a full-range gauge going from 0 to 360, a value of
	 * 400 will point to 40. When <code>wrap</code> is <code>false</code>, the
	 * dial stops at 360. (Requires gauge)
	 * @since 10.2.1.0
	 */
	public void setWrap(boolean wrap) {
		setAttr(DAttrs.wrap, wrap);
	}



	/**
	 * When using dual or multiple x axes, this number defines which xAxis the
	 * particular series is connected to. It refers to either the axis id or the
	 * index of the axis in the xAxis array, with 0 being the first. Defaults to
	 * 0.
	 */
	public void setXAxis(int xAxis) {
		setAttr(DAttrs.xAxis, xAxis);
	}

	/**
	 * When using dual or multiple x axes, this number defines which xAxis the
	 * particular series is connected to. It refers to either the axis id or the
	 * index of the axis in the xAxis array, with 0 being the first. Defaults to
	 * 0.
	 */
	public void setXAxis(String xAxis) {
		setAttr(DAttrs.xAxis, xAxis);
	}

	/**
	 * When using dual or multiple x axes, this number defines which xAxis the
	 * particular series is connected to. It refers to either the axis id or the
	 * index of the axis in the xAxis array, with 0 being the first. Defaults to
	 * 0.
	 */
	public Object getXAxis() {
		return getAttr(DAttrs.xAxis, 0).asValue();
	}

	/**
	 * When using dual or multiple y axes, this number defines which yAxis the
	 * particular series is connected to. It refers to either the axis id or the
	 * index of the axis in the yAxis array, with 0 being the first. Defaults to
	 * 0.
	 */
	public void setYAxis(int yAxis) {
		setAttr(DAttrs.yAxis, yAxis);
	}

	/**
	 * When using dual or multiple y axes, this number defines which yAxis the
	 * particular series is connected to. It refers to either the axis id or the
	 * index of the axis in the yAxis array, with 0 being the first. Defaults to
	 * 0.
	 */
	public void setYAxis(String yAxis) {
		setAttr(DAttrs.yAxis, yAxis);
	}

	/**
	 * When using dual or multiple y axes, this number defines which yAxis the
	 * particular series is connected to. It refers to either the axis id or the
	 * index of the axis in the yAxis array, with 0 being the first. Defaults to
	 * 0.
	 */
	public Object getYAxis() {
		return getAttr(DAttrs.yAxis, 0).asValue();
	}

	/**
	 * Returns horizontal offset from the cartesian position, in pixels.
	 * When the chart is inverted, this option allows translation like yOffset in non inverted charts.
	 * (Requires windbarb)
	 * <p>
	 * Default: {@code 0}
	 * @since 10.2.1.0
	 *
	 */
	public Number getXOffset() {
		return getAttr(DAttrs.xOffset, 0).asNumber();
	}

	/**
	 * Sets horizontal offset from the cartesian position, in pixels.
	 * When the chart is inverted, this option allows translation like yOffset in non inverted charts.
	 * (Requires windbarb)
	 * @since 10.2.1.0
	 */
	public void setXOffset(Number xOffset) {
		setAttr(DAttrs.xOffset, xOffset, NOT_NULL_VALUE);
	}

	/**
	 * Returns vertical offset from the cartesian position, in pixels.
	 * The default value makes sure the symbols don't overlap the X axis when onSeries is null,
	 * and that they don't overlap the linked series when onSeries is given.
	 * (Requires windbarb)
	 * <p>
	 * Default: {@code -20}
	 * @since 10.2.1.0
	 */
	public Number getYOffset() {
		return getAttr(DAttrs.yOffset, -20).asNumber();
	}

	/**
	 * Sets vertical offset from the cartesian position, in pixels.
	 * The default value makes sure the symbols don't overlap the X axis when onSeries is null,
	 * and that they don't overlap the linked series when onSeries is given.
	 * (Requires windbarb)
	 * @since 10.2.1.0
	 */
	public void setYOffset(Number yOffset) {
		setAttr(DAttrs.yOffset, yOffset, NOT_NULL_VALUE);
	}

	/**
	 * Define the visual z index of the series.
	 */
	public void setZIndex(int zIndex) {
		setAttr(DAttrs.zIndex, zIndex);
	}

	/**
	 * Define the visual z index of the series.
	 */
	public int getZIndex() {
		return getAttr(DAttrs.zIndex, -1).asInt();
	}

	/**
	 * Returns the maximum for the Z value range. Defaults to the highest Z value in the data.
	 * (Requires bubble and variablepie)
	 * <p>
	 * Default: null
	 * @since 10.2.1.0
	 */
	public Number getZMax() {
		return getAttr(DAttrs.zMax, null).asNumber();
	}

	/**
	 * Sets the maximum for the Z value range. (Requires bubble and variablepie)
	 * @since 10.2.1.0
	 */
	public void setZMax(Number zMax) {
		setAttr(DAttrs.zMax, zMax);
	}
	/**
	 * Returns the minimum for the Z value range. Defaults to the lowest Z value in the data.
	 * (Requires bubble and variablepie)
	 * <p>
	 * Default: {@code null}
	 * @since 10.2.1.0
	 */
	public Number getZMin() {
		return getAttr(DAttrs.zMin, null).asNumber();
	}

	/**
	 * Sets the minimum for the Z value range. (Requires bubble and variablepie)
	 * @since 10.2.1.0
	 */
	public void setZMin(Number zMin) {
		setAttr(DAttrs.zMin, zMin);
	}

	/**
	 * Defines the Axis on which the zones are applied.
	 * @return the zone axis or the default value "y"
	 * @since 2.1.0
	 */
	public String getZoneAxis() {
		return getAttr(DAttrs.zoneAxis, "y").asString();
	}
	/**
	 * Set the axis on which the zones are applied. Only X, Y, or Z axis are supported.
	 * @param zoneAxis either "x", "y", or "z"
	 * @since 2.1.0
	 */
	public void setZoneAxis(String zoneAxis) {
		if (!("x".equalsIgnoreCase(zoneAxis) || "y".equalsIgnoreCase(zoneAxis) || "z".equalsIgnoreCase(zoneAxis))) {
			throw new IllegalArgumentException("Unsupported zone axis: [" + zoneAxis + "]");
		}
		setAttr(DAttrs.zoneAxis, zoneAxis, "y");
	}
	
	/**
	 * An list defining zones within a series. Zones can be applied to the X
	 * axis, Y axis or Z axis for bubbles, according to the zoneAxis option.
	 * @since 2.1.0
	 * @see Zone
	 */
	public List<Zone> getZones() {
		return Generics.cast(getAttr(DAttrs.zones, null).asValue());
	}
	/**
	 * Set the zones within a series.
	 * @since 2.1.0
	 * @see #getZones()
	 * @see Zone
	 */
	public void setZones(List<Zone> zones) {
		setAttr(DAttrs.zones, zones);
	}

	/**
	 * Returns an array specifying which option maps to which key in the data point array.
	 * This makes it convenient to work with unstructured data arrays from different sources.
	 * <p> Defaults: null.
	 * @since 10.2.1.0
	 */
	public List<String> getKeys() {
		return Generics.cast(getAttr(DAttrs.keys, null));
	}

	/**
	 * Returns an array specifying which option maps to which key in the data point array.
	 * This makes it convenient to work with unstructured data arrays from different sources.
	 * <p> Defaults: null.
	 * @since 10.2.1.0
	 */
	public void setKeys(List<String> keys) {
		setAttr(DAttrs.keys, keys);
	}

	/**
	 * Returns an array specifying which option maps to which key in the data point array.
	 * This makes it convenient to work with unstructured data arrays from different sources.
	 * <p> Defaults: null.
	 * @since 10.2.1.0
	 */
	public void setKeys(String... keys) {
		setKeys(Arrays.asList(keys));
	}


	/**
	 * Returns series label options, if any. Otherwise, create a new one.
	 * Series labels are placed as close to the series as possible in a natural way,
	 * seeking to avoid other series.
	 * The goal of this feature is to make the chart more easily readable,
	 * like if a human designer placed the labels in the optimal position.
	 *
	 * The series labels currently work with series types having a graph or an area.
	 * @since 10.2.1.0
	 */
	public SeriesLabel getLabel() {
		SeriesLabel label = (SeriesLabel) getAttr(DAttrs.label);
		if (label == null) {
			label = new SeriesLabel();
			setLabel(label);
		}
		return label;
	}

	/**
	 * Sets series label options.
	 * Series labels are placed as close to the series as possible in a natural way,
	 * seeking to avoid other series.
	 * The goal of this feature is to make the chart more easily readable,
	 * like if a human designer placed the labels in the optimal position.
	 *
	 * The series labels currently work with series types having a graph or an area.
	 * @since 10.2.1.0
	 */
	public void setLabel(SeriesLabel label) {
		setAttr(DAttrs.label, label);
	}

	/**
	 * Returns when {@link #setDisplayNegative(boolean)} is <code>false</code>,
	 * bubbles with lower Z values are skipped. When
	 * <code>displayNegative</code> is <code>true</code> and a
	 * {@link #setNegativeColor(String)} is given, points with lower Z is
	 * colored.
	 * <p>
	 * Default: 0
	 * @since 10.2.1.0
	 */
	public Number getZThreshold() {
		return getAttr(DAttrs.zThreshold, 0).asNumber();
	}

	/**
	 * Sets when {@link #setDisplayNegative(boolean)} is <code>false</code>,
	 * bubbles with lower Z values are skipped. When
	 * <code>displayNegative</code> is <code>true</code> and a
	 * {@link #setNegativeColor(String)} is given, points with lower Z is
	 * colored.
	 * @since 10.2.1.0
	 */
	public void setZThreshold(Number zThreshold) {
		setAttr(DAttrs.zThreshold, zThreshold, 0);
	}

	/**
	 * Hides the series if visible. If the {@link Chart#isIgnoreHiddenSeries()}
	 * option is true,the chart is redrawn without this series.
	 */
	public void hide() {
		setVisible(false);
	}

	/**
	 * Returns whether the series is visible or not.
	 * <p>
	 * Default: true
	 */
	public boolean isVisible() {
		return getAttr(Attrs.visible, true).asBoolean();
	}

	/**
	 * Sets whether the series is visible.
	 */
	public void setVisible(final boolean visible) {
		if (setVisibleDirectly(visible)) {
			fireEvent(EventType.CHANGED, "visible", this, new DeferredCall() {
				public void execute(JSFunction func) {
					// B-ZKCHARTS-72: set redraw false then call chart.redraw()
					func.callFunction("setVisible", visible, false);
				}
			});
		}
	}

	/**
	 * Sets whether the series is visible directly.
	 */
	public boolean setVisibleDirectly(final boolean visible) {
		return setAttr(Attrs.visible, visible, true);
	}

	/**
	 * Shows the series if hidden.
	 */
	public void show() {
		setVisible(true);
	}

	public void remove() {
		fireEvent(EventType.DESTROYED, "series", this, new DeferredCall() {
			public void execute(JSFunction func) {
				func.callFunction("remove");
			}
		});
		clearOptonDataListener();
	}

	public void select() {
		fireEvent(EventType.CHANGED, "select", this, new DeferredCall() {
			public void execute(JSFunction func) {
				func.callFunction("select");
			}

		});
	}

	public void onChange(OptionDataEvent event) {
		final Optionable target = event.getTarget();
		if (target instanceof Point) {
			final int index = getData().indexOf(target);
			if (event.hasJSFunctionCall()) {
				event.addJSFunctionCall(new DeferredCall() {
					public void execute(JSFunction func) {
						func.callArray("data", index);
					}

				});
			}
			if (event.getType() == OptionDataEvent.EventType.DESTROYED) {
				List<Point> list = getData();
				if (list != null)
					list.remove(index);
				event.setCurrentType(EventType.REMOVED); // ZKCHARTS-56 REMOVED for series
			}
		} else {
			event.addJSFunctionCall(new DeferredCall() {
				public void execute(JSFunction func) {
					if (target instanceof DataLabels) {
						JSONObject json = new JSONObject();
						json.put("dataLabels", target);
						func.callFunction("update", json);
					} else if (target instanceof DragDrop) {
						JSONObject json = new JSONObject();
						json.put("dragDrop", target);
						func.callFunction("update", json);
					} else {
						func.callFunction("update", target);
					}
				}
			});
		}
		super.onChange(event);
	}

	public String toJSONString() {
		if (plotOptions != null) {
			// we don't use merge(options) here
			Map<String, AnyVal<Object>> clone = new LinkedHashMap<String, AnyVal<Object>>(
					options);
			clone.putAll(plotOptions.options);
			return JSONObject.toJSONString(clone);
		}
		return super.toJSONString();
	}
}
